package demo.barcharts.category ;

import java.awt.Color ;
import java.awt.GradientPaint ;
import java.awt.GridLayout ;
import java.awt.event.ActionEvent ;
import java.awt.event.ActionListener ;

import javax.swing.JPanel ;
import javax.swing.Timer ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.render.category.Bar2DRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class BarChartDemo10 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.VERTICAL ;

  public BarChartDemo10() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel() ;
  }

  private void addPanel() {
    final DefaultMatrixDataset dataset = createDataset() ;
    final IJChart chart = ChartFactory.createCategoryBar2DChart(
      "Bar Chart Demo 10", "Category", "Value", dataset, orientation, false, true) ;

    CategoryPlot plot = (CategoryPlot) chart.getPlot() ;
    plot.setValueGridlinePaint(Color.white) ;
    plot.getRender().setBackgroundPaint(Color.lightGray) ;

    NumberValueAxis numberaxis = (NumberValueAxis) plot.getValueAxis() ;
    numberaxis.setTickUnit(new LongTickUnit()) ;

    Bar2DRender render = (Bar2DRender) plot.getRender() ;
    GradientPaint gradientpaint = new GradientPaint(0.0F, 0.0F, Color.blue, 0.0F, 0.0F,
                                                    new Color(0, 0, 64)) ;
    GradientPaint gradientpaint1 = new GradientPaint(0.0F, 0.0F, Color.green, 0.0F, 0.0F,
                                                     new Color(0, 64, 0)) ;
    GradientPaint gradientpaint2 = new GradientPaint(0.0F, 0.0F, Color.red, 0.0F, 0.0F,
                                                     new Color(64, 0, 0)) ;
    render.setColumnItemFillPaint(0, gradientpaint) ;
    render.setColumnItemFillPaint(1, gradientpaint1) ;
    render.setColumnItemFillPaint(2, gradientpaint2) ;
    CategoryAxis categoryaxis = plot.getCategoryAxis() ;
    categoryaxis.setTickLabelAngle( -0.52359877559829882D) ;

    Animator animator = new Animator(dataset) ;
    animator.start() ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static DefaultMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addColKey("First") ;
    dataset.addColKey("Second") ;
    dataset.addColKey("Third") ;

    for (int i = 1 ; i <= 5 ; i++) {
      dataset.addRowKey("Category " + i) ;
    }

    double data[][] = { {31D, 44D, 33D, 45D, 35D}, {45D, 37D, 46D, 38D, 44D}, {34D, 43D, 32D, 43D,
                      36D}
    } ;
    dataset.addRowData(5) ;

    for (int i = 0 ; i < 5 ; i++) {
      for (int j = 0 ; j < data.length ; j++) {
        dataset.addData(i, new NumberChartData(data[j][i])) ;
      }
    }

    return dataset ;
  }

  class Animator extends Timer implements ActionListener {
    private DefaultMatrixDataset dataset ;

    Animator(DefaultMatrixDataset defaultcategorydataset) {
      super(20, null) ;
      dataset = defaultcategorydataset ;
      addActionListener(this) ;
    }

    public void actionPerformed(ActionEvent actionevent) {
      int i = (int) (Math.random() * dataset.getRowCount()) ;
      int j = (int) (Math.random() * dataset.getColumnCount()) ;
      int k = (Math.random() - 0.5D >= 0.0D) ? 5 : -5 ;
      double v = Math.max(0.0D, ((NumberChartData) dataset.getData(i, j)).getValue() + (double) k) ;
      dataset.setData(i, j, new NumberChartData(v)) ;
    }
  }
}
