package demo.barcharts.category ;

import java.awt.Font ;
import java.awt.GridLayout ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.render.category.Bar2DRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;
import com.ijchart.xychart.title.TextTitle ;
import com.ijchart.xychart.tooltip.DefaultTooltipGenerator ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class BarChartDemo5 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.HORIZONTAL ;

  public BarChartDemo5() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel() ;
  }

  private void addPanel() {
    final IMatrixDataset dataset = createDataset() ;
    final IJChart chart = ChartFactory.createCategoryBar2DChart(
      "Prison Population Rates - Selected Countries", "Country",
      "Prisoners Per 100,000 National Population", dataset, orientation, false, true) ;

    chart.addSubtitle(new TextTitle("Source: http://www.homeoffice.gov.uk/rds/pdfs2/r188.pdf",
                                    new Font("Dialog", 2, 12))) ;

    CategoryPlot plot = (CategoryPlot) chart.getPlot() ;
    Bar2DRender render = (Bar2DRender) plot.getRender() ;
    render.setBaseItemLabelPosition(TextPosition.OUTSIDE_RIGHT) ;
    render.setBaseItemLabelVisible(true) ;
    render.setToolTipGenerator(new DefaultTooltipGenerator() {
      public String generateToolTip(IMatrixDataset dataset, int row, int column) {
        return "(" + dataset.getColKey(column) + ", " + dataset.getRowKey(row)
          + ") = " + ((NumberChartData) dataset.getData(row, column)).getValue() + " per 100,000" ;
      }
    }) ;

    CategoryAxis categoryaxis = plot.getCategoryAxis() ;
    categoryaxis.setCategoryMargin(0.25D) ;
    categoryaxis.setUpperMargin(0.02D) ;
    categoryaxis.setLowerMargin(0.02D) ;

    NumberValueAxis numberaxis = (NumberValueAxis) plot.getValueAxis() ;
    numberaxis.setTickUnit(new LongTickUnit()) ;
    numberaxis.setUpperMargin(0.10000000000000001D) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    String[] rowKeys = {"Norway", "Switzerland", "France", "Syria", "Germany", "China", "Australia",
                       "Egypt", "England&Wales", "NewZealand", "Chile", "Iran", "Singapore",
                       "SouthAfrica", "Ukraine", "USA"} ;
    for (int i = 0 ; i < rowKeys.length ; i++) {
      dataset.addRowKey(rowKeys[i]) ;
    }
    dataset.addColKey("Prison Population Rates") ;

    double data[] = {59D, 69D, 85D, 93D, 96D, 111D, 116D, 121D, 129D, 157D, 205D, 229D, 359D, 404D,
                    406D, 686D} ;
    dataset.addRowData(data.length) ;

    for (int i = 0 ; i < data.length ; i++) {
      dataset.addData(i, new NumberChartData(data[i])) ;
    }

    return dataset ;
  }

}
