package demo.financialcharts ;

import java.awt.GridLayout ;
import java.text.DecimalFormat ;
import java.text.SimpleDateFormat ;
import java.util.Calendar ;
import java.util.Date ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.IntervalXYValueAxis ;
import com.ijchart.xychart.axis.value.OHLCDateAxis ;
import com.ijchart.xychart.axis.value.OHLCValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.constant.XYValueAxisType ;
import com.ijchart.xychart.data.IntervalXYNumberChartData ;
import com.ijchart.xychart.data.OHLCChartData ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.dataset.SingleRowDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.HighLowRender ;
import com.ijchart.xychart.render.xy.XYClusteredBar2DRender ;
import com.ijchart.xychart.tooltip.DefaultTooltipGenerator ;
/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class PriceVolumeDemo2 extends JPanel {

  private ChartOrientation vertical = ChartOrientation.VERTICAL ;

  /** A working calendar. */
  private static final Calendar CALENDAR = Calendar.getInstance() ;

  private SimpleDateFormat dateFormat = new SimpleDateFormat("dd-MMM-yyyy") ;

  private DecimalFormat numFormat = new DecimalFormat("0.00") ;

  private DecimalFormat numFormat2 = new DecimalFormat("0,000.00") ;

  public PriceVolumeDemo2() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel1() ;
  }

  private void addPanel1() {
    SingleRowDataset ohlcdataset = createPriceDataset() ;
    IJChart chart = ChartFactory.createHighLowChart("Sun Microsystems (SUNW)", "Date", "Price",
                                                    ohlcdataset, vertical, true, true) ;
    XYPlot xyplot = (XYPlot) chart.getPlot() ;
    OHLCDateAxis dateaxis = (OHLCDateAxis) xyplot.getDomainAxis() ;
    dateaxis.setLowerMargin(0.01D) ;
    dateaxis.setUpperMargin(0.01D) ;
    OHLCValueAxis numberaxis = (OHLCValueAxis) xyplot.getValueAxis() ;
    numberaxis.setAxisRangeIncludeZero(false) ;
    HighLowRender render1 = (HighLowRender) xyplot.getRender() ;
    render1.setToolTipGenerator(new DefaultTooltipGenerator() {
      public String generateToolTip(IMatrixDataset dataset, int row, int column) {
        Object rowKey = dataset.getRowKey(row) ;
        OHLCChartData data = (OHLCChartData) dataset.getData(row, column) ;
        return rowKey + ":(" + dateFormat.format(data.getDate()) + ","
          + numFormat.format(data.getVolume()) + ")" ;
      }
    }) ;

    SingleRowDataset volumeDataset = createVolumeDataset() ;
    IntervalXYValueAxis volumeYAxis = new IntervalXYValueAxis("Volume", volumeDataset,
                                                              XYValueAxisType.XY_VALUE_AXIS_Y, false) ;
    volumeYAxis.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    volumeYAxis.setValueLineVisible(true) ;
    xyplot.setValueAxis(1, volumeYAxis) ;

    XYClusteredBar2DRender render2 = new XYClusteredBar2DRender(volumeDataset) ;
    render2.setCenterBarAtStartValue(true) ;
    xyplot.setRender(1, render2) ;
    render2.setToolTipGenerator(new DefaultTooltipGenerator() {
      public String generateToolTip(IMatrixDataset dataset, int row, int column) {
        Object rowKey = dataset.getRowKey(row) ;
        IntervalXYNumberChartData data = (IntervalXYNumberChartData) dataset.getData(row, column) ;
        return rowKey + ":(" + dateFormat.format(data.getStartXData()) + ","
          + numFormat2.format(data.getEndYValue()) + ")" ;
      }
    }) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static SingleRowDataset createVolumeDataset() {
    SingleRowDataset volumeDataset = new SingleRowDataset() ;
    volumeDataset.addRowKey("Volume") ;
    volumeDataset.addRowData() ;

    volumeDataset.addData(0, createIntervalData(createDate(12, 4, 2007), 49646800D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(13, 4, 2007), 67319300D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(16, 4, 2007), 56840200D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(17, 4, 2007), 40752300D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(18, 4, 2007), 42533800D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(19, 4, 2007), 34232100D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(20, 4, 2007), 32437600D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(23, 4, 2007), 28755900D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(24, 4, 2007), 74503300D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(25, 4, 2007), 322214000D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(26, 4, 2007), 96141700D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(27, 4, 2007), 62326200D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(30, 4, 2007), 52334400D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(1, 5, 2007), 133096100D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(2, 5, 2007), 93874500D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(3, 5, 2007), 101011400D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(4, 5, 2007), 56629700D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(7, 5, 2007), 43302200D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(8, 5, 2007), 51456500D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(9, 5, 2007), 48230900D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(10, 5, 2007), 65536000D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(11, 5, 2007), 46469700D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(14, 5, 2007), 118580000D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(15, 5, 2007), 50774100D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(16, 5, 2007), 44590200D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(17, 5, 2007), 125482500D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(18, 5, 2007), 49987500D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(21, 5, 2007), 48387400D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(22, 5, 2007), 67992600D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(23, 5, 2007), 45629200D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(24, 5, 2007), 123288600D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(25, 5, 2007), 47345300D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(29, 5, 2007), 90454000D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(30, 5, 2007), 73049200D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(31, 5, 2007), 60395400D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(1, 6, 2007), 48792300D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(4, 6, 2007), 109972800D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(5, 6, 2007), 198201200D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(6, 6, 2007), 121415700D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(7, 6, 2007), 56637400D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(8, 6, 2007), 64116600D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(11, 6, 2007), 60274800D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(12, 6, 2007), 93451300D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(13, 6, 2007), 103309000D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(14, 6, 2007), 103135400D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(15, 6, 2007), 104415900D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(18, 6, 2007), 51506900D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(19, 6, 2007), 74592100D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(20, 6, 2007), 69027600D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(21, 6, 2007), 97212500D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(22, 6, 2007), 51612000D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(25, 6, 2007), 63845400D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(26, 6, 2007), 84818400D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(27, 6, 2007), 73512900D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(28, 6, 2007), 75275100D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(29, 6, 2007), 104203900D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(2, 7, 2007), 66540400D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(3, 7, 2007), 28520800D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(5, 7, 2007), 47176300D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(6, 7, 2007), 70084800D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(9, 7, 2007), 91630800D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(10, 7, 2007), 114071700D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(11, 7, 2007), 34217900D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(12, 7, 2007), 30298000D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(13, 7, 2007), 40423500D)) ;
    volumeDataset.addData(0, createIntervalData(createDate(16, 7, 2007), 39238000D)) ;

    return volumeDataset ;
  }

  private static SingleRowDataset createPriceDataset() {
    SingleRowDataset priceDataset = new SingleRowDataset() ;
    priceDataset.addRowKey("Price") ;
    priceDataset.addRowData() ;

    priceDataset.addData(0,
                         new OHLCChartData(createDate(12, 4, 2007), 5.9000000000000004D, 5.96D,
                                           5.8700000000000001D,
                                           5.9000000000000004D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(13, 4, 2007), 5.8899999999999997D,
                                           5.9000000000000004D,
                                           5.7800000000000002D, 5.7999999999999998D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(16, 4, 2007), 5.8099999999999996D,
                                           5.8700000000000001D,
                                           5.79D, 5.8600000000000003D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(17, 4, 2007), 5.8700000000000001D,
                                           5.9500000000000002D,
                                           5.8200000000000003D, 5.9500000000000002D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(18, 4, 2007), 5.8899999999999997D,
                                           5.9500000000000002D,
                                           5.8700000000000001D, 5.9400000000000004D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(19, 4, 2007), 5.8700000000000001D, 5.96D,
                                           5.8600000000000003D,
                                           5.8899999999999997D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(20, 4, 2007), 5.9400000000000004D,
                                           5.9500000000000002D,
                                           5.8700000000000001D, 5.9299999999999997D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(23, 4, 2007), 5.9299999999999997D,
                                           5.9500000000000002D,
                                           5.8899999999999997D, 5.9199999999999999D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(24, 4, 2007), 5.9299999999999997D,
                                           5.9699999999999998D,
                                           5.8799999999999999D, 5.9400000000000004D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(25, 4, 2007), 5.5800000000000001D,
                                           5.5800000000000001D,
                                           5.1699999999999999D, 5.2699999999999996D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(26, 4, 2007), 5.25D, 5.2999999999999998D,
                                           5.2000000000000002D,
                                           5.25D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(27, 4, 2007), 5.2300000000000004D,
                                           5.2800000000000002D,
                                           5.1900000000000004D, 5.2599999999999998D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(30, 4, 2007), 5.25D, 5.2599999999999998D,
                                           5.2000000000000002D,
                                           5.2199999999999998D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(1, 5, 2007), 5.2300000000000004D,
                                           5.2400000000000002D,
                                           4.9900000000000002D, 5.0899999999999999D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(2, 5, 2007), 5.0899999999999999D,
                                           5.1699999999999999D,
                                           5.0800000000000001D, 5.1500000000000004D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(3, 5, 2007), 5.1399999999999997D,
                                           5.2000000000000002D,
                                           5.1100000000000003D, 5.1799999999999997D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(4, 5, 2007), 5.21D, 5.2999999999999998D,
                                           5.2000000000000002D,
                                           5.2400000000000002D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(7, 5, 2007), 5.2199999999999998D,
                                           5.2800000000000002D,
                                           5.21D, 5.2199999999999998D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(8, 5, 2007), 5.2400000000000002D,
                                           5.2699999999999996D,
                                           5.21D, 5.2199999999999998D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(9, 5, 2007), 5.21D, 5.2199999999999998D,
                                           5.1500000000000004D,
                                           5.2000000000000002D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(10, 5, 2007), 5.1600000000000001D,
                                           5.1900000000000004D,
                                           5.1299999999999999D, 5.1299999999999999D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(11, 5, 2007), 5.1399999999999997D,
                                           5.1799999999999997D,
                                           5.1200000000000001D, 5.1500000000000004D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(14, 5, 2007), 5.1600000000000001D, 5.29D,
                                           5.1600000000000001D,
                                           5.2199999999999998D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(15, 5, 2007), 5.2199999999999998D,
                                           5.2300000000000004D,
                                           5.1299999999999999D, 5.1399999999999997D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(16, 5, 2007), 5.1399999999999997D,
                                           5.1600000000000001D,
                                           5.0700000000000003D, 5.1200000000000001D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(17, 5, 2007), 5.3499999999999996D,
                                           5.4299999999999997D,
                                           5.2999999999999998D, 5.2999999999999998D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(18, 5, 2007), 5.3499999999999996D,
                                           5.3499999999999996D,
                                           5.2599999999999998D, 5.29D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(21, 5, 2007), 5.29D, 5.3899999999999997D,
                                           5.2400000000000002D,
                                           5.3899999999999997D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(22, 5, 2007), 5.3899999999999997D,
                                           5.4199999999999999D,
                                           5.3399999999999999D, 5.3799999999999999D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(23, 5, 2007), 5.3700000000000001D,
                                           5.4299999999999997D,
                                           5.3600000000000003D, 5.3799999999999999D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(24, 5, 2007), 5.3600000000000003D,
                                           5.3700000000000001D,
                                           5.1500000000000004D, 5.1500000000000004D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(25, 5, 2007), 5.1799999999999997D, 5.21D,
                                           5.1500000000000004D,
                                           5.1600000000000001D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(29, 5, 2007), 5.1600000000000001D,
                                           5.1699999999999999D,
                                           5D, 5.0599999999999996D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(30, 5, 2007), 5.0099999999999998D,
                                           5.1500000000000004D,
                                           4.96D, 5.1200000000000001D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(31, 5, 2007), 5.1600000000000001D,
                                           5.1900000000000004D,
                                           5.0700000000000003D, 5.0999999999999996D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(1, 6, 2007), 5.1200000000000001D,
                                           5.2000000000000002D,
                                           5.1200000000000001D, 5.1799999999999997D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(4, 6, 2007), 5.1500000000000004D,
                                           5.2400000000000002D,
                                           5.0700000000000003D, 5.1100000000000003D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(5, 6, 2007), 5.0800000000000001D,
                                           5.0800000000000001D,
                                           4.9699999999999998D, 5.0700000000000003D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(6, 6, 2007), 5.0300000000000002D,
                                           5.0499999999999998D,
                                           4.9900000000000002D, 5.0199999999999996D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(7, 6, 2007), 5D, 5.0499999999999998D,
                                           4.9699999999999998D,
                                           4.9699999999999998D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(8, 6, 2007), 4.9800000000000004D, 5.04D,
                                           4.9500000000000002D,
                                           5.04D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(11, 6, 2007), 5.0499999999999998D,
                                           5.0599999999999996D,
                                           4.9500000000000002D, 4.96D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(12, 6, 2007), 4.9500000000000002D,
                                           5.0099999999999998D,
                                           4.9199999999999999D, 4.9199999999999999D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(13, 6, 2007), 4.9500000000000002D,
                                           4.9900000000000002D,
                                           4.8300000000000001D, 4.9900000000000002D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(14, 6, 2007), 5.0499999999999998D,
                                           5.0999999999999996D,
                                           5.0199999999999996D, 5.0800000000000001D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(15, 6, 2007), 5.1299999999999999D,
                                           5.1299999999999999D,
                                           5.0300000000000002D, 5.0499999999999998D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(18, 6, 2007), 5.0599999999999996D,
                                           5.0700000000000003D,
                                           5.0099999999999998D, 5.0499999999999998D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(19, 6, 2007), 5.0300000000000002D,
                                           5.1600000000000001D,
                                           5.0300000000000002D, 5.0999999999999996D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(20, 6, 2007), 5.1399999999999997D,
                                           5.1500000000000004D,
                                           5.0499999999999998D, 5.0499999999999998D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(21, 6, 2007), 5.0700000000000003D,
                                           5.1799999999999997D,
                                           5.0599999999999996D, 5.1299999999999999D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(22, 6, 2007), 5.1100000000000003D,
                                           5.1399999999999997D,
                                           5.0800000000000001D, 5.0800000000000001D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(25, 6, 2007), 5.0800000000000001D,
                                           5.0999999999999996D,
                                           4.9900000000000002D, 5.0199999999999996D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(26, 6, 2007), 5.04D, 5.0999999999999996D,
                                           4.9900000000000002D,
                                           5.0099999999999998D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(27, 6, 2007), 5D, 5.0899999999999999D,
                                           4.9900000000000002D,
                                           5.0700000000000003D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(28, 6, 2007), 5.0800000000000001D,
                                           5.1900000000000004D,
                                           5.0700000000000003D, 5.1600000000000001D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(29, 6, 2007), 5.1900000000000004D,
                                           5.3300000000000001D,
                                           5.1600000000000001D, 5.2599999999999998D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(2, 7, 2007), 5.1299999999999999D,
                                           5.3300000000000001D,
                                           5.1200000000000001D, 5.1900000000000004D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(3, 7, 2007), 5.2000000000000002D,
                                           5.2400000000000002D,
                                           5.1699999999999999D, 5.2199999999999998D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(5, 7, 2007), 5.2800000000000002D,
                                           5.3499999999999996D,
                                           5.2400000000000002D, 5.3300000000000001D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(6, 7, 2007), 5.3600000000000003D,
                                           5.4900000000000002D,
                                           5.3399999999999999D, 5.3799999999999999D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(9, 7, 2007), 5.4000000000000004D,
                                           5.4400000000000004D,
                                           5.3099999999999996D, 5.3300000000000001D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(10, 7, 2007), 5.29D, 5.4100000000000001D,
                                           5.2800000000000002D,
                                           5.3200000000000003D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(11, 7, 2007), 5.29D, 5.3799999999999999D,
                                           5.29D,
                                           5.3799999999999999D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(12, 7, 2007), 5.3799999999999999D,
                                           5.4299999999999997D,
                                           5.3300000000000001D, 5.4299999999999997D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(13, 7, 2007), 5.4199999999999999D,
                                           5.4299999999999997D,
                                           5.3399999999999999D, 5.3700000000000001D, 0)) ;
    priceDataset.addData(0,
                         new OHLCChartData(createDate(16, 7, 2007), 5.3300000000000001D,
                                           5.3799999999999999D,
                                           5.2999999999999998D, 5.3399999999999999D, 0)) ;

    return priceDataset ;
  }

  /**
   * Creates a date.
   *
   * @param day  the day.
   * @param month  the month (1 - 12).
   * @param yyyy  the year.
   *
   * @return a date.
   */
  public static synchronized Date createDate(final int day, final int month, final int yyyy) {
    CALENDAR.clear() ;
    CALENDAR.set(yyyy, month - 1, day, 0, 0) ;

    return CALENDAR.getTime() ;
  }

  private static IntervalXYNumberChartData createIntervalData(Date calStart, double value) {
    long start = calStart.getTime() ;

    long end = start + 24 * 60 * 60 * 1000 ;

    return new IntervalXYNumberChartData(start, end, 0, value) ;
  }

  private static XYNumberChartData createXYData(Date calStart, double value) {
    long start = calStart.getTime() ;

    return new XYNumberChartData(start, value) ;
  }

}
