package demo.miscellaneous.orientation;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.GridLayout ;
import java.awt.geom.Rectangle2D ;

import javax.swing.JPanel ;

import com.ijchart.xychart.annotation.XYLineAnnotation ;
import com.ijchart.xychart.annotation.XYShapeAnnotation ;
import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.marker.IntervalValueMarker ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.AbstractXYRender ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;
import com.ijchart.xychart.ui.Layer ;
import com.ijchart.xychart.ui.RectangleInsets ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class PlotOrientationDemo2 extends JPanel {
  private final static int CHART_COUNT = 8 ;

  private IMatrixDataset datasets[] ;

  private IJChart charts[] ;

  private ChartPanel panels[] ;

  private ChartOrientation orientation = ChartOrientation.VERTICAL ;

  public PlotOrientationDemo2() {
    super(new GridLayout(2, 4)) ;

    datasets = new IMatrixDataset[CHART_COUNT] ;
    charts = new IJChart[CHART_COUNT] ;
    panels = new ChartPanel[CHART_COUNT] ;
    addPanel1() ;
  }

  private void addPanel1() {
    for (int i = 0 ; i < CHART_COUNT ; i++) {
      datasets[i] = createDataset(i) ;
      charts[i] = createChart(i, datasets[i]) ;

      XYPlot plot = (XYPlot) charts[i].getPlot() ;
      Rectangle2D rect = new Rectangle2D.Double(1.0D, 2D, 2D, 3D) ;
      XYShapeAnnotation xyshapeannotation = new XYShapeAnnotation(rect, Color.blue,
                                                                  new BasicStroke(1.0F)) ;
      XYLineAnnotation xylineannotation = new XYLineAnnotation(0.0D, -5D, 10D, -5D) ;
      xylineannotation.setPaint(Color.black) ;

      AbstractXYRender render = plot.getRender() ;
      render.addAnnotation(xyshapeannotation) ;
      render.addAnnotation(xylineannotation) ;

      IntervalValueMarker marker1 = new IntervalValueMarker(5D, 10D) ;
      IntervalValueMarker marker2 = new IntervalValueMarker( -2D, 0.0D) ;
      marker1.setFillPaint(Color.gray) ;
      marker2.setFillPaint(Color.gray) ;

      plot.getDomainAxis().addIntervalValueMarker(Layer.BACKGROUND, marker1) ;
      plot.getValueAxis().addIntervalValueMarker(Layer.BACKGROUND, marker2) ;

      panels[i] = new ChartPanel(charts[i]) ;
    }

    XYPlot xyplot = (XYPlot) charts[1].getPlot() ;
    XYPlot xyplot2 = (XYPlot) charts[2].getPlot() ;
    XYPlot xyplot3 = (XYPlot) charts[3].getPlot() ;
    XYPlot xyplot4 = (XYPlot) charts[4].getPlot() ;
    XYPlot xyplot5 = (XYPlot) charts[5].getPlot() ;
    XYPlot xyplot6 = (XYPlot) charts[6].getPlot() ;
    XYPlot xyplot7 = (XYPlot) charts[7].getPlot() ;

    xyplot.getDomainAxis().setInverted(true) ;
    xyplot2.getValueAxis().setInverted(true) ;

    xyplot3.getDomainAxis().setInverted(true) ;
    xyplot3.getValueAxis().setInverted(true) ;

    xyplot4.getDomainAxis().setInverted(true) ;
    xyplot4.getValueAxis().setInverted(true) ;
    xyplot4.setOrientation(ChartOrientation.HORIZONTAL) ;

    xyplot5.getDomainAxis().setInverted(true) ;
    xyplot5.setOrientation(ChartOrientation.HORIZONTAL) ;
    xyplot6.getValueAxis().setInverted(true) ;
    xyplot6.setOrientation(ChartOrientation.HORIZONTAL) ;

    xyplot7.setOrientation(ChartOrientation.HORIZONTAL) ;

    add(panels[0]) ;
    add(panels[1]) ;
    add(panels[4]) ;
    add(panels[5]) ;
    add(panels[2]) ;
    add(panels[3]) ;
    add(panels[6]) ;
    add(panels[7]) ;

  }

  private IJChart createChart(int i, IMatrixDataset xydataset) {
    IJChart chart = ChartFactory.createXYLineAndShapeChart("Chart " + (i + 1), "X", "Y", xydataset,
                                                   orientation, false, false, false, false) ;
    XYPlot xyplot = (XYPlot) chart.getPlot() ;
    xyplot.setAxisOffset(new RectangleInsets(5D, 5D, 5D, 5D)) ;
    xyplot.setDomainGridlinePaint(Color.white) ;
    xyplot.setValueGridlinePaint(Color.white) ;

    XYLineAndShapeRender xylineandshaperenderer = (XYLineAndShapeRender) xyplot.getRender() ;
    xylineandshaperenderer.setBaseItemShapeVisible(true) ;
    xylineandshaperenderer.setBaseItemShapeFilled(true) ;
    xylineandshaperenderer.setBackgroundPaint(Color.lightGray) ;

    XYNumberValueAxis xAxis = (XYNumberValueAxis) xyplot.getDomainAxis() ;
    xAxis.setTickUnit(new LongTickUnit()) ;
    xAxis.setLowerMargin(0.03) ;
    xAxis.setUpperMargin(0.03) ;

    XYNumberValueAxis yAxis = (XYNumberValueAxis) xyplot.getValueAxis() ;
    yAxis.setTickUnit(new LongTickUnit()) ;
    yAxis.setLowerMargin(0.03) ;
    yAxis.setUpperMargin(0.03) ;

    return chart ;
  }

  private static DefaultMatrixDataset createDataset(int i) {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Series " + (i + 1)) ;
    dataset.addRowData(1) ;

    dataset.addData(0, new XYNumberChartData( -10D, -5D)) ;
    dataset.addData(0, new XYNumberChartData(10D, 5D)) ;

    return dataset ;
  }

}
