package demo.timeseriescharts ;

import java.awt.BorderLayout ;
import java.awt.Color ;
import java.text.SimpleDateFormat ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.DateTickUnitType ;
import com.ijchart.xychart.constant.XYValueAxisType ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.format.AbstractChartDataFormatterAdapter ;
import com.ijchart.xychart.format.IChartDataFormatter ;
import com.ijchart.xychart.format.IFormatter ;
import com.ijchart.xychart.format.MillisecondsToDateFormatter ;
import com.ijchart.xychart.format.NumberFormatter ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.tickunit.DateTickUnit ;
import com.ijchart.xychart.tooltip.DefaultTooltipGenerator ;
import com.ijchart.xychart.util.DateUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class CompareToPreviousYearDemo extends JPanel {
  private ChartOrientation orientation = ChartOrientation.VERTICAL ;

  public CompareToPreviousYearDemo() {
    setLayout(new BorderLayout()) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset2006 = createDataset2006() ;
    IMatrixDataset dataset2007 = createDataset2007() ;
    IJChart chart = ChartFactory.createXYLineAndShapeChart("Sales Comparison Chart", "Date",
                                                           "Sales", dataset2007, orientation, true,
                                                           true, true, false) ;

    XYPlot xyplot = (XYPlot) chart.getPlot() ;
    XYLineAndShapeRender render = (XYLineAndShapeRender) xyplot.getRender() ;
    IChartDataFormatter formatter = new AbstractChartDataFormatterAdapter() {
      private IFormatter format1 = new MillisecondsToDateFormatter("MMM-yyyy") ;

      private IFormatter format2 = new NumberFormatter("0.00") ;

      public String format(IMatrixDataset dataset, int row, int column) {
        XYNumberChartData data = (XYNumberChartData) dataset.getData(row, column) ;
        return format1.format(data.getXData(), row) + ":"
          + format2.format(data.getYData(), row) ;
      }
    } ;
    DefaultTooltipGenerator tooltip = new DefaultTooltipGenerator(formatter) ;
    render.setToolTipGenerator(tooltip) ;
    render.setAutoSelectFillPaint(false) ;
    render.setBaseItemFillPaint(Color.white) ;

    XYNumberValueAxis dateAxis = (XYNumberValueAxis) xyplot.getDomainAxis() ;
    dateAxis.setUserDefineDatasetRange(monthToMills(1, 2007), monthToMills(12, 2007)) ;
    dateAxis.setTickUnit(new DateTickUnit(DateTickUnitType.DATE_UNIT_MONTH,
                                          new SimpleDateFormat("MMM-yyyy"))) ;

    XYNumberValueAxis dateAxis1 = new XYNumberValueAxis(dataset2006,
                                                        XYValueAxisType.XY_VALUE_AXIS_X, true) ;
    dateAxis1.setVisible(false) ;
    dateAxis1.setUserDefineDatasetRange(monthToMills(1, 2006), monthToMills(12, 2006)) ;
    dateAxis1.setTickUnit(new DateTickUnit(DateTickUnitType.DATE_UNIT_MONTH,
                                           new SimpleDateFormat("MMM-yyyy"))) ;
    xyplot.setDomainAxis(1, dateAxis1) ;

    XYLineAndShapeRender render1 = new XYLineAndShapeRender(dataset2006) ;
    render1.setToolTipGenerator(tooltip) ;
    render1.setAutoSelectFillPaint(false) ;
    render1.setBaseItemFillPaint(Color.white) ;
    xyplot.setRender(1, render1) ;
    xyplot.mapRenderToDomainAxis(1, 1) ;

    XYNumberValueAxis valueAxis = (XYNumberValueAxis) xyplot.getValueAxis() ;
    valueAxis.setAxisRangeIncludeZero(true) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset2006() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Sales 2006") ;
    dataset.addRowData() ;

    dataset.addData(0, new XYNumberChartData(monthToMills(1, 2006), 100D)) ;
    dataset.addData(0, new XYNumberChartData(monthToMills(2, 2006), 102.3D)) ;
    dataset.addData(0, new XYNumberChartData(monthToMills(3, 2006), 105.7D)) ;
    dataset.addData(0, new XYNumberChartData(monthToMills(4, 2006), 104.2D)) ;
    dataset.addData(0, new XYNumberChartData(monthToMills(5, 2006), 114.7D)) ;
    dataset.addData(0, new XYNumberChartData(monthToMills(6, 2006), 121.7D)) ;
    dataset.addData(0, new XYNumberChartData(monthToMills(7, 2006), 155.59999999999999D)) ;
    dataset.addData(0, new XYNumberChartData(monthToMills(8, 2006), 143.19999999999999D)) ;
    dataset.addData(0, new XYNumberChartData(monthToMills(9, 2006), 131.90000000000001D)) ;
    dataset.addData(0, new XYNumberChartData(monthToMills(10, 2006), 120D)) ;
    dataset.addData(0, new XYNumberChartData(monthToMills(11, 2006), 109.90000000000001D)) ;
    dataset.addData(0, new XYNumberChartData(monthToMills(12, 2006), 99.599999999999994D)) ;

    return dataset ;
  }

  private static IMatrixDataset createDataset2007() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Sales 2007") ;
    dataset.addRowData() ;

    dataset.addData(0, new XYNumberChartData(monthToMills(1, 2007), 163.90000000000001D)) ;
    dataset.addData(0, new XYNumberChartData(monthToMills(2, 2007), 163.80000000000001D)) ;
    dataset.addData(0, new XYNumberChartData(monthToMills(3, 2007), 162D)) ;
    dataset.addData(0, new XYNumberChartData(monthToMills(4, 2007), 167.09999999999999D)) ;
    dataset.addData(0, new XYNumberChartData(monthToMills(5, 2007), 170D)) ;
    dataset.addData(0, new XYNumberChartData(monthToMills(6, 2007), 175.69999999999999D)) ;
    dataset.addData(0, new XYNumberChartData(monthToMills(7, 2007), 171.90000000000001D)) ;

    return dataset ;
  }

  private static long monthToMills(int month, int year) {
    return DateUtil.dateToTimeInMillis(year, month) ;
  }

}
