package demo.timeseriescharts ;

import java.awt.Graphics2D ;
import java.awt.GridLayout ;
import java.awt.geom.Rectangle2D ;
import java.text.SimpleDateFormat ;
import java.util.Calendar ;
import java.util.Date ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.IntervalXNumberYBaseValueAxis ;
import com.ijchart.xychart.chartinfo.AxisShapeCollection ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.AxisLabelType ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.DateTickUnitType ;
import com.ijchart.xychart.constant.XYValueAxisType ;
import com.ijchart.xychart.data.IntervalXNumberYBaseChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.format.IFormatter ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.tickunit.DateTickUnit ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.DateUtil ;
import com.ijchart.xychart.util.StringUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class PeriodAxisDemo3 extends JPanel {

  public PeriodAxisDemo3() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset = createDataset() ;
    IJChart chart = ChartFactory.createIntervalXBaseYBar2DChart("Maximum Temperature", "Day",
                                                                "Temperature", dataset,
                                                                ChartOrientation.VERTICAL, true, true,
                                                                true, false) ;
    XYPlot plot = (XYPlot) chart.getPlot() ;
    plot.setDomainCrosshairVisible(true) ;
    plot.setValueCrosshairVisible(true) ;

    IntervalXNumberYBaseValueAxis domainAxis =
      new IntervalXNumberYBaseValueAxis("Day", dataset, XYValueAxisType.XY_VALUE_AXIS_X, true) {
      protected double drawTickLabel(Graphics2D g2, Rectangle2D plotArea, Rectangle2D axisArea,
                                     RectangleEdge edge, AxisShapeCollection shapeCollection,
                                     double cursor) {
        cursor = super.drawTickLabel(g2, plotArea, axisArea, edge, shapeCollection, cursor) ;

        SimpleDateFormat dateFormat = new SimpleDateFormat("E") ;
        double[] ticks = super.valueTicks ;
        String[] secondLineTicks = new String[ticks.length] ;
        for (int i = 0 ; i < ticks.length ; i++) {
          Date date = new Date((long) ticks[i]) ;
          secondLineTicks[i] = dateFormat.format(date) ;
        }
        secondLineTicks = StringUtil.removeRepeatValues(secondLineTicks) ;
        cursor = drawTickLabelReally(g2, plotArea, axisArea, edge, shapeCollection, cursor,
                                     secondLineTicks) ;

        dateFormat = new SimpleDateFormat("MMM") ;
        String[] thirdLineTicks = new String[ticks.length] ;
        for (int i = 0 ; i < ticks.length ; i++) {
          Date date = new Date((long) ticks[i]) ;
          thirdLineTicks[i] = dateFormat.format(date) ;
        }
        thirdLineTicks = StringUtil.removeRepeatValues(thirdLineTicks) ;
        cursor = drawTickLabelReally(g2, plotArea, axisArea, edge, shapeCollection, cursor,
                                     thirdLineTicks) ;

        return cursor ;
      }

      protected int getTickLabelLines() {
        return 3 ;
      }

    } ;
    domainAxis.setAxisLabelType(AxisLabelType.COLUMN) ;
    domainAxis.setAxisRangeIncludeZero(false) ;
    domainAxis.setTickValueFormatter(new IFormatter() {
      private Calendar cal = Calendar.getInstance() ;

      private SimpleDateFormat format = new SimpleDateFormat("d") ;

      public String format(Object obj, int index) {
        if (obj != null) {
          String str = obj.toString() ;
          cal.setTimeInMillis(Long.parseLong(str)) ;

          return format.format(cal.getTime()) ;
        }

        return null ;
      }
    }) ;
    DateTickUnit unit = new DateTickUnit(DateTickUnitType.DATE_UNIT_DAY) ;
    unit.setDateFormatter(new SimpleDateFormat("dd")) ;
    domainAxis.setTickUnit(unit) ;

    plot.setDomainAxis(domainAxis) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Temperature") ;
    dataset.addRowData() ;

    dataset.addData(0, createData(1, 4, 2006, 14.5D)) ;
    dataset.addData(0, createData(2, 4, 2006, 11.5D)) ;
    dataset.addData(0, createData(3, 4, 2006, 13.699999999999999D)) ;
    dataset.addData(0, createData(4, 4, 2006, 10.5D)) ;
    dataset.addData(0, createData(5, 4, 2006, 14.9D)) ;
    dataset.addData(0, createData(6, 4, 2006, 15.699999999999999D)) ;
    dataset.addData(0, createData(7, 4, 2006, 11.5D)) ;
    dataset.addData(0, createData(8, 4, 2006, 9.5D)) ;
    dataset.addData(0, createData(9, 4, 2006, 10.9D)) ;
    dataset.addData(0, createData(10, 4, 2006, 14.1D)) ;
    dataset.addData(0, createData(11, 4, 2006, 12.300000000000001D)) ;
    dataset.addData(0, createData(12, 4, 2006, 14.300000000000001D)) ;
    dataset.addData(0, createData(13, 4, 2006, 19D)) ;
    dataset.addData(0, createData(14, 4, 2006, 17.899999999999999D)) ;

    return dataset ;
  }

  private static IntervalXNumberYBaseChartData createData(int day, int month, int year,
                                                          double value) {
    long startTime = DateUtil.dateToTimeInMillis(year, month, day) ;
    long endTime = startTime + 24 * 60 * 60 * 1000 ;
    return new IntervalXNumberYBaseChartData(startTime, endTime, value) ;
  }

}
