package demo.xycharts ;

import java.awt.Color ;
import java.awt.GridLayout ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.XYZNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.paint.LookupPaintScale ;
import com.ijchart.xychart.render.xy.XYBlockRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;
import com.ijchart.xychart.ui.RectangleInsets ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYBlockChartDemo3 extends JPanel {

  public XYBlockChartDemo3() {
    setLayout(new GridLayout(1, 1)) ;
    createPanel1() ;
  }

  private void createPanel1() {
    IMatrixDataset dataset = createDataset() ;
    IJChart chart = ChartFactory.createXYBlockChart(
      "XYBlockChartDemo3", "X", "Y", null, dataset, ChartOrientation.VERTICAL,
      true, true, false, false) ;

    XYPlot plot = (XYPlot) chart.getPlot() ;
    NumberValueAxis numberAxis = (NumberValueAxis) plot.getValueAxis() ;
    numberAxis.setTickUnit(new LongTickUnit()) ;
    numberAxis.setInverted(true) ;
    numberAxis.setAxisRangeIncludeZero(false) ;

    plot.setDomainGridlinePaint(Color.white) ;
    plot.setValueGridlinePaint(Color.white) ;
    plot.setForegroundAlpha(0.66F) ;
    plot.setAxisOffset(new RectangleInsets(5D, 5D, 5D, 5D)) ;

    String[] symbols = new String[] {"", "", "", "OK", "", "", "Uncertain", "", "", "Bad", ""} ;
    LookupPaintScale lookuppaintscale = new LookupPaintScale(0.5D, 3.5D, Color.black, symbols) ;
    lookuppaintscale.add(0.5D, Color.green) ;
    lookuppaintscale.add(1.5D, Color.orange) ;
    lookuppaintscale.add(2.5D, Color.red) ;

    XYBlockRender render = (XYBlockRender) plot.getRender() ;
    render.setPaintScale(lookuppaintscale) ;
    render.setBackgroundPaint(Color.lightGray) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static void setValue(double ad[][], int i, int j, double d) {
    ad[0][(j - 8) * 60 + i] = i ;
    ad[1][(j - 8) * 60 + i] = j ;
    ad[2][(j - 8) * 60 + i] = d ;
  }

  private static IMatrixDataset createDataset() {
    double ad[] = new double[840] ;
    double ad1[] = new double[840] ;
    double ad2[] = new double[840] ;
    double ad3[][] = {ad, ad1, ad2} ;
    for (int i = 0 ; i < 60 ; i++) {
      for (int j1 = 8 ; j1 < 22 ; j1++) {
        setValue(ad3, i, j1, 0.0D) ;
      }
    }

    for (int j = 8 ; j < 12 ; j++) {
      for (int k1 = 13 ; k1 < 48 ; k1++) {
        setValue(ad3, k1, j, 1.0D) ;
      }
    }

    for (int k = 12 ; k < 20 ; k++) {
      for (int l1 = 23 ; l1 < 43 ; l1++) {
        setValue(ad3, l1, k, 1.0D) ;
      }
    }

    setValue(ad3, 2, 20, 2D) ;
    setValue(ad3, 5, 20, 3D) ;
    setValue(ad3, 6, 20, 3D) ;
    setValue(ad3, 7, 20, 3D) ;
    setValue(ad3, 8, 20, 3D) ;
    setValue(ad3, 9, 20, 3D) ;
    setValue(ad3, 11, 20, 3D) ;
    setValue(ad3, 17, 20, 2D) ;
    setValue(ad3, 18, 20, 2D) ;
    setValue(ad3, 19, 20, 2D) ;
    setValue(ad3, 20, 20, 2D) ;
    setValue(ad3, 22, 20, 2D) ;
    setValue(ad3, 25, 20, 2D) ;
    setValue(ad3, 28, 20, 2D) ;
    setValue(ad3, 35, 20, 2D) ;
    for (int l = 40 ; l < 60 ; l++) {
      setValue(ad3, l, 20, 3D) ;
    }

    for (int i1 = 23 ; i1 < 43 ; i1++) {
      setValue(ad3, i1, 21, 1.0D) ;
    }

    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowData() ;
    dataset.addRowKey("Series 1") ;

    for (int i = 0 ; i < ad.length ; i++) {
      dataset.addData(0, new XYZNumberChartData(ad3[0][i], ad3[1][i], ad3[2][i])) ;
    }

    return dataset ;
  }

}
