package com.ijchart.xychart.axis.category ;

import java.awt.Graphics2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.Collections ;
import java.util.List ;

import com.ijchart.xychart.axis.AbstractAxis ;
import com.ijchart.xychart.chartinfo.AxisShapeCollection ;
import com.ijchart.xychart.data.group.AbstractGroupChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.format.IFormatter ;
import com.ijchart.xychart.listener.IMarkerChangeListener ;
import com.ijchart.xychart.render.category.AbstractCategoryRender ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.RectangleInsets ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.StringUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Էʽʾֵֵ.</zh>
 * <en>A group category axis that can display group-categories.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class GroupCategoryAxis extends CategoryAxis implements IMarkerChangeListener, Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -3865482239482396447L ;

  /**
   * <zh>֮ļ.</zh>
   * <en>The gap size between two axis.</en>
   */
  private static final int AXIS_GAP = 5 ;

  /**
   * <zh>Ἧ.</zh>
   * <en>Storage for the sub-categories.</en>
   */
  private List subCategoryAxes ;

  /**
   * <zh>͵Ļ.</zh>
   * <en>The render for category item.</en>
   */
  private AbstractCategoryRender categoryRender ;

  /**
   * <zh>ݼ.</zh>
   * <en>Constructs an axis using a dataset.</en>
   *
   * @param categoryRender AbstractCategoryRender <zh>͵Ļ (Ϊnull).</zh>
   * <en>The render for category item (null not permitted).</en>
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   */
  public GroupCategoryAxis(AbstractCategoryRender categoryRender, IMatrixDataset dataset) {
    this(categoryRender, null, dataset) ;
  }

  /**
   * <zh>ǩݼ.</zh>
   * <en>Constructs an axis using label and a dataset.</en>
   *
   * @param categoryRender AbstractCategoryRender <zh>͵Ļ (Ϊnull).</zh>
   * <en>The render for category item (null not permitted).</en>
   * @param label <zh>ǩ.</zh>
   * <en>The axis label (null permitted).</en>
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   */
  public GroupCategoryAxis(AbstractCategoryRender categoryRender, String label,
                           IMatrixDataset dataset) {
    super(label, dataset) ;
    if (categoryRender == null) {
      throw new IllegalArgumentException("Null 'categoryRender' argument.") ;
    }

    this.categoryRender = categoryRender ;
    this.subCategoryAxes = new ArrayList() ;
  }

  /**
   * <zh>һ,м{@link com.ijchart.xychart.event.AxisChangeEvent}¼.</zh>
   * <en>Adds a sub-category to the axis and sends an
   *  {@link com.ijchart.xychart.event.AxisChangeEvent} to all registered listeners.</en>
   *
   * @param categoryAxis CategoryAxis <zh> (Ϊnull).</zh>
   * <en>The sub-category (null not permitted).</en>
   */
  public void addSubCategoryAxis(CategoryAxis categoryAxis) {
    if (categoryAxis == null) {
      throw new IllegalArgumentException("Null 'categoryAxis' axis.") ;
    }

    this.subCategoryAxes.add(categoryAxis) ;
    fireAxisChanged() ;
  }

  /**
   * <zh>ӷĸ.</zh>
   * <en>Returns the size of sub-category axes.</en>
   *
   * @return int <zh>ӷĸ.</zh>
   * <en>The number of sub-category axes.</en>
   */
  public int getSubCategoryAxisCount() {
    return this.subCategoryAxes.size() ;
  }

  /**
   * <zh>ӷļ,˼ϲ޸.</zh>
   * <en>Returns the collection of sub-category axes,this list is unmodifiable.</en>
   *
   * @return List <zh> (Ϊnull).</zh>
   * <en>The axes list (never null).</en>
   */
  public List getSubCategoryAxes() {
    return Collections.unmodifiableList(this.subCategoryAxes) ;
  }

  /**
   * <zh>ɾָӷ,м
   * {@link com.ijchart.xychart.event.AxisChangeEvent}¼.</zh>
   * <en>Removes the category at the specified position and sends
   * an {@link com.ijchart.xychart.event.AxisChangeEvent} to all registered listeners.</en>
   *
   * @param index int <zh>ֵ.</zh>
   * <en>The index.</en>
   *
   * @return CategoryAxis <zh>ɾ.</zh>
   * <en>The removed CategoryAxis.</en>
   */
  public CategoryAxis removeSubCategoryAxis(int index) {
    return (CategoryAxis)this.subCategoryAxes.remove(index) ;
  }

  /**
   * <zh>ɾеӷ.</zh>
   * <en>Remove all sub-category axes.</en>
   */
  public void removeAllSubCategoryAxes() {
    this.subCategoryAxes.clear() ;
  }

  /**
   * <zh>ط͵Ļ.</zh>
   * <en>Returns the render for category item.</en>
   *
   * @return AbstractCategoryRender <zh>Ϊnull.</zh>
   * <en>Never null.</en>
   */
  public AbstractCategoryRender getCategoryRender() {
    return this.categoryRender ;
  }

  /**
   * <zh>÷͵Ļ,м{@link com.ijchart.xychart.event.AxisChangeEvent}
   * ¼.</zh>
   * <en>Sets the render for category item and sends an
   * {@link com.ijchart.xychart.event.AxisChangeEvent} to all registered listeners.</en>
   *
   * @param categoryRender AbstractCategoryRender
   */
  public void setCategoryRender(AbstractCategoryRender categoryRender) {
    if (categoryRender == null) {
      throw new IllegalArgumentException("Null 'categoryRender' argument.") ;
    }

    this.categoryRender = categoryRender ;
    fireAxisChanged() ;
  }

  /**
   * <zh>ҪĿȻ߶.</zh>
   * <en>Calculation the width or height of painting coordinate axis.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param plotArea Rectangle2D <zh>ͼplot.</zh>
   * <en>The area within which the plot should be drawn.</en>
   * @param maxAxisArea Rectangle2D <zh>,϶СplotArea.</zh>
   * <en>The maximum area within which the axis should be drawn.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   *
   * @return double <zh>Ȼ߶.</zh>
   * <en>The width or height.</en>
   */
  public double calculateAxisSpace(Graphics2D g2, Rectangle2D plotArea, Rectangle2D maxAxisArea,
                                   RectangleEdge edge) {
    if (this.subCategoryAxes.size() == 0) { //û
      return super.calculateAxisSpace(g2, plotArea, maxAxisArea, edge) ;
    }

    double result = super.calculateAxisSpace(g2, plotArea, maxAxisArea, edge) ;
    for (int i = 0 ; i < this.subCategoryAxes.size() ; i++) {
      AbstractAxis axis = (AbstractAxis)this.subCategoryAxes.get(i) ;
      result += AXIS_GAP ;
      result += axis.calculateAxisSpace(g2, plotArea, maxAxisArea, edge) ;
    }

    return result ;
  }

  /**
   * <zh>ͼplot(plotArea)XY.ͼеͼδshapeCollection.</zh>
   * <en>Draw the X-axis or Y-axis on plot area(plotArea).All axis shape is stored in
   * shapeCollection.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param plotArea Rectangle2D <zh>ͼplot.</zh>
   * <en>The area within which the plot should be drawn.</en>
   * @param axisArea Rectangle2D <zh>ʣĿհ.</zh>
   * <en>The empty chart area.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   * @param shapeCollection AxisShapeCollection <zh>ͼμ.</zh>
   * <en>Collects information about the axis (null permitted).</en>
   * @param alignEdge boolean <zh>־ʵʻС,ǷԶ߽.</zh>
   * <en>Controls whether or not translate shapes when the draw area less than the axis area.</en>
   *
   * @return double <zh>ʵʻƵĿȻ߶.</zh>
   * <en>Drawing the actual coordinates axis width or height.</en>
   */
  public double draw(Graphics2D g2, Rectangle2D plotArea, Rectangle2D axisArea,
                     RectangleEdge edge, AxisShapeCollection shapeCollection, boolean alignEdge) {
    if (this.subCategoryAxes.size() == 0) { //û
      return super.draw(g2, plotArea, axisArea, edge, shapeCollection, alignEdge) ;
    }

    Rectangle2D cloneArea = (Rectangle2D) axisArea.clone() ;

    AxisShapeCollection axisShapes = new AxisShapeCollection() ;
    double axisHeight = super.draw(g2, plotArea, cloneArea, edge, axisShapes, false) ;
    axisHeight += AXIS_GAP ;
    shapeCollection.copyShapes(axisShapes) ;
    double result = axisHeight ;

    for (int i = 0 ; i < this.subCategoryAxes.size() ; i++) {
      //ƶελ
      double x = cloneArea.getX() ;
      double y = cloneArea.getY() ;
      double width = cloneArea.getWidth() ;
      double height = cloneArea.getHeight() ;
      if (edge.equals(RectangleEdge.TOP)) {
        height -= axisHeight ;
      } else if (edge.equals(RectangleEdge.BOTTOM)) {
        y += axisHeight ;
        height -= axisHeight ;
      } else if (edge.equals(RectangleEdge.LEFT)) {
        width -= axisHeight ;
      } else if (edge.equals(RectangleEdge.RIGHT)) {
        x += axisHeight ;
        width -= axisHeight ;
      }
      cloneArea.setFrame(x, y, width, height) ;

      AbstractAxis axis = (AbstractAxis)this.subCategoryAxes.get(i) ;
      AxisShapeCollection subAxisShapes = new AxisShapeCollection() ;
      axisHeight = axis.draw(g2, plotArea, cloneArea, edge, subAxisShapes, false) ;
      if (i != this.subCategoryAxes.size() - 1) {
        axisHeight += AXIS_GAP ;
      }
      shapeCollection.copyShapes(subAxisShapes) ;
      result += axisHeight ;
    }

    if (alignEdge) { //߶
      double translateX = 0, translateY = 0 ;
      if (edge.equals(RectangleEdge.TOP)) {
        double axisAreaHeight = axisArea.getHeight() ;
        if (axisAreaHeight > result) {
          translateY = -(axisAreaHeight - result) ;
        }
      } else if (edge.equals(RectangleEdge.BOTTOM)) {
        double axisAreaHeight = axisArea.getHeight() ;
        if (axisAreaHeight > result) {
          translateY = axisAreaHeight - result ;
        }
      } else if (edge.equals(RectangleEdge.LEFT)) {
        double axisWidth = axisArea.getWidth() ;
        if (axisWidth > result) {
          translateX = -(axisWidth - result) ;
        }
      } else if (edge.equals(RectangleEdge.RIGHT)) {
        double axisWidth = axisArea.getWidth() ;
        if (axisWidth > result) {
          translateX = axisWidth - result ;
        }
      }

      if (!(translateX == 0 && translateY == 0)) { //ƶͼ
        axisShapes.translate(translateX, translateY) ;
      }

    }

    return result ;
  }

  /**
   * <zh>ϵĿ̶ֵıǩ.</zh>
   * <en>Draws tick mark labels in a specified area and edge.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param plotArea Rectangle2D <zh>ͼplot.</zh>
   * <en>The area within which the plot should be drawn.</en>
   * @param axisArea Rectangle2D <zh>ʣĿհ.</zh>
   * <en>The empty chart area.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   * @param shapeCollection AxisShapeCollection <zh>ͼμ.</zh>
   * <en>Collects information about the axis (null permitted).</en>
   * @param cursor <zh>λ.</zh>
   * <en>The cursor location.</en>
   *
   * @return double <zh>̶ֵλ.</zh>
   * <en>The cursor location after draw tick label.</en>
   */
  protected double drawTickLabel(Graphics2D g2, Rectangle2D plotArea, Rectangle2D axisArea,
                                 RectangleEdge edge, AxisShapeCollection shapeCollection,
                                 double cursor) {
    if (!isVisible() || !isTickLabelsVisible()) {
      return cursor ;
    }

    IMatrixDataset dataset = getDataset() ;
    int columns = dataset.getColumnCount() ;
    int rows = dataset.getRowCount() ;
    this.categoryLabelPosition.clear() ;

    String[] keyArr = getFormatedTickLabels(null, null, null) ;
    if (StringUtil.isEmpty(keyArr, true)) {
      return cursor ;
    }

    int tickMarkCount = keyArr.length ;
    double tickLabelMaxHeight = getLabelMaxDrawWidth(g2, plotArea, edge) ;
    Rectangle2D tickLabelDrawRect = null ;

    if (RectangleEdge.isTopOrBottom(edge)) { //top or bottom axis
      double gapSize = 0 ; //С
      if (rows > 1) {
        gapSize = axisArea.getWidth() * getCategoryMargin() / (rows - 1) ;
      }
      double axisMargin = getLowerMargin() + getUpperMargin()
                          + ((rows == 1) ? 0 : getCategoryMargin()) ; //еĿհ
      double categoryWidth = axisArea.getWidth() * (1 - axisMargin) / rows ; //ÿϵеĿ
      double categoryOneMargin = (axisArea.getWidth() * (1 - getLowerMargin() - getUpperMargin())
                                  * this.categoryRender.getItemMargin()) / rows ; //ÿcategoryĿհ

      double calculateBarWidth = (categoryWidth - categoryOneMargin) / columns ; //Ŀ
      double barItemMargin = categoryOneMargin / columns ; //ÿ֮Ŀ

      Rectangle2D labelRect = calculateLabelBound(g2, calculateBarWidth, tickLabelMaxHeight,
                                                  getTickLabelAngle(), edge, keyArr,
                                                  getTickLabelFont(), RectangleInsets.INSETS_ZERO) ;
      for (int i = 0 ; i < rows ; i++) {
        double startX = axisArea.getX() + axisArea.getWidth() * getLowerMargin()
                        + i * (categoryWidth + gapSize) ;
        int currRowCols = dataset.getColumnCount(i) ;

        for (int j = 0 ; j < currRowCols ; j++) {
          double x11 = startX + 0.5 * barItemMargin + (calculateBarWidth + barItemMargin) * j ;
          Object dataObj = dataset.getData(i, j) ;
          Object group = getGroupObject(dataObj) ;
          this.categoryLabelPosition.put(StringUtil.nullToString(group),
                                         new Double(x11 + labelRect.getX())) ;
          if (edge.equals(RectangleEdge.TOP)) {
            tickLabelDrawRect = new Rectangle2D.Double(x11 + labelRect.getX(),
                                                       cursor + labelRect.getY(),
                                                       labelRect.getWidth(), labelRect.getHeight()) ;

            if (i == tickMarkCount - 1) { //λ
              cursor -= getTickLabelRotateBound(tickLabelDrawRect, getTickLabelAngle()).getHeight() ;
            }
          } else if (edge.equals(RectangleEdge.BOTTOM)) {
            tickLabelDrawRect = new Rectangle2D.Double(x11 + labelRect.getX(),
                                                       cursor + labelRect.getY(),
                                                       labelRect.getWidth(), labelRect.getHeight()) ;
            if (i == tickMarkCount - 1) { //λ
              cursor += getTickLabelRotateBound(tickLabelDrawRect, getTickLabelAngle()).getHeight() ;
            }
          }

          String label = keyArr[i * columns + j] ;
          shapeCollection.addTickLabelShape(createChartShape(tickLabelDrawRect, label, i)) ;
        }
      }
    } else if (RectangleEdge.isLeftOrRight(edge)) { //left or right axis
      double gapSize = 0 ; //С
      if (rows > 1) {
        gapSize = axisArea.getHeight() * getCategoryMargin() / (rows - 1) ;
      }
      double axisMargin = getLowerMargin() + getUpperMargin()
                          + ((rows == 1) ? 0 : getCategoryMargin()) ; //еĿհ
      double categoryWidth = axisArea.getHeight() * (1 - axisMargin) / rows ; //ÿϵеĿ
      double categoryOneMargin = (axisArea.getHeight() * (1 - getLowerMargin() - getUpperMargin())
                                  * this.categoryRender.getItemMargin()) / rows ; //ÿcategoryĿհ

      double calculateBarWidth = (categoryWidth - categoryOneMargin) / columns ; //Ŀ
      double barItemMargin = categoryOneMargin / columns ; //ÿ֮Ŀ

      Rectangle2D labelRect = calculateLabelBound(g2, calculateBarWidth, tickLabelMaxHeight,
                                                  getTickLabelAngle(), edge, keyArr,
                                                  getTickLabelFont(), RectangleInsets.INSETS_ZERO) ;
      for (int i = 0 ; i < rows ; i++) {
        double startY = axisArea.getY() + axisArea.getHeight() * getLowerMargin()
                        + i * (categoryWidth + gapSize) ;
        int currRowCols = dataset.getColumnCount(i) ;

        for (int j = 0 ; j < currRowCols ; j++) {
          double y11 = startY + 0.5 * barItemMargin + (calculateBarWidth + barItemMargin) * j ;
          Object dataObj = dataset.getData(i, j) ;
          Object group = getGroupObject(dataObj) ;
          this.categoryLabelPosition.put(StringUtil.nullToString(group),
                                         new Double(y11 + labelRect.getY())) ;

          if (edge.equals(RectangleEdge.LEFT)) {
            tickLabelDrawRect = new Rectangle2D.Double(cursor + labelRect.getX(),
                                                       y11 + labelRect.getY(),
                                                       labelRect.getWidth(), labelRect.getHeight()) ;
            if (i == tickMarkCount - 1) { //λ
              cursor -= getTickLabelRotateBound(tickLabelDrawRect, getTickLabelAngle()).getWidth() ;
            }
          } else if (edge.equals(RectangleEdge.RIGHT)) {
            tickLabelDrawRect = new Rectangle2D.Double(cursor + labelRect.getX(),
                                                       y11 + labelRect.getY(),
                                                       labelRect.getWidth(), labelRect.getHeight()) ;
            if (i == tickMarkCount - 1) { //λ
              cursor += getTickLabelRotateBound(tickLabelDrawRect, getTickLabelAngle()).getWidth() ;
            }
          }

          String label = keyArr[i * columns + j] ;
          shapeCollection.addTickLabelShape(createChartShape(tickLabelDrawRect, label, i)) ;
        }
      }
    }

    return cursor ;
  }

  /**
   * <zh>Group.</zh>
   * <en>Calculate the Group object.</en>
   *
   * @param dataObj Object
   * @return Object
   */
  private static Object getGroupObject(Object dataObj) {
    Object group = null ;
    if (dataObj == null) {
      group = null ;
    } else {
      if (dataObj instanceof AbstractGroupChartData) {
        AbstractGroupChartData groupData = (AbstractGroupChartData) dataObj ;
        group = groupData.getGroup() ;
      } else {
        group = null ;
      }
    }

    return group ;
  }

  /**
   * <zh>ظʽʾĿ̶ֵ.</zh>
   * <en>Returns the formated tick labels.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ),Ϊnull.</zh>
   * <en>The graphics device (null permitted).</en>
   * @param maxAxisArea Rectangle2D <zh>,϶СplotArea (Ϊnull).</zh>
   * <en>The maximum area within which the axis should be drawn (null permitted).</en>
   * @param edge RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The axis position (null permitted).</en>
   *
   * @return String[] <zh>̶ֵ.</zh>
   * <en>The array of tick labels.</en>
   */
  protected String[] getFormatedTickLabels(Graphics2D g2, Rectangle2D maxAxisArea,
                                           RectangleEdge edge) {
    List labelList = new ArrayList() ;

    IMatrixDataset dataset = getDataset() ;
    IFormatter formatter = getRowKeyFormatter() ;
    int rowCount = dataset.getRowCount() ;

    for (int i = 0 ; i < rowCount ; i++) {
      int currRowCols = dataset.getColumnCount(i) ;
      for (int j = 0 ; j < currRowCols ; j++) {
        Object dataObj = dataset.getData(i, j) ;
        if (dataObj == null) {
          labelList.add(null) ;
        } else {
          if (dataObj instanceof AbstractGroupChartData) {
            AbstractGroupChartData groupData = (AbstractGroupChartData) dataObj ;
            if (formatter == null) {
              Object group = groupData.getGroup() ;
              labelList.add((group == null) ? null : group.toString()) ;
            } else {
              labelList.add(formatter.format(groupData.getGroup(), i)) ;
            }
          } else {
            labelList.add(null) ;
          }
        }
      }
    }

    return (String[]) labelList.toArray(new String[0]) ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Creates a clone of the axis.</en>
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException if some component of the axis does
   *         not support cloning.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>ԱǷһ.</zh>
   * <en>Tests this axis for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪԵĶ (Ϊnull).</zh>
   * <en>The object (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof GroupCategoryAxis)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    GroupCategoryAxis that = (GroupCategoryAxis) obj ;
    if (!EqualsUtil.equals(this.subCategoryAxes, that.subCategoryAxes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.categoryRender, that.categoryRender)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code for this object.</en>
   *
   * @return <zh>hashcode.</zh>
   * <en>A hash code.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 29 * result + HashcodeUtil.hashCode(this.subCategoryAxes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.categoryRender) ;

    return result ;
  }

}
