package com.ijchart.xychart.axis.value ;

import java.io.Serializable ;

import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.exception.InvalidChartDataException ;
import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.render.IStackedRender ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ջ͵ͼʹõֵ.</zh>
 * <en>The axis used for stack bar chart.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class StackedNumberValueAxis extends NumberValueAxis implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 1488878936787053004L ;

  /**
   * <zh>ջ״ͼƶ.</zh>
   * <en>The stacked bar render object.</en>
   */
  private IStackedRender stackedRender ;

  /**
   * <zh>ݼ.</zh>
   * <en>Constructs an axis using a dataset.</en>
   *
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   * @param render IStackedRender <zh>ջ״ͼƶ (Ϊnull).</zh>
   * <en>The stacked bar render object (null not permitted).</en>
   */
  public StackedNumberValueAxis(IMatrixDataset dataset, IStackedRender render) {
    this(null, dataset, render) ;
  }

  /**
   * <zh>ǩݼ.</zh>
   * <en>Constructs an axis using label and a dataset.</en>
   *
   * @param label <zh>ǩ.</zh>
   * <en>The axis label (null permitted).</en>
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   * @param render IStackedRender <zh>ջ״ͼƶ (Ϊnull).</zh>
   * <en>The stacked bar render object (null not permitted).</en>
   */
  public StackedNumberValueAxis(String label, IMatrixDataset dataset, IStackedRender render) {
    super(label, dataset) ;
    if (render == null) {
      throw new IllegalArgumentException("Null 'render' argument.") ;
    }

    this.stackedRender = render ;
  }

  /**
   * <zh>״ͼƶ.</zh>
   * <en>Returns the stacked bar render.</en>
   *
   * @return StackedBarRender <zh>ջ״ͼƶ (Ϊnull).</zh>
   * <en>The stacked bar render object (never null).</en>
   */
  public IStackedRender getStackedBarRender() {
    return this.stackedRender ;
  }

  /**
   * <zh>״ͼƶ,м{@link com.ijchart.xychart.event.AxisChangeEvent}¼.</zh>
   * <en>Sets the stacked bar render object and sends a
   *  {@link com.ijchart.xychart.event.AxisChangeEvent} to all registered listeners.</en>
   *
   * @param render IStackedRender <zh>ջ״ͼƶ (Ϊnull).</zh>
   * <en>The stacked bar render object (null not permitted).</en>
   */
  public void setStackedBarRender(IStackedRender render) {
    if (render == null) {
      throw new IllegalArgumentException("Null 'render' argument.") ;
    }

    this.stackedRender = render ;
    fireAxisChanged() ;
  }

  /**
   * <zh>õݼݷΧ.</zh>
   * <en>Returns the value range of a dataset.</en>
   *
   * @return NumberRange <zh>ݷΧ (Ϊnull).</zh>
   * <en>The value range of a dataset (never null).</en>
   */
  public NumberRange getDatasetRange() {
    if (getUserDefineDatasetRange() != null) {
      return getUserDefineDatasetRange() ;
    }

    double minValue = Double.NaN ;
    double maxValue = Double.NaN ;
    IMatrixDataset dataset = getDataset() ;
    if (dataset != null) {
      if (this.stackedRender != null && this.stackedRender.isRenderAsPercentages()) {
        boolean positive = false, negative = false ; //Ƿֵ͸ֵ

        int rows = dataset.getRowCount() ;
        for (int i = 0 ; i < rows ; i++) {
          int currRowCols = dataset.getColumnCount(i) ;
          for (int j = 0 ; j < currRowCols ; j++) {
            Object data = dataset.getData(i, j) ;
            if (data != null) {
              if (!(data instanceof NumberChartData)) {
                throw new InvalidChartDataException(
                  "Invalid chart data type,musts be 'NumberChartData'.") ;
              } else { //Сֵиֵ֮,ֵֵ֮
                NumberChartData numberData = (NumberChartData) data ;
                double value = numberData.getValue() ;
                if (Double.isNaN(value)) {
                  continue ;
                }

                if (value > 0) {
                  positive = true ;
                } else if (value < 0) {
                  negative = true ;
                }
                if (positive && negative) {
                  return new NumberRange( -100, 100) ;
                }
              }
            }
          }
        }

        if (negative && !positive) { //
          return new NumberRange( -100, 0) ;
        } else if (positive && !negative) { //
          return new NumberRange(0, 100) ;
        } else {
          return new NumberRange( -100, 100) ;
        }
      } else {
        LegendType legendType = this.stackedRender.getLegendType() ;
        if (legendType.equals(LegendType.COLUMN)) {
          int rows = dataset.getRowCount() ;
          for (int i = 0 ; i < rows ; i++) {
            double rowMinValue = 0, rowMaxValue = 0 ; //һݵСֵ
            int currRowCols = dataset.getColumnCount(i) ;

            for (int j = 0 ; j < currRowCols ; j++) {
              Object data = dataset.getData(i, j) ;
              if (data != null) {
                if (!(data instanceof NumberChartData)) {
                  throw new InvalidChartDataException(
                    "Invalid chart data type,musts be 'NumberChartData'.") ;
                } else { //Сֵиֵ֮,ֵֵ֮
                  NumberChartData numberData = (NumberChartData) data ;
                  double value = numberData.getValue() ;
                  if (!Double.isNaN(value)) {
                    rowMinValue += Math.min(0, value) ;
                    rowMaxValue += Math.max(0, value) ;
                  }
                }
              }
            }

            if (Double.isNaN(minValue)) {
              minValue = rowMinValue ;
            } else {
              minValue = Math.min(minValue, rowMinValue) ;
            }

            if (Double.isNaN(maxValue)) {
              maxValue = rowMaxValue ;
            } else {
              maxValue = Math.max(maxValue, rowMaxValue) ;
            }
          }
        } else if (legendType.equals(LegendType.ROW)) {
          int rows = dataset.getRowCount() ;
          int cols = dataset.getColumnCount() ;
          for (int i = 0 ; i < cols ; i++) {
            double rowMinValue = 0, rowMaxValue = 0 ; //һݵСֵ
            for (int j = 0 ; j < rows ; j++) {
              Object data = dataset.getData(j, i) ;
              if (data != null) {
                if (!(data instanceof NumberChartData)) {
                  throw new InvalidChartDataException(
                    "Invalid chart data type,musts be 'NumberChartData'.") ;
                } else { //Сֵиֵ֮,ֵֵ֮
                  NumberChartData numberData = (NumberChartData) data ;
                  double value = numberData.getValue() ;
                  if (!Double.isNaN(value)) {
                    rowMinValue += Math.min(0, value) ;
                    rowMaxValue += Math.max(0, value) ;
                  }
                }
              }
            }

            if (Double.isNaN(minValue)) {
              minValue = rowMinValue ;
            } else {
              minValue = Math.min(minValue, rowMinValue) ;
            }

            if (Double.isNaN(maxValue)) {
              maxValue = rowMaxValue ;
            } else {
              maxValue = Math.max(maxValue, rowMaxValue) ;
            }
          }

        }
      }
    }

    if (Double.isNaN(minValue)) {
      minValue = 0 ;
    }
    if (Double.isNaN(maxValue)) {
      maxValue = 0 ;
    }

    return new NumberRange(minValue, maxValue) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument;
   * false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!(obj instanceof StackedNumberValueAxis)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    StackedNumberValueAxis that = (StackedNumberValueAxis) obj ;
    if (!EqualsUtil.equals(this.stackedRender, that.stackedRender)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int hashCode = super.hashCode() ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.stackedRender) ;

    return hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[render=" + this.stackedRender + "]" ;
  }

}
