package com.ijchart.xychart.axis.value ;

import java.io.Serializable ;

import com.ijchart.xychart.constant.XYValueAxisType ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.exception.InvalidChartDataException ;
import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.tickunit.DateTickUnit ;
import com.ijchart.xychart.util.DatasetUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ջ͵ͼʹõֵ,ֵXYNumberChartData.</zh>
 * <en>The axis used for xy stack area chart,the data value type is XYNumberChartData.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class StackedXYValueAxis extends AbstractNumberValueAxisWithType implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -6740712922634281858L ;

  /**
   * <zh>ݼ.</zh>
   * <en>Constructs an axis using a dataset.</en>
   *
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   * @param valueAxisType XYValueAxisType <zh>value (Ϊnull).</zh>
   * <en>The type of value axis (null not permitted).</en>
   * @param isDateAxis boolean <zh>Ƿ͵.</zh>
   * <en>Whether or not is a date axis.</en>
   */
  public StackedXYValueAxis(IMatrixDataset dataset, XYValueAxisType valueAxisType,
                            boolean isDateAxis) {
    this(null, dataset, valueAxisType, isDateAxis) ;
  }

  /**
   * <zh>ǩݼ.</zh>
   * <en>Constructs an axis using label and a dataset.</en>
   *
   * @param label <zh>ǩ.</zh>
   * <en>The axis label (null permitted).</en>
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   * @param valueAxisType XYValueAxisType <zh>value (Ϊnull).</zh>
   * <en>The type of value axis (null not permitted).</en>
   * @param isDateAxis boolean <zh>Ƿ͵.</zh>
   * <en>Whether or not is a date axis.</en>
   */
  public StackedXYValueAxis(String label, IMatrixDataset dataset,
                            XYValueAxisType valueAxisType, boolean isDateAxis) {
    super(label, dataset, valueAxisType) ;

    if (isDateAxis) {
      this.tickUnit = new DateTickUnit(null) ;
    }
  }

  /**
   * <zh>õݼݷΧ.</zh>
   * <en>Returns the value range of a dataset.</en>
   *
   * @return NumberRange <zh>ݷΧ (Ϊnull).</zh>
   * <en>The value range of a dataset (never null).</en>
   */
  public NumberRange getDatasetRange() {
    if (getUserDefineDatasetRange() != null) {
      return getUserDefineDatasetRange() ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (getValueAxisType().equals(XYValueAxisType.XY_VALUE_AXIS_X)) {
      return getXAxisDataRange(dataset) ;
    }
    if (getValueAxisType().equals(XYValueAxisType.XY_VALUE_AXIS_Y)) {
      return getYAxisDataRange(dataset) ;
    }

    return NumberRange.RANGE_ZERO ;
  }

  /**
   * <zh>y귶Χ.</zh>
   * <en>Returns the data range of y axis.</en>
   *
   * @param dataset IMatrixDataset
   * @return NumberRange
   * @throws InvalidChartDataException
   */
  private static NumberRange getXAxisDataRange(IMatrixDataset dataset) {
    double minValue = Double.NaN ;
    double maxValue = Double.NaN ;

    if (dataset != null) {
      for (int i = 0 ; i < dataset.getRowCount() ; i++) {
        int currRowCols = dataset.getColumnCount(i) ;
        for (int j = 0 ; j < currRowCols ; j++) {
          Object data = dataset.getData(i, j) ;
          if (data != null) {
            if (!(data instanceof XYNumberChartData)) {
              throw new InvalidChartDataException(
                "Invalid chart data type,musts be 'XYNumberChartData'.") ;
            } else {
              double xValue = ((XYNumberChartData) data).getX() ;

              if (!Double.isNaN(xValue)) {
                double startXValue = xValue ;
                double endXValue = xValue ;

                if (Double.isNaN(minValue)) {
                  minValue = startXValue ;
                } else {
                  minValue = Math.min(minValue, startXValue) ;
                }

                if (Double.isNaN(maxValue)) {
                  maxValue = endXValue ;
                } else {
                  maxValue = Math.max(maxValue, endXValue) ;
                }
              }
            }
          }
        }
      }
    }

    if (Double.isNaN(minValue)) {
      minValue = 0 ;
    }
    if (Double.isNaN(maxValue)) {
      maxValue = 0 ;
    }

    return new NumberRange(minValue, maxValue) ;

  }

  /**
   * <zh>y귶Χ.</zh>
   * <en>Returns the data range of y axis.</en>
   *
   * @param dataset IMatrixDataset
   * @return NumberRange
   * @throws InvalidChartDataException
   */
  private static NumberRange getYAxisDataRange(IMatrixDataset dataset) throws
    InvalidChartDataException {
    return DatasetUtil.getDatasetYRangeForStackBarRender(dataset) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument;
   * false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!(obj instanceof StackedXYValueAxis)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    return super.hashCode() ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "StackedXYValueAxis]" ;
  }

}
