package com.ijchart.xychart.chartinfo ;

import java.awt.Graphics2D ;
import java.awt.geom.Point2D ;
import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.Collections ;
import java.util.List ;

import com.ijchart.xychart.constant.ChartShapeDrawType ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.web.IAreaFragment ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ڴ洢plotͼμ.</zh>
 * <en>Collection to be used to store the shapes in render class.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class PlotShapeCollection extends AbstractShapeCollection implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -4108103651569433580L ;

  /**
   * <zh>߿ͼ.</zh>
   * <en>The border shape.</en>
   */
  private IChartShape borderShape ;

  /**
   * <zh>ɫͼ.</zh>
   * <en>The background shape.</en>
   */
  private IChartShape backgroundShape ;

  /**
   * <zh>ͼƬͼ.</zh>
   * <en>The background image shape.</en>
   */
  private IChartShape backgroundImageShape ;

  /**
   * <zh>ûʱϢͼ.</zh>
   * <en>The no data message shape.</en>
   */
  private IChartShape noDataMessageShape ;

  /**
   * <zh>Xͼμ.Xж,Դ˼ɶList,ÿListһXͼμ.</zh>
   * <en>The X-axis shape list.If the chart has multiple X-axis,every X-axis has
   * a shape collection.</en>
   */
  private List xAxisShapeList ;

  /**
   * <zh>Yͼμ.Yж,Դ˼ɶList,ÿListһYͼμ.</zh>
   * <en>The Y-axis shape list.If the chart has multiple Y-axis,every Y-axis has
   * a shape collection.</en>
   */
  private List yAxisShapeList ;

  /**
   * <zh>renderͼμ.</zh>
   * <en>The list of render shapes.</en>
   */
  private List renderShapes ;

  /**
   * <zh>ע(annotation)ͼμ.</zh>
   * <en>The list of annotation shapes.</en>
   */
  private List annotationShapes ;

  /**
   * <zh>xʮ.</zh>
   * <en>The x-cross shape.</en>
   */
  private IChartShape xCrossShape ;

  /**
   * <zh>yʮ.</zh>
   * <en>The y-cross shape.</en>
   */
  private IChartShape yCrossShape ;

  /**
   * <zh>domain0ֵ.</zh>
   * <en>The zero line of domain axis.</en>
   */
  private IChartShape zeroDomainBaseline ;

  /**
   * <zh>value0ֵ.</zh>
   * <en>The zero line of value axis.</en>
   */
  private IChartShape zeroValueBaseline ;

  /**
   * <zh>޲.</zh>
   * <en>No argument constructor.</en>
   */
  public PlotShapeCollection() {
    this.borderShape = null ;
    this.backgroundShape = null ;
    this.backgroundImageShape = null ;
    this.noDataMessageShape = null ;
    this.xAxisShapeList = new ArrayList() ;
    this.yAxisShapeList = new ArrayList() ;
    this.renderShapes = new ArrayList() ;
    this.annotationShapes = new ArrayList() ;

    this.zeroDomainBaseline = null ;
    this.zeroValueBaseline = null ;
    this.xCrossShape = null ;
    this.yCrossShape = null ;
  }

  /**
   * <zh>رͼƬͼ.</zh>
   * <en>Returns the background shape.</en>
   *
   * @return IChartShape <zh>ͼƬͼ(Ϊnull).</zh>
   * <en>The background shape (maybe null).</en>
   */
  public IChartShape getBackgroundShape() {
    return this.backgroundShape ;
  }

  /**
   * <zh>ñͼƬͼ.</zh>
   * <en>Sets the background shape.</en>
   *
   * @param shape IChartShape <zh>ͼƬͼ(Ϊnull).</zh>
   * <en>The background shape (null permitted).</en>
   */
  public void setBackgroundShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.PLOT_BACKGROUND_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.backgroundShape = shape ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>رͼƬͼ.</zh>
   * <en>Returns the background image shape.</en>
   *
   * @return IChartShape <zh>ͼƬͼ(Ϊnull).</zh>
   * <en>The background image shape (maybe null).</en>
   */
  public IChartShape getBackgroundImageShape() {
    return this.backgroundImageShape ;
  }

  /**
   * <zh>ñͼƬͼ.</zh>
   * <en>Sets the background image shape.</en>
   *
   * @param shape IChartShape <zh>ͼƬͼ(Ϊnull).</zh>
   * <en>The background image shape (null permitted).</en>
   */
  public void setBackgroundImageShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.PLOT_BACKGROUND_IMAGE_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.backgroundImageShape = shape ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>ûʱϢͼ.</zh>
   * <en>Returns the no data message shape.</en>
   *
   * @return IChartShape <zh>ͼƬͼ(Ϊnull).</zh>
   * <en>The no data message shape (maybe null).</en>
   */
  public IChartShape getNoDataMessageShape() {
    return this.noDataMessageShape ;
  }

  /**
   * <zh>ûʱϢͼ.</zh>
   * <en>Sets the no data message shape.</en>
   *
   * @param shape IChartShape <zh>ͼƬͼ(Ϊnull).</zh>
   * <en>The no data message shape (null permitted).</en>
   */
  public void setNoDataMessageShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.PLOT_NO_DATA_MESSAGE_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.noDataMessageShape = shape ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>ñ߿ͼ.</zh>
   * <en>Sets the border shape.</en>
   *
   * @param borderShape IChartShape <zh>߿ͼ (Ϊnull).</zh>
   * <en>The border shape (null permitted).</en>
   */
  public void setBorderShape(IChartShape borderShape) {
    if (borderShape != null) {
      borderShape.setId(shapeIdSequence++) ;
      borderShape.setShapeDrawType(ChartShapeDrawType.PLOT_BORDER_SHAPE) ;
      borderShape.addChangeListener(this) ;
    }

    this.borderShape = borderShape ;
    fireChartShapeChanged(borderShape) ;
  }

  /**
   * <zh>ñ߿ͼ.</zh>
   * <en>The border shape.</en>
   *
   * @return IChartShape <zh>߿ͼ (Ϊnull).</zh>
   * <en>The border shape (maybe null).</en>
   */
  public IChartShape getBorderShape() {
    return this.borderShape ;
  }

  /**
   * <zh>Xͼμ.</zh>
   * <en>Add X-axis shapes collection.</en>
   * @param xAxisShape AxisShapeCollection <zh>Xͼμ.</zh>
   * <en>The X-axis shapes collection.</en>
   */
  public void addXAxisShape(AxisShapeCollection xAxisShape) {
    if (xAxisShape != null) {
      xAxisShape.addChangeListener(this) ;
    }

    this.xAxisShapeList.add(xAxisShape) ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>ɾеXͼμ.</zh>
   * <en>Remove all X-axis shape collection.</en>
   */
  public void clearXAxisShape() {
    for (int i = 0 ; i < this.xAxisShapeList.size() ; i++) {
      AxisShapeCollection axisShapes = (AxisShapeCollection)this.xAxisShapeList.get(i) ;
      if (axisShapes != null) {
        axisShapes.removeChangeListener(this) ;
      }
    }

    this.xAxisShapeList.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>ص0Xͼμ.</zh>
   * <en>Returns the first X-axis shapes collection.</en>
   *
   * @return AxisShapeCollection <zh>xͼμ.</zh>
   * <en>The axis shapes collection.</en>
   */
  public AxisShapeCollection getXAxisShape() {
    return getXAxisShape(0) ;
  }

  /**
   * <zh>ֵָXͼμ.</zh>
   * <en>Returns the X-axis shapes collection for the specified index.</en>
   *
   * @param index int <zh>ֵ.</zh>
   * <en>The specified index.</en>
   *
   * @return AxisShapeCollection
   */
  public AxisShapeCollection getXAxisShape(int index) {
    if (index >= 0 && index < this.xAxisShapeList.size()) {
      return (AxisShapeCollection)this.xAxisShapeList.get(index) ;
    }

    return null ;
  }

  /**
   * <zh>Xͼμ.˼ϲ޸.</zh>
   * <en>Get the X-axis shape list,The returned list should be unmodifiable.</en>
   * @return List <zh>еXͼμ</zh>
   * <en>The X-axis shapes collection.</en>
   */
  public List getXAxisShapeList() {
    return Collections.unmodifiableList(this.xAxisShapeList) ;
  }

  /**
   * <zh>Yͼμ.</zh>
   * <en>Add Y-axis shapes collection.</en>
   * @param yAxisShape AxisShapeCollection <zh>Yͼμ.</zh>
   * <en>The Y-axis shapes collection.</en>
   */
  public void addYAxisShape(AxisShapeCollection yAxisShape) {
    if (yAxisShape != null) {
      yAxisShape.addChangeListener(this) ;
    }

    this.yAxisShapeList.add(yAxisShape) ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>ɾеYͼ.</zh>
   * <en>Remove all shape in Y-axis shape collection.</en>
   */
  public void clearYAxisShape() {
    for (int i = 0 ; i < this.yAxisShapeList.size() ; i++) {
      AxisShapeCollection axisShapes = (AxisShapeCollection)this.yAxisShapeList.get(i) ;
      if (axisShapes != null) {
        axisShapes.removeChangeListener(this) ;
      }
    }

    this.yAxisShapeList.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>ص0Yͼμ.</zh>
   * <en>Returns the first Y-axis shapes collection.</en>
   *
   * @return AxisShapeCollection <zh>yͼμ.</zh>
   * <en>The axis shapes collection.</en>
   */
  public AxisShapeCollection getYAxisShape() {
    return getYAxisShape(0) ;
  }

  /**
   * <zh>ֵָYͼμ.</zh>
   * <en>Returns the Y-axis shapes collection for the specified index.</en>
   *
   * @param index int <zh>ֵ.</zh>
   * <en>The specified index.</en>
   *
   * @return AxisShapeCollection
   */
  public AxisShapeCollection getYAxisShape(int index) {
    if (index >= 0 && index < this.yAxisShapeList.size()) {
      return (AxisShapeCollection)this.yAxisShapeList.get(index) ;
    }

    return null ;
  }

  /**
   * <zh>Yͼμ.˼ϲ޸.</zh>
   * <en>Get the Y-axis shape list,The returned list should be unmodifiable.</en>
   * @return List <zh>Yͼμ</zh>
   * <en>The Y-axis shapes collection.</en>
   */
  public List getYAxisShapeList() {
    return Collections.unmodifiableList(this.yAxisShapeList) ;
  }

  /**
   * <zh>һrenderͼ.</zh>
   * <en>Add a render shape.</en>
   *
   * @param renderShapes RenderShapeCollection <zh>renderͼμ.</zh>
   * <en>A render shapes collection.</en>
   */
  public void addRenderShape(RenderShapeCollection renderShapes) {
    if (renderShapes != null) {
      renderShapes.addChangeListener(this) ;
    }

    this.renderShapes.add(renderShapes) ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>ص0renderͼμ.</zh>
   * <en>Returns the first render shapes collection.</en>
   *
   * @return RenderShapeCollection <zh>renderͼμ.</zh>
   * <en>The render shapes collection.</en>
   */
  public RenderShapeCollection getRenderShapes() {
    return getRenderShapes(0) ;
  }

  /**
   * <zh>ָrenderͼμ.</zh>
   * <en>Returns the render shapes collection of the specified index.</en>
   *
   * @param index int <zh>ֵ.</zh>
   * <en>The index.</en>
   *
   * @return RenderShapeCollection <zh>renderͼμ.</zh>
   * <en>The render shapes collection.</en>
   */
  public RenderShapeCollection getRenderShapes(int index) {
    return (RenderShapeCollection)this.renderShapes.get(index) ;
  }

  /**
   * <zh>renderͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of render shapes and is unmodifiable.</en>
   *
   * @return List <zh>renderͼμ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of render shapes and is unmodifiable.</en>
   */
  public List getAllRenderShapes() {
    return Collections.unmodifiableList(this.renderShapes) ;
  }

  /**
   * <zh>ɾеrenderͼ.</zh>
   * <en>Remove all render shapes.</en>
   */
  public void clearRenderShapes() {
    for (int i = 0 ; i < this.renderShapes.size() ; i++) {
      RenderShapeCollection renderShapes = (RenderShapeCollection)this.renderShapes.get(i) ;
      if (renderShapes != null) {
        renderShapes.removeChangeListener(this) ;
      }
    }

    this.renderShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һannotationͼ.</zh>
   * <en>Add an annotation shape collection.</en>
   *
   * @param annotationShapes AnnotationShapeCollection <zh>annotationͼμ.</zh>
   * <en>A annotation shapes collection.</en>
   */
  public void addAnnotationShape(AnnotationShapeCollection annotationShapes) {
    if (annotationShapes != null) {
      annotationShapes.addChangeListener(this) ;
    }

    this.annotationShapes.add(annotationShapes) ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>annotationͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of annotation shapes and is unmodifiable.</en>
   *
   * @return List <zh>annotationͼμ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of annotation shapes and is unmodifiable.</en>
   */
  public List getAnnotationShapes() {
    return Collections.unmodifiableList(this.annotationShapes) ;
  }

  /**
   * <zh>ɾеannotationͼ.</zh>
   * <en>Remove all annotation shapes.</en>
   */
  public void clearAnnotationShapes() {
    for (int i = 0 ; i < this.annotationShapes.size() ; i++) {
      AnnotationShapeCollection annoShapes = (AnnotationShapeCollection)this.annotationShapes.get(i) ;
      if (annoShapes != null) {
        annoShapes.removeChangeListener(this) ;
      }
    }

    this.annotationShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>xʮ.</zh>
   * <en>Returns the x-cross shape.</en>
   *
   * @return IChartShape <zh>xʮ (Ϊnull).</zh>
   * <en>The x-cross shape (maybe null).</en>
   */
  public IChartShape getXCrossShape() {
    return this.xCrossShape ;
  }

  /**
   * <zh>xʮ.</zh>
   * <en>Sets the x-cross shape.</en>
   *
   * @param shape IChartShape <zh>xʮ (Ϊnull).</zh>
   * <en>The x-cross shape (null permitted).</en>
   */
  public void setXCrossShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.PLOT_X_CROSS_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.xCrossShape = shape ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>yʮ.</zh>
   * <en>Returns the y-cross shape.</en>
   *
   * @return IChartShape <zh>yʮ (Ϊnull).</zh>
   * <en>The y-cross shape (maybe null).</en>
   */
  public IChartShape getYCrossShape() {
    return this.yCrossShape ;
  }

  /**
   * <zh>yʮ.</zh>
   * <en>Sets the y-cross shape.</en>
   *
   * @param shape IChartShape <zh>yʮ (Ϊnull).</zh>
   * <en>The y-cross shape (null permitted).</en>
   */
  public void setYCrossShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.PLOT_Y_CROSS_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.yCrossShape = shape ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>domain0ֵ.</zh>
   * <en>Returns the zero line of domain axis.</en>
   *
   * @return IChartShape <zh>domain0ֵ (Ϊnull).</zh>
   * <en>The zero line of domain axis (maybe null).</en>
   */
  public IChartShape getZeroDomainBaseline() {
    return this.zeroDomainBaseline ;
  }

  /**
   * <zh>domain0ֵ.</zh>
   * <en>Sets the zero line of domain axis.</en>
   *
   * @param shape IChartShape <zh>domain0ֵ (Ϊnull).</zh>
   * <en>The zero line of domain axis (null permitted).</en>
   */
  public void setZeroDomainBaseline(IChartShape shape) {
    this.zeroDomainBaseline = shape ;
  }

  /**
   * <zh>value0ֵ.</zh>
   * <en>Returns the zero line of value axis.</en>
   *
   * @return IChartShape <zh>value0ֵ (Ϊnull).</zh>
   * <en>The zero line of value axis (maybe null).</en>
   */
  public IChartShape getZeroValueBaseline() {
    return this.zeroValueBaseline ;
  }

  /**
   * <zh>value0ֵ.</zh>
   * <en>Sets the zero line of value axis.</en>
   *
   * @param shape IChartShape <zh>value0ֵ (Ϊnull).</zh>
   * <en>The zero line of value axis (null permitted).</en>
   */
  public void setZeroValueBaseline(IChartShape shape) {
    this.zeroValueBaseline = shape ;
  }

  /**
   * <zh>λƼÿһͼ.</zh>
   * <en>Drawing all ChartShape in this collection.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (this.backgroundShape != null) {
      this.backgroundShape.draw(g2) ;
    }
    if (this.backgroundImageShape != null) {
      this.backgroundImageShape.draw(g2) ;
    }
    if (this.noDataMessageShape != null) {
      this.noDataMessageShape.draw(g2) ;
    }

    if (this.borderShape != null) {
      this.borderShape.draw(g2) ;
    }

    for (int i = 0 ; i < xAxisShapeList.size() ; i++) {
      AxisShapeCollection axisShapes = (AxisShapeCollection) xAxisShapeList.get(i) ;
      if (axisShapes != null) {
        axisShapes.draw(g2) ;
      }
    }

    for (int i = 0 ; i < yAxisShapeList.size() ; i++) {
      AxisShapeCollection axisShapes = (AxisShapeCollection) yAxisShapeList.get(i) ;
      if (axisShapes != null) {
        axisShapes.draw(g2) ;
      }
    }

    for (int i = 0 ; i < this.renderShapes.size() ; i++) {
      RenderShapeCollection renderShapes = (RenderShapeCollection)this.renderShapes.get(i) ;
      if (renderShapes != null) {
        renderShapes.draw(g2) ;
      }
    }

    for (int i = 0 ; i < this.annotationShapes.size() ; i++) {
      AnnotationShapeCollection annotationShapes =
        (AnnotationShapeCollection)this.annotationShapes.get(i) ;
      if (annotationShapes != null) {
        annotationShapes.draw(g2) ;
      }
    }

    if (this.zeroDomainBaseline != null) {
      this.zeroDomainBaseline.draw(g2) ;
    }
    if (this.zeroValueBaseline != null) {
      this.zeroValueBaseline.draw(g2) ;
    }

    if (this.xCrossShape != null) {
      this.xCrossShape.draw(g2) ;
    }
    if (this.yCrossShape != null) {
      this.yCrossShape.draw(g2) ;
    }
  }

  /**
   * <zh>ƶͼεԭ㵽(x,y).</zh>
   * <en>Translates the origin of the all shape in this chart to the point (x,&nbsp;y)
   * in the current coordinate system. </en>
   *
   * @param  x double <zh>x.</zh>
   * <en>The x coordinate.</en>
   * @param  y double <zh>y.</zh>
   * <en>The y coordinate.</en>
   */
  public void translate(double x, double y) {
    if (x == 0 && y == 0) {
      return ;
    }

    super.translate(x, y) ;
    if (this.borderShape != null) {
      this.borderShape.translate(x, y) ;
    }
    if (this.backgroundShape != null) {
      this.backgroundShape.translate(x, y) ;
    }
    if (this.backgroundImageShape != null) {
      this.backgroundImageShape.translate(x, y) ;
    }
    if (this.noDataMessageShape != null) {
      this.noDataMessageShape.translate(x, y) ;
    }

    for (int i = 0 ; i < xAxisShapeList.size() ; i++) {
      AxisShapeCollection axisShapes = (AxisShapeCollection) xAxisShapeList.get(i) ;
      if (axisShapes != null) {
        axisShapes.translate(x, y) ;
      }
    }

    for (int i = 0 ; i < yAxisShapeList.size() ; i++) {
      AxisShapeCollection axisShapes = (AxisShapeCollection) yAxisShapeList.get(i) ;
      if (axisShapes != null) {
        axisShapes.translate(x, y) ;
      }
    }

    for (int i = 0 ; i < this.renderShapes.size() ; i++) {
      RenderShapeCollection shape = (RenderShapeCollection)this.renderShapes.get(i) ;
      if (shape != null) {
        shape.translate(x, y) ;
      }
    }

    for (int i = 0 ; i < this.annotationShapes.size() ; i++) {
      AnnotationShapeCollection shape = (AnnotationShapeCollection)this.annotationShapes.get(i) ;
      if (shape != null) {
        shape.translate(x, y) ;
      }
    }

    if (this.zeroDomainBaseline != null) {
      this.zeroDomainBaseline.translate(x, y) ;
    }
    if (this.zeroValueBaseline != null) {
      this.zeroValueBaseline.translate(x, y) ;
    }

    if (this.xCrossShape != null) {
      this.xCrossShape.translate(x, y) ;
    }
    if (this.yCrossShape != null) {
      this.yCrossShape.translate(x, y) ;
    }
  }

  /**
   * <zh>еͼεļ.</zh>
   * <en>Remove all the shapes of the listener.</en>
   */
  public void clearShapesListener() {
    if (this.borderShape != null) {
      this.borderShape.removeAllChangeListener() ;
    }
    if (this.backgroundShape != null) {
      this.backgroundShape.removeAllChangeListener() ;
    }
    if (this.backgroundImageShape != null) {
      this.backgroundImageShape.removeAllChangeListener() ;
    }
    if (this.noDataMessageShape != null) {
      this.noDataMessageShape.removeAllChangeListener() ;
    }

    for (int i = 0 ; i < xAxisShapeList.size() ; i++) {
      AxisShapeCollection axisShapes = (AxisShapeCollection) xAxisShapeList.get(i) ;
      if (axisShapes != null) {
        axisShapes.clearShapesListener() ;
      }
    }

    for (int i = 0 ; i < yAxisShapeList.size() ; i++) {
      AxisShapeCollection axisShapes = (AxisShapeCollection) yAxisShapeList.get(i) ;
      if (axisShapes != null) {
        axisShapes.clearShapesListener() ;
      }
    }

    for (int i = 0 ; i < this.renderShapes.size() ; i++) {
      RenderShapeCollection shape = (RenderShapeCollection)this.renderShapes.get(i) ;
      if (shape != null) {
        shape.clearShapesListener() ;
      }
    }

    for (int i = 0 ; i < this.annotationShapes.size() ; i++) {
      AnnotationShapeCollection shape = (AnnotationShapeCollection)this.annotationShapes.get(i) ;
      if (shape != null) {
        shape.clearShapesListener() ;
      }
    }

    if (this.zeroDomainBaseline != null) {
      this.zeroDomainBaseline.removeAllChangeListener() ;
    }
    if (this.zeroValueBaseline != null) {
      this.zeroValueBaseline.removeAllChangeListener() ;
    }

    if (this.xCrossShape != null) {
      this.xCrossShape.removeAllChangeListener() ;
    }
    if (this.yCrossShape != null) {
      this.yCrossShape.removeAllChangeListener() ;
    }
  }

  /**
   * <zh>еͼΪѡ״̬.</zh>
   * <en>Sets all shapes to unselected.</en>
   */
  public void clearSelectedShapes() {
    clearSelectedShape(this.borderShape) ;
    clearSelectedShape(this.backgroundShape) ;
    clearSelectedShape(this.backgroundImageShape) ;
    clearSelectedShape(this.noDataMessageShape) ;

    for (int i = 0 ; i < xAxisShapeList.size() ; i++) {
      AxisShapeCollection axisShapes = (AxisShapeCollection) xAxisShapeList.get(i) ;
      if (axisShapes != null) {
        axisShapes.clearAxisLineShapes() ;
      }
    }

    for (int i = 0 ; i < yAxisShapeList.size() ; i++) {
      AxisShapeCollection axisShapes = (AxisShapeCollection) yAxisShapeList.get(i) ;
      if (axisShapes != null) {
        axisShapes.clearSelectedShapes() ;
      }
    }

    for (int i = 0 ; i < this.renderShapes.size() ; i++) {
      RenderShapeCollection shape = (RenderShapeCollection)this.renderShapes.get(i) ;
      if (shape != null) {
        shape.clearSelectedShapes() ;
      }
    }

    for (int i = 0 ; i < this.annotationShapes.size() ; i++) {
      AnnotationShapeCollection shape = (AnnotationShapeCollection)this.annotationShapes.get(i) ;
      if (shape != null) {
        shape.clearSelectedShapes() ;
      }
    }

    clearSelectedShape(this.zeroDomainBaseline) ;
    clearSelectedShape(this.zeroValueBaseline) ;

    clearSelectedShape(this.xCrossShape) ;
    clearSelectedShape(this.yCrossShape) ;
  }

  /**
   * <zh>ذpointͼ.</zh>
   * <en>Returns the {@link IChartShape} whose area contains the specified point.</en>
   *
   * @param point Point2D <zh>ҪԵĵ (Ϊnull).</zh>
   * <en>The point (null not permitted).</en>
   * @param drawType ChartShapeDrawType <zh>ͼεĻ (Ϊnull).</zh>
   * <en>The shape draw shape (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ (Ϊnull).</zh>
   * <en>The chart shape (maybe null).</en>
   */
  public IChartShape findChartShape(Point2D point, ChartShapeDrawType drawType) {
    if (point == null) {
      throw new IllegalArgumentException("Null 'point' argument") ;
    }
    if (getDrawArea() != null && !getDrawArea().contains(point)) {
      return null ;
    }

    IChartShape shape = findShape(this.annotationShapes, point, drawType) ;
    if (shape != null) {
      return shape ;
    }

    shape = findShape(this.renderShapes, point, drawType) ;
    if (shape != null) {
      return shape ;
    }

    shape = findShape(this.yAxisShapeList, point, drawType) ;
    if (shape != null) {
      return shape ;
    }

    shape = findShape(this.xAxisShapeList, point, drawType) ;
    if (shape != null) {
      return shape ;
    }

    boolean flag = drawType == null || drawType.equals(ChartShapeDrawType.ANY_SHAPE) ;
    if (flag || drawType.equals(ChartShapeDrawType.PLOT_ZERO_DOMAIN_BASE_LINE_SHAPE)) {
      if (this.zeroDomainBaseline != null && this.zeroDomainBaseline.contains(point)) {
        return this.zeroDomainBaseline ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.PLOT_ZERO_VALUE_BASE_LINE_SHAPE)) {
      if (this.zeroValueBaseline != null && this.zeroValueBaseline.contains(point)) {
        return this.zeroValueBaseline ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.PLOT_X_CROSS_SHAPE)) {
      if (this.xCrossShape != null && this.xCrossShape.contains(point)) {
        return this.xCrossShape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.PLOT_Y_CROSS_SHAPE)) {
      if (this.yCrossShape != null && this.yCrossShape.contains(point)) {
        return this.yCrossShape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.PLOT_BORDER_SHAPE)) {
      if (this.borderShape != null && this.borderShape.contains(point)) {
        return this.borderShape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.PLOT_NO_DATA_MESSAGE_SHAPE)) {
      if (this.noDataMessageShape != null && this.noDataMessageShape.contains(point)) {
        return this.noDataMessageShape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.PLOT_BACKGROUND_IMAGE_SHAPE)) {
      if (this.backgroundImageShape != null && this.backgroundImageShape.contains(point)) {
        return this.backgroundImageShape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.PLOT_BACKGROUND_SHAPE)) {
      if (this.backgroundShape != null && this.backgroundShape.contains(point)) {
        return this.backgroundShape ;
      }
    }

    return null ;
  }

  /**
   * <zh>ͼεȵArea.</zh>
   * <en>Returns the hotspot areas.</en>
   *
   * @param drawTypes List <zh>ʾȵͼ.</zh>
   * <en>The types of hotspot shape.</en>
   * @param fragment IAreaFragment <zh>ʵֽӿ{@link IAreaFragment}Ķ,(Ϊnull)</zh>
   * <en>The {@link IAreaFragment} object (maybe null).</en>
   *
   * @return List <zh>ͼεȵArea (Ϊnull).</zh>
   * <en>The hotspot areas (never null).</en>
   */
  public List getHotspotAreas(List drawTypes, IAreaFragment fragment) {
    if (drawTypes == null) {
      throw new IllegalArgumentException("Null 'drawTypes' argument.") ;
    }

    List result = new ArrayList() ;
    if (drawTypes.contains(ChartShapeDrawType.PLOT_BACKGROUND_SHAPE)) {
      addHotspotArea(result, this.backgroundShape, fragment,
                     ChartShapeDrawType.PLOT_BACKGROUND_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.PLOT_BACKGROUND_IMAGE_SHAPE)) {
      addHotspotArea(result, this.backgroundImageShape, fragment,
                     ChartShapeDrawType.PLOT_BACKGROUND_IMAGE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.PLOT_NO_DATA_MESSAGE_SHAPE)) {
      addHotspotArea(result, this.noDataMessageShape, fragment,
                     ChartShapeDrawType.PLOT_NO_DATA_MESSAGE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.PLOT_BORDER_SHAPE)) {
      addHotspotArea(result, this.borderShape, fragment, ChartShapeDrawType.PLOT_BORDER_SHAPE) ;
    }

    for (int i = 0 ; i < this.xAxisShapeList.size() ; i++) {
      AxisShapeCollection xAxisShapes = (AxisShapeCollection)this.xAxisShapeList.get(i) ;
      if (xAxisShapes != null) {
        result.addAll(xAxisShapes.getHotspotAreas(drawTypes, fragment)) ;
      }
    }
    for (int i = 0 ; i < this.yAxisShapeList.size() ; i++) {
      AxisShapeCollection yAxisShapes = (AxisShapeCollection)this.yAxisShapeList.get(i) ;
      if (yAxisShapes != null) {
        result.addAll(yAxisShapes.getHotspotAreas(drawTypes, fragment)) ;
      }
    }
    for (int i = 0 ; i < this.renderShapes.size() ; i++) {
      RenderShapeCollection renderShapes = (RenderShapeCollection)this.renderShapes.get(i) ;
      if (renderShapes != null) {
        result.addAll(renderShapes.getHotspotAreas(drawTypes, fragment)) ;
      }
    }
    for (int i = 0 ; i < this.annotationShapes.size() ; i++) {
      AnnotationShapeCollection annotationShapes =
        (AnnotationShapeCollection)this.annotationShapes.get(i) ;
      if (annotationShapes != null) {
        result.addAll(annotationShapes.getHotspotAreas(drawTypes, fragment)) ;
      }
    }

    if (drawTypes.contains(ChartShapeDrawType.PLOT_ZERO_DOMAIN_BASE_LINE_SHAPE)) {
      addHotspotArea(result, this.zeroDomainBaseline, fragment,
                     ChartShapeDrawType.PLOT_ZERO_DOMAIN_BASE_LINE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.PLOT_ZERO_VALUE_BASE_LINE_SHAPE)) {
      addHotspotArea(result, this.zeroDomainBaseline, fragment,
                     ChartShapeDrawType.PLOT_ZERO_VALUE_BASE_LINE_SHAPE) ;
    }

    if (drawTypes.contains(ChartShapeDrawType.PLOT_X_CROSS_SHAPE)) {
      addHotspotArea(result, this.xCrossShape, fragment, ChartShapeDrawType.PLOT_X_CROSS_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.PLOT_Y_CROSS_SHAPE)) {
      addHotspotArea(result, this.yCrossShape, fragment, ChartShapeDrawType.PLOT_Y_CROSS_SHAPE) ;
    }

    return result ;
  }

  /**
   * <zh>clone.</zh>
   * <en>Returns a clone of the axis.</en>
   *
   * @return A clone.
   * @throws CloneNotSupportedException if some component of the axis does
   *         not support cloning.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>hashCode.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = HashcodeUtil.hashCode(this.borderShape) ;
    result = 29 * result + HashcodeUtil.hashCode(this.backgroundShape) ;
    result = 29 * result + HashcodeUtil.hashCode(this.backgroundImageShape) ;
    result = 29 * result + HashcodeUtil.hashCode(this.noDataMessageShape) ;
    result = 29 * result + HashcodeUtil.hashCode(xAxisShapeList) ;
    result = 29 * result + HashcodeUtil.hashCode(yAxisShapeList) ;
    result = 29 * result + HashcodeUtil.hashCode(this.renderShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.annotationShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.zeroDomainBaseline) ;
    result = 29 * result + HashcodeUtil.hashCode(this.zeroValueBaseline) ;
    result = 29 * result + HashcodeUtil.hashCode(this.xCrossShape) ;
    result = 29 * result + HashcodeUtil.hashCode(this.yCrossShape) ;

    return result ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this axis for equality with another object.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>The object (null permitted).</en>
   *
   * @return true or false.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof RenderShapeCollection)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    PlotShapeCollection that = (PlotShapeCollection) obj ;
    if (!EqualsUtil.equals(this.borderShape, that.borderShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.backgroundShape, that.backgroundShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.backgroundImageShape, that.backgroundImageShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.noDataMessageShape, that.noDataMessageShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.xAxisShapeList, that.xAxisShapeList)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.yAxisShapeList, that.yAxisShapeList)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.renderShapes, that.renderShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.annotationShapes, that.annotationShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.zeroDomainBaseline, that.zeroDomainBaseline)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.zeroValueBaseline, that.zeroValueBaseline)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.xCrossShape, that.xCrossShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.yCrossShape, that.yCrossShape)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  A string representation of the object.
   */
  public String toString() {
    return "[borderShape=" + this.borderShape
      + ",backgroundShape=" + this.backgroundShape
      + ",noDataMessageShape=" + this.noDataMessageShape
      + ",backgroundImageShape=" + this.backgroundImageShape
      + ",xAxisShapes size=" + this.xAxisShapeList.size()
      + ",yAxisShapes size=" + this.yAxisShapeList.size()
      + ",renderShapes size=" + this.renderShapes.size()
      + ",annotationShapes size=" + this.annotationShapes.size()
      + ",zeroDomainBaseline=" + this.zeroDomainBaseline
      + ",zeroValueBaseline=" + this.zeroValueBaseline
      + ",xCrossShape=" + this.xCrossShape
      + ",yCrossShape=" + this.yCrossShape
      + "]" ;
  }

}
