package com.ijchart.xychart.constant ;

import java.io.Serializable ;

import com.ijchart.xychart.util.EqualsUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ֱķʽ,ˡ׶˺ʹֱ.</zh>
 * <en>An enumeration of the vertical alignment types (TOP,BOTTOM and VERTICAL_CENTER).</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public final class VerticalAlignment implements Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 1774958053556949937L ;

  /**
   * <zh>.</zh>
   * <en>Top alignment.</en>
   */
  public static final VerticalAlignment TOP = new VerticalAlignment("VerticalAlignment.TOP") ;

  /**
   * <zh>׶.</zh>
   * <en>Bottom alignment.</en>
   */
  public static final VerticalAlignment BOTTOM = new VerticalAlignment("VerticalAlignment.BOTTOM") ;

  /**
   * <zh>ֱ.</zh>
   * <en>Center alignment. </en>
   */
  public static final VerticalAlignment CENTER = new VerticalAlignment("VerticalAlignment.CENTER") ;

  /**
   * <zh>.</zh>
   * <en>The name.</en>
   */
  private String name ;

  /**
   * <zh>˽йֹʵ.</zh>
   * <en>Private constructor to prevent instance.</en>
   *
   * @param name <zh>.</zh>
   * <en>The name.</en>
   */
  private VerticalAlignment(final String name) {
    this.name = name ;
  }

  /**
   * <zh>෴Ķ뷽ʽ.</zh>
   * <en>Returns the opposite vertical alignment.</en>
   *
   * @param align VerticalAlignment <zh>ֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment.</en>
   *
   * @return VerticalAlignment <zh>෴Ĵֱ뷽ʽ (Ϊnull).</zh>
   * <en>The opposite vertical alignment (never null).</en>
   */
  public static VerticalAlignment getOpposite(VerticalAlignment align) {
    if (align == null) {
      throw new IllegalArgumentException("Null 'align' argument.") ;
    }

    if (align.equals(VerticalAlignment.TOP)) {
      return VerticalAlignment.BOTTOM ;
    } else if (align.equals(VerticalAlignment.CENTER)) {
      return VerticalAlignment.CENTER ;
    } else if (align.equals(VerticalAlignment.BOTTOM)) {
      return VerticalAlignment.TOP ;
    }

    throw new IllegalArgumentException("Not available 'align' argument.") ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representing the object.</en>
   *
   * @return The string.
   */
  public String toString() {
    return this.name ;
  }

  /**
   * <zh>Ƿ,ȷtrue,򷵻false.</zh>
   * <en>Returns true if this object is equal to the specified object, and false otherwise.</en>
   *
   * @param obj <zh>ҪȽϵĶ (Ϊnull)</zh>
   * <en>The object (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(final Object obj) {

    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof VerticalAlignment)) {
      return false ;
    }

    VerticalAlignment that = (VerticalAlignment) obj ;
    if (!EqualsUtil.equals(this.name, that.name)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return <zh>hashcodeֵ.</zh>
   * <en>The hashcode.</en>
   */
  public int hashCode() {
    return this.name.hashCode() ;
  }

}
