package com.ijchart.xychart.data ;

import java.io.Serializable ;
import java.text.DateFormat ;
import java.text.NumberFormat ;
import java.text.SimpleDateFormat ;
import java.util.Date ;

import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ṩ{@link IChartData}ĳʵ.</zh>
 * <en>The abstract class for all {@link IChartData} object.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class AbstractChartData implements IChartData, Serializable, Cloneable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -2256608502372530107L ;

  /**
   * <zh>ʽ,ʽΪ"-- ʱ::"</zh>
   * <en>The date mask of long type.</en>
   */
  public static final String DATE_MASK_LONG = "yyyy-MM-dd HH:mm:ss" ;

  /**
   * <zh>ʽڵ.</zh>
   * <en>The formater for date object.</en>
   */
  private DateFormat dateFormat ;

  /**
   * <zh>ʽֵ.</zh>
   * <en>The formater for number object.</en>
   */
  private NumberFormat numberFormat ;

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   */
  public AbstractChartData() {
    this.dateFormat = new SimpleDateFormat(DATE_MASK_LONG) ;

    this.numberFormat = NumberFormat.getNumberInstance() ;
    this.numberFormat.setMaximumFractionDigits(3) ;
  }

  /**
   * <zh>ظʽڵ.</zh>
   * <en>Returns the formater for date object.</en>
   *
   * @return DateFormat <zh>ʽڵ.</zh>
   * <en>The formater for date object.</en>
   */
  public DateFormat getDateFormat() {
    return this.dateFormat ;
  }

  /**
   * <zh>øʽڵ.</zh>
   * <en>Sets the formater for date object.</en>
   *
   * @param dateFormat DateFormat <zh>ʽڵ (Ϊnull).</zh>
   * <en>The formater for date object (null permitted).</en>
   */
  public void setDateFormat(DateFormat dateFormat) {
    this.dateFormat = dateFormat ;
  }

  /**
   * <zh>ظʽֵ.</zh>
   * <en>Returns the formater for number object.</en>
   *
   * @return NumberFormat <zh>ʽֵ.</zh>
   * <en>The formater for number object.</en>
   */
  public NumberFormat getNumberFormat() {
    return this.numberFormat ;
  }

  /**
   * <zh>øʽֵ.</zh>
   * <en>Sets the formater for number object.</en>
   *
   * @param numberFormat NumberFormat <zh>ʽֵ.</zh>
   * <en>The formater for number object.</en>
   */
  public void setNumberFormat(NumberFormat numberFormat) {
    this.numberFormat = numberFormat ;
  }

  /**
   * <zh>رǩַ.</zh>
   * <en>Returns the label text.</en>
   *
   * @return String <zh>ǩַ (Ϊnull).</zh>
   * <en>The label text (maybe null).</en>
   */
  public String getLabelText() {
    return null ;
  }

  /**
   * <zh>ͼȵareaĺ.</zh>
   * <en>Returns the function arguments of hotspot area.</en>
   *
   * @return String <zh> (Ϊnull).</zh>
   * <en>The function arguments (maybe null).</en>
   */
  public String getAreaFunctionArguments() {
    return null ;
  }

  /**
   * <zh>ͼȵareaurl.</zh>
   * <en>Returns the url for hotspot area.</en>
   *
   * @return String <zh>url (Ϊnull).</zh>
   * <en>The url fragment (maybe null).</en>
   */
  public String getAreaURLFragment() {
    return null ;
  }

  /**
   * <zh>ʽ.</zh>
   * <en>Format date object.</en>
   *
   * @param date Date
   * @return String <zh>ʽַ.</zh>
   * <en>The formatted string.</en>
   */
  protected String formatDate(Date date) {
    if (date == null) {
      return null ;
    }
    if (this.dateFormat == null) {
      return date.toString() ;
    }

    String str = this.dateFormat.format(date) ;
    if (str == null) {
      return str ;
    }

    str = str.trim() ;
    if (str.indexOf(" ") >= 0) {
      while (str.endsWith(":00")) {
        str = str.substring(0, str.length() - 3) ;
      }
    }

    if (str.endsWith(" 00")) {
      str = str.substring(0, str.length() - 3) ;
    }

    if (str.endsWith(" ")) {
      str = str.substring(0, str.length() - 1) ;
    }

    return str ;
  }

  /**
   * <zh>ʽ,ɺ.</zh>
   * <en>Format date object for function argument.</en>
   *
   * @param date Date
   * @return String <zh>ʽַ.</zh>
   * <en>The formatted string.</en>
   */
  protected String formatDateForFunction(Date date) {
    if (date == null) {
      return "''" ;
    }
    if (this.dateFormat == null) {
      return "'" + date.toString() + "'" ;
    }

    String str = this.dateFormat.format(date) ;
    if (str == null) {
      return "''" ;
    }

    str = str.trim() ;
    if (str.indexOf(" ") >= 0) {
      while (str.endsWith(":00")) {
        str = str.substring(0, str.length() - 3) ;
      }
    }

    if (str.endsWith(" 00")) {
      str = str.substring(0, str.length() - 3) ;
    }

    if (str.endsWith(" ")) {
      str = str.substring(0, str.length() - 1) ;
    }

    return "'" + str + "'" ;
  }

  /**
   * <zh>ʽ.</zh>
   * <en>Format number object.</en>
   *
   * @param value double
   * @return String <zh>ʽַ.</zh>
   * <en>The formatted string.</en>
   */
  protected String formatNumber(double value) {
    if (Double.isNaN(value)) {
      return "" ;
    }

    return formatNumber(new Double(value)) ;
  }

  /**
   * <zh>ʽ.</zh>
   * <en>Format number object.</en>
   *
   * @param num Number
   * @return String <zh>ʽַ.</zh>
   * <en>The formatted string.</en>
   */
  protected String formatNumber(Number num) {
    if (num == null) {
      return null ;
    }
    if (this.numberFormat == null) {
      return num.toString() ;
    }

    String str = this.numberFormat.format(num.doubleValue()) ;
    if (str == null) {
      return str ;
    }

    if (str.indexOf(".") >= 0) {
      while (str.endsWith("0")) {
        str = str.substring(0, str.length() - 1) ;
      }
    }

    if (str.endsWith(".")) {
      str = str.substring(0, str.length() - 1) ;
    }

    if (str.length() == 0) {
      str = "0" ;
    }
    str = str.replaceAll(",", "") ;

    return str ;
  }

  /**
   * <zh>ʽ,ɺ.</zh>
   * <en>Format number object for function argument.</en>
   *
   * @param value double
   * @return String <zh>ʽַ.</zh>
   * <en>The formatted string.</en>
   */
  protected String formatNumberForFunction(double value) {
    if (Double.isNaN(value)) {
      return "''" ;
    }

    return formatNumberForFunction(new Double(value)) ;
  }

  /**
   * <zh>ʽ,ɺ.</zh>
   * <en>Format number object for function argument.</en>
   *
   * @param num Number
   * @return String <zh>ʽַ.</zh>
   * <en>The formatted string.</en>
   */
  protected String formatNumberForFunction(Number num) {
    if (num == null) {
      return "''" ;
    }
    if (this.numberFormat == null) {
      return num.toString() ;
    }

    String str = this.numberFormat.format(num.doubleValue()) ;
    if (str == null) {
      return "''" ;
    }

    if (str.indexOf(".") >= 0) {
      while (str.endsWith("0")) {
        str = str.substring(0, str.length() - 1) ;
      }
    }

    if (str.endsWith(".")) {
      str = str.substring(0, str.length() - 1) ;
    }

    if (str.length() == 0) {
      str = "0" ;
    }
    str = str.replaceAll(",", "") ;

    return str ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof AbstractChartData)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    AbstractChartData that = (AbstractChartData) obj ;
    if (!EqualsUtil.equals(this.dateFormat, that.dateFormat)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.numberFormat, that.numberFormat)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double result = super.hashCode() ;
    result = result * 37 + HashcodeUtil.hashCode(this.dateFormat) ;
    result = result * 37 + HashcodeUtil.hashCode(this.numberFormat) ;

    return (int) result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[dateFormat=" + this.dateFormat
      + ",numberFormat=" + this.numberFormat
      + "]" ;
  }

}
