package com.ijchart.xychart.data ;

import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.Arrays ;
import java.util.Collections ;
import java.util.Iterator ;
import java.util.List ;

import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>BoxAndWhisker(ͼ)ͼݶ,ⲿǲ޸ĵ.</zh>
 * <en>BoxAndWhisker ChartData object,this class is immutable.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class BoxAndWhiskerData extends AbstractChartData implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -2082242276393422837L ;

  /**
   * <zh>ƽ.</zh>
   * <en>The mean.</en>
   */
  private double mean ;

  /**
   * <zh>ֵ.</zh>
   * <en>The median.</en>
   */
  private double median ;

  /**
   * <zh>һķλ.</zh>
   * <en>The first quarter.</en>
   */
  private double q1 ;

  /**
   * <zh>ķλ.</zh>
   * <en>The third quarter.</en>
   */
  private double q3 ;

  /**
   * <zh>Сֵ.</zh>
   * <en>The minimum regular value.</en>
   */
  private double minRegularValue ;

  /**
   * <zh>ֵ.</zh>
   * <en>The maximum regular value.</en>
   */
  private double maxRegularValue ;

  /**
   * <zh>СȺֵ.</zh>
   * <en>The minimum outlier.</en>
   */
  private double minOutlier ;

  /**
   * <zh>Ⱥֵ.</zh>
   * <en>The maximum outlier.</en>
   */
  private double maxOutlier ;

  /**
   * <zh>Ⱥֵ.</zh>
   * <en>The outliers.</en>
   */
  private List outliers ;

  /**
   * <zh>ݼ϶.</zh>
   * <en>Storage for the data.</en>
   */
  private List data ;

  /**
   * <zh>ǷѾ.</zh>
   * <en>Whether or not calculate values.</en>
   */
  private boolean isCalculate ;

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   */
  public BoxAndWhiskerData() {
    this.data = new ArrayList() ;
    this.isCalculate = false ;
  }

  /**
   * <zh>һֵ.</zh>
   * <en>Add a value.</en>
   *
   * @param value Number <zh>ֵ.</zh>
   * <en>A value.</en>
   */
  public void addValue(Number value) {
    this.isCalculate = false ;
    this.data.add(value) ;
  }

  /**
   * <zh>Ӷֵ.</zh>
   * <en>Add a value array.</en>
   *
   * @param values Number[] <zh>ֵ (Ϊnull).</zh>
   * <en>A value array (null not permitted).</en>
   */
  public void addValues(Number[] values) {
    if (values == null) {
      throw new IllegalArgumentException("Null 'values' argument") ;
    }

    this.isCalculate = false ;
    this.data.addAll(Arrays.asList(values)) ;
  }

  /**
   * <zh>ɾindex.</zh>
   * <en>Removes the element at the specified position in this list.</en>
   *
   * @param index int <zh>ֵ.</zh>
   * <en>The index of the element to removed..</en>
   *
   * @return Object <zh>ɾԪ.</zh>
   * <en>The element that was removed from the list.</en>
   * @throws IndexOutOfBoundsException <zh>indexԽ,׳쳣.</zh>
   * <en>If index out of range (index &lt; 0 || index &gt;= size()).</en>
   */
  public Object removeValue(int index) {
    this.isCalculate = false ;
    return this.data.remove(index) ;
  }

  /**
   * <zh>ɾеֵ.</zh>
   * <en>Remove all values.</en>
   */
  public void clearValues() {
    this.isCalculate = false ;
    this.data.clear() ;
  }

  /**
   * <zh>еֵ,˼ϲ޸.</zh>
   * <en>Returns the list of values,and is unmodifiable.</en>
   *
   * @return List <zh>еֵ,˼ϲ޸.</zh>
   * <en>The list of values,and is unmodifiable.</en>
   */
  public List getValues() {
    this.isCalculate = false ;
    return Collections.unmodifiableList(this.data) ;
  }

  /**
   * <zh>ƽ.</zh>
   * <en>Returns the mean.</en>
   *
   * @return double <zh>ƽ.</zh>
   * <en>The mean.</en>
   */
  public double getMean() {
    calculateBoxAndWhiskerStatistics(true) ;
    return this.mean ;
  }

  /**
   * <zh>ֵ.</zh>
   * <en>Returns the median.</en>
   *
   * @return double <zh>ֵ (Ϊnull).</zh>
   * <en>The median (possibly null).</en>
   */
  public double getMedian() {
    calculateBoxAndWhiskerStatistics(true) ;
    return this.median ;
  }

  /**
   * <zh>صһķλ.</zh>
   * <en>Returns the first quarter.</en>
   *
   * @return double <zh>һķλ.</zh>
   * <en>The first quartile.</en>
   */
  public double getQ1() {
    calculateBoxAndWhiskerStatistics(true) ;
    return this.q1 ;
  }

  /**
   * <zh>صķλ.</zh>
   * <en>Returns the third quarter.</en>
   *
   * @return double <zh>ķλ.</zh>
   * <en>The third quartile.</en>
   */
  public double getQ3() {
    calculateBoxAndWhiskerStatistics(true) ;
    return this.q3 ;
  }

  /**
   * <zh>Сֵ.</zh>
   * <en>Returns the minimum regular value.</en>
   *
   * @return double <zh>Сֵ.</zh>
   * <en>The minimum regular value.</en>
   */
  public double getMinRegularValue() {
    calculateBoxAndWhiskerStatistics(true) ;
    return this.minRegularValue ;
  }

  /**
   * <zh>ֵ.</zh>
   * <en>Returns the maximum regular value.</en>
   *
   * @return double <zh>ֵ.</zh>
   * <en>The maximum regular value.</en>
   */
  public double getMaxRegularValue() {
    calculateBoxAndWhiskerStatistics(true) ;
    return this.maxRegularValue ;
  }

  /**
   * <zh>СȺֵ.</zh>
   * <en>Returns the minimum outlier.</en>
   *
   * @return double <zh>СȺֵ.</zh>
   * <en>The minimum outlier.</en>
   */
  public double getMinOutlier() {
    calculateBoxAndWhiskerStatistics(true) ;
    return this.minOutlier ;
  }

  /**
   * <zh>Ⱥֵ.</zh>
   * <en>Returns the maximum outlier.</en>
   *
   * @return double <zh>Ⱥֵ.</zh>
   * <en>The maximum outlier.</en>
   */
  public double getMaxOutlier() {
    calculateBoxAndWhiskerStatistics(true) ;
    return this.maxOutlier ;
  }

  /**
   * <zh>Ⱥֵ.</zh>
   * <en>Returns the outliers.</en>
   *
   * @return List <zh>Ⱥֵ.</zh>
   * <en>The outliers.</en>
   */
  public List getOutliers() {
    calculateBoxAndWhiskerStatistics(true) ;
    if (this.outliers == null) {
      return null ;
    }

    return Collections.unmodifiableList(this.outliers) ;
  }

  /**
   * <zh>еķֵ,еnullֵֵDouble.NaNֵ.</zh>
   * <en>Calculates the statistics from a list of Number objects.  Any items in the list
   * that are null, not an instance of Number, or equivalent to Double.NaN, will be ignored.</en>
   *
   * @param stripNullAndNaNItems <zh>ǷȥֵNaNֵ.</zh>
   * <en>A flag that controls the handling of null and NaN items.</en>
   */
  private void calculateBoxAndWhiskerStatistics(boolean stripNullAndNaNItems) {
    if (this.isCalculate) {
      return ;
    }

    List vlist = null ;
    if (stripNullAndNaNItems) {
      vlist = new ArrayList(this.data.size()) ;
      Iterator iterator = this.data.listIterator() ;
      while (iterator.hasNext()) {
        Object obj = iterator.next() ;
        if (obj != null && obj instanceof Number) {
          Number n = (Number) obj ;
          double v = n.doubleValue() ;
          if (!Double.isNaN(v)) {
            vlist.add(n) ;
          }
        }
      }
    } else {
      vlist = this.data ;
    }
    Collections.sort(vlist) ;

    this.mean = calculateMean(vlist, false) ;
    this.median = calculateMedian(vlist, false) ;
    this.q1 = calculateQ1Value(vlist) ;
    this.q3 = calculateQ3Value(vlist) ;

    double interQuartileRange = this.q3 - this.q1 ;
    double upperOutlierThreshold = this.q3 + (interQuartileRange * 1.5) ;
    double lowerOutlierThreshold = this.q1 - (interQuartileRange * 1.5) ;

    double upperFaroutThreshold = this.q3 + (interQuartileRange * 2.0) ;
    double lowerFaroutThreshold = this.q1 - (interQuartileRange * 2.0) ;

    this.minRegularValue = Double.POSITIVE_INFINITY ;
    this.maxRegularValue = Double.NEGATIVE_INFINITY ;
    this.minOutlier = Double.POSITIVE_INFINITY ;
    this.maxOutlier = Double.NEGATIVE_INFINITY ;
    this.outliers = new ArrayList() ;

    Iterator iterator = vlist.iterator() ;
    while (iterator.hasNext()) {
      Number number = (Number) iterator.next() ;
      double value = number.doubleValue() ;
      if (value > upperOutlierThreshold) {
        this.outliers.add(number) ;
        if (value > this.maxOutlier && value <= upperFaroutThreshold) {
          this.maxOutlier = value ;
        }
      } else if (value < lowerOutlierThreshold) {
        this.outliers.add(number) ;
        if (value < this.minOutlier && value >= lowerFaroutThreshold) {
          this.minOutlier = value ;
        }
      } else {
        this.minRegularValue = Math.min(this.minRegularValue, value) ;
        this.maxRegularValue = Math.max(this.maxRegularValue, value) ;
      }
      this.minOutlier = Math.min(this.minOutlier, this.minRegularValue) ;
      this.maxOutlier = Math.max(this.maxOutlier, this.maxRegularValue) ;
    }

    this.isCalculate = true ;
  }

  /**
   * <zh>ֵϵֵ.</zh>
   * <en>Calculate the mean of a collection of Number objects.</en>
   *
   * @param values List <zh>ֵļ (Ϊnull).</zh>
   * <en>The values (null not permitted).</en>
   * @param includeNullAndNaN boolean <zh>аnullֵNaNֵ,򷵻{@link Double#NaN}.</zh>
   * <en>A flag that controls whether or not null and Double.NaN values are included in the
   * calculation (if either is present in the array, the result is {@link Double#NaN}).</en>
   *
   * @return double <zh>ֵ.</zh>
   * <en>The mean.</en>
   */
  private static double calculateMean(List values, boolean includeNullAndNaN) {
    if (values == null) {
      throw new IllegalArgumentException("Null 'values' argument.") ;
    }

    int count = 0 ;
    double total = 0.0 ;
    Iterator iterator = values.iterator() ;
    while (iterator.hasNext()) {
      Object object = iterator.next() ;
      if (object == null) {
        if (includeNullAndNaN) {
          return Double.NaN ;
        }
      } else {
        if (object instanceof Number) {
          Number number = (Number) object ;
          double value = number.doubleValue() ;
          if (Double.isNaN(value)) {
            if (includeNullAndNaN) {
              return Double.NaN ;
            }
          } else {
            total += number.doubleValue() ;
            count++ ;
          }
        }
      }
    }

    return (count == 0) ? Double.NaN : total / count ;
  }

  /**
   * <zh>ּvaluesλ,copyAndSortΪtrue,ΪֵѾ.ÿӦΪ
   * Number.</zh>
   * <en>Calculates the median for a list of values (Number objects).If copyAndSort is false, the
   * list is assumed to be presorted in ascending order by value.</en>
   *
   * @param values List <zh>ֵļ (Ϊnull).</zh>
   * <en>The values (null permitted).</en>
   * @param copyAndSort boolean <zh>ǷҪƲ.</zh>
   * <en>A flag that controls whether the list of values is copied and sorted.</en>
   *
   * @return double <zh>λ.</zh>
   * <en>The median.</en>
   */
  private static double calculateMedian(List values, boolean copyAndSort) {
    if (values == null) {
      return Double.NaN ;
    }

    return calculateMedian(values, 0, values.size() - 1, copyAndSort) ;
  }

  /**
   * <zh>㼯ϵӼλ,copyAndSortΪtrue,ΪֵѾ.ÿӦΪ
   * Number.</zh>
   * <en>Calculates the median for a sublist within a list of values (Number objects).
   * The entire list will be sorted if the ascending argument is false.</en>
   *
   * @param values  List <zh>ֵļ (Ϊnull).</zh>
   * <en>The values (null permitted).</en>
   * @param start int <zh>ʼ.</zh>
   * <en>The start index.</en>
   * @param end int <zh>.</zh>
   * <en>The end index.</en>
   * @param copyAndSort boolean <zh>ǷҪƲ.</zh>
   * <en>A flag that controls whether the list of values is copied and sorted.</en>
   *
   * @return double <zh>λ.</zh>
   * <en>The median.</en>
   */
  private static double calculateMedian(List values, int start, int end, boolean copyAndSort) {
    double result = Double.NaN ;
    if (values == null) {
      return result ;
    }

    if (copyAndSort) {
      List working = new ArrayList(end - start + 1) ;
      for (int i = start ; i <= end ; i++) {
        working.add(values.get(i)) ;
      }
      Collections.sort(working) ;
      result = calculateMedian(working, false) ;
    } else {
      int count = end - start + 1 ;
      if (count > 0) {
        if (count % 2 == 1) {
          if (count > 1) {
            Number value = (Number) values.get(start + (count - 1) / 2) ;
            result = value.doubleValue() ;
          } else {
            Number value = (Number) values.get(start) ;
            result = value.doubleValue() ;
          }
        } else {
          Number value1 = (Number) values.get(start + count / 2 - 1) ;
          Number value2 = (Number) values.get(start + count / 2) ;
          result = (value1.doubleValue() + value2.doubleValue()) / 2.0 ;
        }
      }
    }

    return result ;
  }

  /**
   * <zh>һķλ.</zh>
   * <en>Calculate the first quartile value.</en>
   *
   * @param values List <zh>ֵļ (Ϊnull).</zh>
   * <en>The values (null permitted).</en>
   *
   * @return double <zh>һķλ.</zh>
   * <en>The first quartile value.</en>
   */
  private static double calculateQ1Value(List values) {
    double result = Double.NaN ;
    if (values == null) {
      return result ;
    }

    int count = values.size() ;
    if (count > 0) {
      if (count % 2 == 1) {
        if (count > 1) {
          result = calculateMedian(values, 0, count / 2, true) ;
        } else {
          result = calculateMedian(values, 0, 0, true) ;
        }
      } else {
        result = calculateMedian(values, 0, count / 2 - 1, true) ;
      }
    }

    return result ;
  }

  /**
   * <zh>ķλ.</zh>
   * <en>Calculates the third quartile for a list of numbers in ascending order.
   * If the items in the list are not in ascending order, the result is
   * unspecified.  If the list contains items that are null, not
   * an instance of Number, or equivalent to Double.NaN, the result is unspecified.</en>
   *
   * @param values List <zh>ֵļ (Ϊnull).</zh>
   * <en>The values (null permitted).</en>
   *
   * @return double <zh>ķλ.</zh>
   * <en>The third quartile value.</en>
   */
  public double calculateQ3Value(List values) {
    double result = Double.NaN ;
    if (values == null) {
      return result ;
    }

    int count = values.size() ;
    if (count > 0) {
      if (count % 2 == 1) {
        if (count > 1) {
          result = calculateMedian(values, count / 2, count - 1, true) ;
        } else {
          result = calculateMedian(values, 0, 0, true) ;
        }
      } else {
        result = calculateMedian(values, count / 2, count - 1, true) ;
      }
    }

    return result ;
  }

  /**
   * <zh>رǩַ.</zh>
   * <en>Returns the label text.</en>
   *
   * @return String <zh>ǩַ (Ϊnull).</zh>
   * <en>The label text (maybe null).</en>
   */
  public String getLabelText() {
    return "(" + formatNumber(getMedian())
      + "," + formatNumber(getMean())
      + "," + formatNumber(getMinRegularValue())
      + "," + formatNumber(getMaxRegularValue())
      + "," + formatNumber(getQ1())
      + "," + formatNumber(getQ3())
      + "," + formatNumber(getMinOutlier())
      + "," + formatNumber(getMaxOutlier()) + ")" ;
  }

  /**
   * <zh>ͼȵareaĺ.</zh>
   * <en>Returns the function arguments of hotspot area.</en>
   *
   * @return String <zh> (Ϊnull).</zh>
   * <en>The function arguments (maybe null).</en>
   */
  public String getAreaFunctionArguments() {
    return "(" + formatNumberForFunction(getMedian())
      + "," + formatNumberForFunction(getMean())
      + "," + formatNumberForFunction(getMinRegularValue())
      + "," + formatNumberForFunction(getMaxRegularValue())
      + "," + formatNumberForFunction(getQ1())
      + "," + formatNumberForFunction(getQ3())
      + "," + formatNumberForFunction(getMinOutlier())
      + "," + formatNumberForFunction(getMaxOutlier()) + ")" ;
  }

  /**
   * <zh>ͼȵareaurl.</zh>
   * <en>Returns the url for hotspot area.</en>
   *
   * @return String <zh>url (Ϊnull).</zh>
   * <en>The url fragment (maybe null).</en>
   */
  public String getAreaURLFragment() {
    return "median=" + formatNumber(getMedian())
      + "&mean=" + formatNumber(getMean())
      + "&minRegularValue=" + formatNumber(getMinRegularValue())
      + "&maxRegularValue=" + formatNumber(getMaxRegularValue())
      + "&q1=" + formatNumber(getQ1())
      + "&q3=" + formatNumber(getQ3())
      + "&minOutlier=" + formatNumber(getMinOutlier())
      + "&maxOutlier=" + formatNumber(getMaxOutlier()) ;
  }

  /**
   * <zh>ʵComparableӿڵcompareTo().</zh>
   * <en>Implements the compareTo() method of interface Comparable.</en>
   *
   * @param o Object <zh>ҪȽϵĶ.</zh>
   * <en>The Object to be compared.</en>
   * @return int <zh>ֵΪʾС,0ʾ,ʾ.</zh>
   * <en>A negative integer, zero, or a positive integer as this object	is less than, equal to, or
   * greater than the specified object.</en>
   *
   * @throws ClassCastException if the specified object's type prevents it
   *         from being compared to this Object.
   */
  public int compareTo(Object o) {
    if (o == null) {
      return -1 ;
    } else {
      if (!(o instanceof BoxAndWhiskerData)) {
        throw new ClassCastException("Must be 'BoxAndWhiskerData'.") ;
      }

      BoxAndWhiskerData other = (BoxAndWhiskerData) o ;
      double diff = this.mean - other.mean ;
      return (diff < 0) ? -1 : ((diff == 0) ? 0 : 1) ;
    }
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof BoxAndWhiskerData)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    BoxAndWhiskerData that = (BoxAndWhiskerData) obj ;
    if (this.isCalculate != that.isCalculate) {
      return false ;
    }
    if (this.mean != that.mean) {
      return false ;
    }
    if (this.median != that.median) {
      return false ;
    }
    if (this.q1 != that.q1) {
      return false ;
    }
    if (this.q3 != that.q3) {
      return false ;
    }
    if (this.minRegularValue != that.minRegularValue) {
      return false ;
    }
    if (this.maxRegularValue != that.maxRegularValue) {
      return false ;
    }
    if (this.minOutlier != that.minOutlier) {
      return false ;
    }
    if (this.maxOutlier != that.maxOutlier) {
      return false ;
    }
    if (!EqualsUtil.equals(this.outliers, that.outliers)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.data, that.data)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double result = super.hashCode() ;
    result = result * 37 + this.mean ;
    result = result * 37 + this.median ;
    result = result * 37 + this.q1 ;
    result = result * 37 + this.q3 ;
    result = result * 37 + this.minRegularValue ;
    result = result * 37 + this.maxRegularValue ;
    result = result * 37 + this.minOutlier ;
    result = result * 37 + this.maxOutlier ;
    result = result * 37 + HashcodeUtil.hashCode(this.outliers) ;
    result = result * 37 + HashcodeUtil.hashCode(this.isCalculate) ;
    result = result * 37 + HashcodeUtil.hashCode(this.data) ;

    return (int) result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[mean=" + getMean()
      + ",median=" + getMedian()
      + ",q1=" + getQ1()
      + ",q3=" + getQ3()
      + "]" ;
  }

}
