package com.ijchart.xychart.data ;

import java.io.Serializable ;

import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ͼݶ,ⲿǲ޸ĵ.</zh>
 * <en>Error ChartData object,this class is immutable.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class ErrorChartData extends AbstractChartData implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -588293602038152012L ;

  /**
   * <zh>xֵ.</zh>
   * <en>The x value.</en>
   */
  private Number x ;

  /**
   * <zh>yֵ.</zh>
   * <en>The y value.</en>
   */
  private Number y ;

  /**
   * <zh>xĵֵ.</zh>
   * <en>The lower bound of x.</en>
   */
  private Number xLow ;

  /**
   * <zh>xĸֵ.</zh>
   * <en>The upper bound of x.</en>
   */
  private Number xHigh ;

  /**
   * <zh>yĵֵ.</zh>
   * <en>The lower bound of y.</en>
   */
  private Number yLow ;

  /**
   * <zh>yĸֵ.</zh>
   * <en>The upper bound of y.</en>
   */
  private Number yHigh ;

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param x double <zh>xֵ.</zh>
   * <en>The x value.</en>
   * @param xLow double <zh>xĵֵ.</zh>
   * <en>The lower bound of x.</en>
   * @param xHigh double <zh>xĸֵ.</zh>
   * <en>The upper bound of x.</en>
   * @param y double <zh>yֵ.</zh>
   * <en>The y value.</en>
   * @param yLow double <zh>yĵֵ.</zh>
   * <en>The lower bound of y.</en>
   * @param yHigh double <zh>yĸֵ.</zh>
   * <en>The upper bound of y.</en>
   */
  public ErrorChartData(double x, double xLow, double xHigh, double y, double yLow, double yHigh) {
    this(new Double(x), new Double(xLow), new Double(xHigh),
         new Double(y), new Double(yLow), new Double(yHigh)) ;
  }

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param x Number <zh>xֵ.</zh>
   * <en>The x value.</en>
   * @param xLow Number <zh>xĵֵ.</zh>
   * <en>The lower bound of x.</en>
   * @param xHigh Number <zh>xĸֵ.</zh>
   * <en>The upper bound of x.</en>
   * @param y Number <zh>yֵ.</zh>
   * <en>The y value.</en>
   * @param yLow Number <zh>yĵֵ.</zh>
   * <en>The lower bound of y.</en>
   * @param yHigh Number <zh>yĸֵ.</zh>
   * <en>The upper bound of y.</en>
   */
  public ErrorChartData(Number x, Number xLow, Number xHigh, Number y, Number yLow, Number yHigh) {
    if (x == null) {
      throw new IllegalArgumentException("Null 'x' argument") ;
    }
    if (xLow == null) {
      throw new IllegalArgumentException("Null 'xLow' argument") ;
    }
    if (xHigh == null) {
      throw new IllegalArgumentException("Null 'xHigh' argument") ;
    }
    if (y == null) {
      throw new IllegalArgumentException("Null 'y' argument") ;
    }
    if (yLow == null) {
      throw new IllegalArgumentException("Null 'yLow' argument") ;
    }
    if (yHigh == null) {
      throw new IllegalArgumentException("Null 'yHigh' argument") ;
    }

    this.x = x ;
    this.xLow = xLow ;
    this.xHigh = xHigh ;

    this.y = y ;
    this.yLow = yLow ;
    this.yHigh = yHigh ;
  }

  /**
   * <zh>xֵ.</zh>
   * <en>Returns the x value.</en>
   *
   * @return Number <zh>Ϊnull.</zh>
   * <en>Never null.</en>
   */
  public Number getX() {
    return this.x ;
  }

  /**
   * <zh>xĵֵ.</zh>
   * <en>Returns the lower bound of x.</en>
   *
   * @return Number <zh>Ϊnull.</zh>
   * <en>Never null.</en>
   */
  public Number getXLow() {
    return this.xLow ;
  }

  /**
   * <zh>xĸֵ.</zh>
   * <en>Returns the upper bound of x.</en>
   *
   * @return Number <zh>Ϊnull.</zh>
   * <en>Never null.</en>
   */
  public Number getXHigh() {
    return this.xHigh ;
  }

  /**
   * <zh>yֵ.</zh>
   * <en>Returns the y value.</en>
   *
   * @return Number <zh>Ϊnull.</zh>
   * <en>Never null.</en>
   */
  public Number getY() {
    return this.y ;
  }

  /**
   * <zh>yĵֵ.</zh>
   * <en>Returns the lower bound of y.</en>
   *
   * @return Number <zh>Ϊnull.</zh>
   * <en>Never null.</en>
   */
  public Number getYLow() {
    return this.yLow ;
  }

  /**
   * <zh>yĸֵ.</zh>
   * <en>Returns the upper bound of y.</en>
   *
   * @return Number <zh>Ϊnull.</zh>
   * <en>Never null.</en>
   */
  public Number getYHigh() {
    return this.yHigh ;
  }

  /**
   * <zh>xֵ.</zh>
   * <en>Returns the x value.</en>
   *
   * @return double <zh>double͵ֵ.</zh>
   * <en>The double value.</en>
   */
  public double getXValue() {
    return this.x.doubleValue() ;
  }

  /**
   * <zh>xĵֵ.</zh>
   * <en>Returns the lower bound of x.</en>
   *
   * @return double <zh>double͵ֵ.</zh>
   * <en>The double value.</en>
   */
  public double getXLowValue() {
    return this.xLow.doubleValue() ;
  }

  /**
   * <zh>xĸֵ.</zh>
   * <en>Returns the upper bound of x.</en>
   *
   * @return double <zh>double͵ֵ.</zh>
   * <en>The double value.</en>
   */
  public double getXHighValue() {
    return this.xHigh.doubleValue() ;
  }

  /**
   * <zh>yֵ.</zh>
   * <en>Returns the y value.</en>
   *
   * @return double <zh>double͵ֵ.</zh>
   * <en>The double value.</en>
   */
  public double getYValue() {
    return this.y.doubleValue() ;
  }

  /**
   * <zh>yĵֵ.</zh>
   * <en>Returns the lower bound of y.</en>
   *
   * @return double <zh>double͵ֵ.</zh>
   * <en>The double value.</en>
   */
  public double getYLowValue() {
    return this.yLow.doubleValue() ;
  }

  /**
   * <zh>yĸֵ.</zh>
   * <en>Returns the upper bound of y.</en>
   *
   * @return double <zh>double͵ֵ.</zh>
   * <en>The double value.</en>
   */
  public double getYHighValue() {
    return this.yHigh.doubleValue() ;
  }

  /**
   * <zh>رǩַ.</zh>
   * <en>Returns the label text.</en>
   *
   * @return String <zh>ǩַ (Ϊnull).</zh>
   * <en>The label text (never null).</en>
   */
  public String getLabelText() {
    return "(" + formatNumber(this.x) + "," + formatNumber(this.xLow) + ","
      + formatNumber(this.xHigh) + "," + formatNumber(this.y) + "," + formatNumber(this.yLow)
      + "," + formatNumber(this.yHigh) + ")" ;
  }

  /**
   * <zh>ͼȵareaĺ.</zh>
   * <en>Returns the function arguments of hotspot area.</en>
   *
   * @return String <zh> (Ϊnull).</zh>
   * <en>The function arguments (maybe null).</en>
   */
  public String getAreaFunctionArguments() {
    return formatNumberForFunction(this.x) + "," + formatNumberForFunction(this.xLow) + ","
      + formatNumberForFunction(this.xHigh) + "," + formatNumberForFunction(this.y) + ","
      + formatNumberForFunction(this.yLow) + "," + formatNumberForFunction(this.yHigh) ;
  }

  /**
   * <zh>ͼȵareaurl.</zh>
   * <en>Returns the url for hotspot area.</en>
   *
   * @return String <zh>url (Ϊnull).</zh>
   * <en>The url fragment (maybe null).</en>
   */
  public String getAreaURLFragment() {
    return "x=" + formatNumber(this.x) + "&xLow=" + formatNumber(this.xLow) + "&xHigh="
      + formatNumber(this.xHigh) + "&y=" + formatNumber(this.y) + "&yLow=" + formatNumber(this.yLow)
      + "&yHigh=" + formatNumber(this.yHigh) ;
  }

  /**
   * <zh>ʵComparableӿڵcompareTo().</zh>
   * <en>Implements the compareTo() method of interface Comparable.</en>
   *
   * @param o Object <zh>ҪȽϵĶ.</zh>
   * <en>The Object to be compared.</en>
   * @return int <zh>ֵΪʾС,0ʾ,ʾ.</zh>
   * <en>A negative integer, zero, or a positive integer as this object	is less than, equal to, or
   * greater than the specified object.</en>
   *
   * @throws ClassCastException if the specified object's type prevents it
   *         from being compared to this Object.
   */
  public int compareTo(Object o) {
    if (o == null) {
      return -1 ;
    } else {
      if (!(o instanceof ErrorChartData)) {
        throw new ClassCastException("Must be 'ErrorChartData'.") ;
      }

      ErrorChartData other = (ErrorChartData) o ;
      double diff = this.x.doubleValue() - other.x.doubleValue() ;

      return (diff < 0) ? -1 : ((diff == 0) ? 0 : 1) ;
    }
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof ErrorChartData)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    ErrorChartData that = (ErrorChartData) obj ;
    if (this.x.doubleValue() != that.x.doubleValue()) {
      return false ;
    }
    if (this.xLow.doubleValue() != that.xLow.doubleValue()) {
      return false ;
    }
    if (this.xHigh.doubleValue() != that.xHigh.doubleValue()) {
      return false ;
    }
    if (this.y.doubleValue() != that.y.doubleValue()) {
      return false ;
    }
    if (this.yLow.doubleValue() != that.yLow.doubleValue()) {
      return false ;
    }
    if (this.yHigh.doubleValue() != that.yHigh.doubleValue()) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 37 * result + HashcodeUtil.hashCode(this.x) ;
    result = 37 * result + HashcodeUtil.hashCode(this.xLow) ;
    result = 37 * result + HashcodeUtil.hashCode(this.xHigh) ;
    result = 37 * result + HashcodeUtil.hashCode(this.y) ;
    result = 37 * result + HashcodeUtil.hashCode(this.yLow) ;
    result = 37 * result + HashcodeUtil.hashCode(this.yHigh) ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[x=" + this.x + ",xLow=" + this.xLow + ",xHigh=" + this.xHigh + ",y="
      + this.y + ",yLow=" + this.yLow + ",yHigh=" + this.yHigh + "]" ;
  }

}
