package com.ijchart.xychart.event ;

import java.io.Serializable ;
import java.util.EventObject ;

import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartChangeEventType ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ͼ仯¼.</zh>
 * <en>A change event that encapsulates information about a change to a chart.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class ChartChangeEvent extends EventObject implements Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -2151626037366856778L ;

  /**
   * <zh>¼.</zh>
   * <en>The type of event.</en>
   */
  private ChartChangeEventType type ;

  /**
   * <zh>¼ͼ.</zh>
   * <en>The chart that generated the event.</en>
   */
  private IJChart chart ;

  /**
   * <zh>һµͼ仯¼,¼Ϊ{@link ChartChangeEventType#GENERAL}.</zh>
   * <en>Creates a new chart change event,the event type is {@link ChartChangeEventType#GENERAL}.
   * </en>
   *
   * @param source Object <zh>¼Դ.</zh>
   * <en>The source of the event (could be the chart, a title,an axis etc.).</en>
   */
  public ChartChangeEvent(Object source) {
    this(source, null, ChartChangeEventType.GENERAL) ;
  }

  /**
   * <zh>һµͼ仯¼,¼Ϊ{@link ChartChangeEventType#GENERAL}.</zh>
   * <en>Creates a new chart change event,the event type is {@link ChartChangeEventType#GENERAL}.
   * </en>
   *
   * @param source <zh>¼Դ.</zh>
   * <en>The source of the event (could be the chart, a title, an axis etc.).</en>
   * @param chart <zh>¼ͼ.</zh>
   * <en>The chart that generated the event.</en>
   */
  public ChartChangeEvent(Object source, IJChart chart) {
    this(source, chart, ChartChangeEventType.GENERAL) ;
  }

  /**
   * <zh>һµͼ仯¼,¼Ϊ{@link ChartChangeEventType#GENERAL}.</zh>
   * <en>Creates a new chart change event,the event type is {@link ChartChangeEventType#GENERAL}.
   * </en>
   *
   * @param source <zh>¼Դ.</zh>
   * <en>The source of the event (could be the chart, a title, an axis etc.).</en>
   * @param chart <zh>¼ͼ.</zh>
   * <en>The chart that generated the event.</en>
   * @param type <zh>¼.</zh>
   * <en>The type of event.</en>
   */
  public ChartChangeEvent(Object source, IJChart chart,
                          ChartChangeEventType type) {
    super(source) ;
    this.chart = chart ;
    this.type = type ;
  }

  /**
   * <zh>ز¼ͼ.</zh>
   * <en>Returns the chart that generated the change event.</en>
   *
   * @return <zh>¼ͼ.</zh>
   * <en>The chart that generated the change event.</en>
   */
  public IJChart getChart() {
    return this.chart ;
  }

  /**
   * <zh>ò¼ͼ.</zh>
   * <en>Sets the chart that generated the change event.</en>
   *
   * @param chart <zh>¼ͼ.</zh>
   * <en>The chart that generated the event.</en>
   */
  public void setChart(IJChart chart) {
    this.chart = chart ;
  }

  /**
   * <zh>ͼ¼仯.</zh>
   * <en>Returns the event type.</en>
   *
   * @return The event type.
   */
  public ChartChangeEventType getType() {
    return this.type ;
  }

  /**
   * <zh>¼.</zh>
   * <en>Sets the event type.</en>
   *
   * @param type <zh>¼.</zh>
   * <en>The event type.</en>
   */
  public void setType(ChartChangeEventType type) {
    this.type = type ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   * @return int <zh>hashcodeֵ.</zh>
   * <en>The hashcode.</en>
   */
  public int hashCode() {
    return this.type.hashCode() ;
  }

  /**
   * <zh>equals.</zh>
   * <en>Override equals method.</en>
   * @param obj Object
   * @return boolean
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }

    if (!(obj instanceof ChartChangeEvent)) {
      return false ;
    }

    ChartChangeEvent other = (ChartChangeEvent) obj ;

    return this.type.equals(other.type) ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Override toString method.</en>
   * @return String
   */
  public String toString() {
    return "[type=" + this.type + "]" ;
  }

}
