package com.ijchart.xychart.format ;

import java.io.Serializable ;
import java.text.DecimalFormat ;
import java.text.NumberFormat ;
import java.util.Locale ;

import com.ijchart.xychart.util.EqualsUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʽ(java.lang.Number)Ͷ.</zh>
 * <en>A Formatter provides support for formatting a java.lang.Number Object.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class NumberFormatter implements IFormatter, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -6255602964483109291L ;

  /**
   * <zh>ػ.</zh>
   * <en>A java.util.Locale instance.</en>
   */
  private Locale locale ;

  /**
   * <zh>ʽֵ.</zh>
   * <en>The number format mask.</en>
   */
  private String mask ;

  /**
   * <zh>ʽֵjava.text.DecimalFormat.</zh>
   * <en>Instance of java.text.DecimalFormat for format number object.</en>
   */
  private DecimalFormat format ;

  /**
   * <zh>빹챾.</zh>
   * <en>Create a new NumberFormatter by default locale and a number mask.</en>
   *
   * @param mask String <zh>.</zh>
   * <en>A number mask.</en>
   */
  public NumberFormatter(String mask) {
    this(Locale.getDefault(), mask) ;
  }

  /**
   * <zh>빹챾.</zh>
   * <en>Create a new NumberFormatter by a number mask.</en>
   *
   * @param locale Locale <zh>ػ.</zh>
   * <en>A java.util.Locale instance.</en>
   * @param mask String <zh>.</zh>
   * <en>A number mask.</en>
   */
  public NumberFormatter(Locale locale, String mask) {
    if (locale == null) {
      throw new IllegalArgumentException("Null 'locale' argument.") ;
    }
    if (mask == null) {
      throw new IllegalArgumentException("Null 'mask' argument.") ;
    }

    this.locale = locale ;
    this.mask = mask ;

    this.format = (DecimalFormat) NumberFormat.getInstance(locale) ;
    this.format.applyPattern(mask) ;
  }

  /**
   * <zh>رػ.</zh>
   * <en>Returns a java.util.Locale instance.</en>
   *
   * @return Locale <zh>ػ.</zh>
   * <en>A java.util.Locale instance.</en>
   */
  public Locale getLocale() {
    return this.locale ;
  }

  /**
   * <zh>ظʽֵ.</zh>
   * <en>Returns the number format mask.</en>
   *
   * @return String <zh>ʽֵ.</zh>
   * <en>The number format mask.</en>
   */
  public String getMask() {
    return this.mask ;
  }

  /**
   * <zh>ظʽֵjava.text.DecimalFormat.</zh>
   * <en>Returns instance of java.text.DecimalFormat for format number object.</en>
   *
   * @return DecimalFormat
   */
  public DecimalFormat getFormat() {
    return this.format ;
  }

  /**
   * <zh>ʽһjava.lang.NumberΪַ.ĳֵһϵж(ϵıǩ
   * ̶ֵ),ʽʱԸindexֵĲͬʽΪַͬ.numberֵΪLong.MAX_VALUE.
   * indexʱûʹ,Ҫʹ,ɼ̳д.</zh>
   * <en>Formatted a java.lang.Number object to a string. If a value is composed of a
   * series of objects (such as the coordinates of the axis labels or scale value),
   * the format could be based on index.The max value of this object is Long.MAX_VALUE.
   * Parameter index do not have to use,to use,extends this class.</en>
   *
   * @param obj Object <zh>ҪʽĶ (Ϊnull).</zh>
   * <en>The object to be formatted (null permitted).</en>
   * @param index int <zh>һϵֵ˳,0ʼ.˲ûʹ.</zh>
   * <en>The index value of a series objects.
   * The first is at index 0, the next is at 1, and so on.This parameter is not used.</en>
   *
   * @return String <zh>ʽַ.</zh>
   * <en>A formatted Number string.</en>
   */
  public String format(Object obj, int index) {
    if (obj == null) {
      return "" ;
    }

    if (obj instanceof Number) {
      Number number = (Number) obj ;

      return format.format(number.doubleValue()) ;
    } else { //תΪַ
      try {
        String str = obj.toString() ;
        double value = Double.parseDouble(str) ;

        return format.format(value) ;
      } catch (NumberFormatException ex) {
        return "" ;
      }
    }

  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this object for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪȽϵһ (Ϊnull).</zh>
   * <en>The object to test against (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof NumberFormatter)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    NumberFormatter that = (NumberFormatter) obj ;
    if (!EqualsUtil.equals(this.locale, that.locale)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.mask, that.mask)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code.</en>
   *
   * @return A hash code.
   */
  public int hashCode() {
    return this.locale.hashCode() + 29 * mask.hashCode() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of this Range.</en>
   *
   * @return A String.
   */
  public String toString() {
    return "[locale=" + this.locale + ",mask=" + this.mask + "]" ;
  }

}
