package com.ijchart.xychart.label ;

import java.io.Serializable ;

import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.format.IChartDataFormatter ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ȱʡıͼǩ.ֶformatterûset,Բüı仯.</zh>
 * <en>The default label generator for pie chart,this class is immutable.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DefaultPieChartLabelGenerator extends LabelGeneratorAdaptor implements ILabelGenerator,
  Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 5490405492194362330L ;

  /**
   * <zh>ʽݵ.</zh>
   * <en>The data formatter.</en>
   */
  private IChartDataFormatter formatter ;

  /**
   * <zh>ûвĹ.</zh>
   * <en>No argument constructor.</en>
   */
  public DefaultPieChartLabelGenerator() {
    this(null) ;
  }

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param formatter IChartDataFormatter <zh>ʽݵ (Ϊnull).</zh>
   * <en>The data formatter (null permitted).</en>
   */
  public DefaultPieChartLabelGenerator(IChartDataFormatter formatter) {
    this.formatter = formatter ;
  }

  /**
   * <zh>ظʽݵ.</zh>
   * <en>Returns this data formatter.</en>
   *
   * @return IChartDataFormatter <zh>ʽݵ (Ϊnull).</zh>
   * <en>The data formatter (maybe null).</en>
   */
  public IChartDataFormatter getFormatter() {
    return this.formatter ;
  }

  /**
   * <zh>ĳͼεıǩ,ͨ±ǩ֪ͨʽrowKeycolKeyֵɵ.
   * ҲԸʽΪַ.</zh>
   * <en>Generates a label for the specified item. The label is typically a
   * formatted version of the data value, but any text can be used.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column number,must be greater than or equal to 0.</en>
   *
   * @return String <zh>ǩַ.</zh>
   * <en>The label (possibly null).</en>
   */
  public String generateLabel(IMatrixDataset dataset, int row, int column) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument.") ;
    }
    if (row < 0) {
      throw new IllegalArgumentException("Must be row>=0.") ;
    }
    if (column < 0) {
      throw new IllegalArgumentException("Must be column>=0.") ;
    }

    if (this.formatter != null) {
      return this.formatter.format(dataset, row, column) ;
    }

    Object data = dataset.getColKey(column) ;
    if (data == null) {
      return "" ;
    }

    return data.toString() ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this object for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪȽϵһ (Ϊnull).</zh>
   * <en>The object to test against (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof DefaultPieChartLabelGenerator)) {
      return false ;
    }

    DefaultPieChartLabelGenerator that = (DefaultPieChartLabelGenerator) obj ;
    if (!EqualsUtil.equals(this.formatter, that.formatter)) {
      return false ;
    }
    return true ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Creates and returns a copy of this object.</en>
   *
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the <code>Cloneable</code> interface. Subclasses
   *               that override the <code>clone</code> method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  protected Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code.</en>
   *
   * @return A hash code.
   */
  public int hashCode() {
    int result = HashcodeUtil.hashCode(this.formatter) ;
    return result ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of this Range.</en>
   *
   * @return A String.
   */
  public String toString() {
    return "[formatter=" + this.formatter + "]" ;
  }

}
