package com.ijchart.xychart.plot ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.Stroke ;
import java.awt.geom.Line2D ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.HashMap ;
import java.util.Hashtable ;
import java.util.List ;
import java.util.Map ;

import com.ijchart.xychart.axis.AbstractAxis ;
import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.AxisLocation ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.event.PlotChangeEvent ;
import com.ijchart.xychart.listener.IRenderChangeListener ;
import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.LineChartShape ;
import com.ijchart.xychart.tickunit.INumberTickUnit ;
import com.ijchart.xychart.tickunit.LogTickUnit ;
import com.ijchart.xychart.ui.IEffect3D ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.RectangleInsets ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Plot.</zh>
 * <en>The plot object which have two axis at least.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class AbstractAxisPlot extends AbstractPlot implements IRenderChangeListener,
  Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 7141192581994435950L ;

  /**
   * <zh>ȱʡͼ.</zh>
   * <en>The default chart orientation.</en>
   */
  public static final ChartOrientation DEFAULT_CHART_ORIENTATION = ChartOrientation.VERTICAL ;

  /**
   * <zh>ȱʡdomain᷽ʮǷɼ.</zh>
   * <en>A default flag that controls whether or not a domain crosshair is drawn.</en>
   */
  public static final boolean DEFAULT_DOMAIN_CROSSHAIR_VISIBLE = false ;

  /**
   * <zh>ȱʡʮǷԶݵ.</zh>
   * <en>A default flag that controls whether or not the crosshair locks
   * onto actual data points.</en>
   */
  public static final boolean DEFAULT_DOMAIN_CROSSHAIR_LOCKED_ON_DATA = true ;

  /**
   * <zh>ȱʡdomainֵ᷽Ϊ0ĻǷɼ.</zh>
   * <en>A default flag that controls whether or not the zero baseline against the domain
   * axis is visible.</en>
   */
  public static final boolean DEFAULT_DOMAIN_ZERO_BASELINE_VISIBLE = false ;

  /**
   * <zh>ȱʡvalue᷽ʮǷɼ.</zh>
   * <en>A default flag that controls whether or not a value crosshair is drawn.</en>
   */
  public static final boolean DEFAULT_VALUE_CROSSHAIR_VISIBLE = false ;

  /**
   * <zh>ȱʡʮǷԶݵ.</zh>
   * <en>A default flag that controls whether or not the crosshair locks onto
   * actual data points.</en>
   */
  public static final boolean DEFAULT_VALUE_CROSSHAIR_LOCKED_ON_DATA = true ;

  /**
   * <zh>ȱʡdomain᷽Ƿɼ.</zh>
   * <en>The default visibility of the grid lines plotted against the domain axis.</en>
   */
  public static final boolean DEFAULT_DOMAIN_GRIDLINE_VISIBLE = true ;

  /**
   * <zh>ȱʡValue᷽Ƿɼ.</zh>
   * <en>The default visibility of the grid lines plotted against the value axis.</en>
   */
  public static final boolean DEFAULT_VALUE_GRIDLINE_VISIBLE = true ;

  /**
   * <zh>ȱʡstroke.</zh>
   * <en>The default grid line stroke.</en>
   */
  public static final Stroke DEFAULT_GRIDLINE_STROKE =
    new BasicStroke(0.5f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 0.0f,
                    new float[] {2.0f, 2.0f}, 0.0f) ;

  /**
   * <zh>ȱʡpaint.</zh>
   * <en>The default grid line paint.</en>
   */
  public static final Paint DEFAULT_GRIDLINE_PAINT = Color.WHITE ;

  /**
   * <zh>ȱʡʮStroke.</zh>
   * <en>The default crosshair stroke.</en>
   */
  public static final Stroke DEFAULT_CROSSHAIR_STROKE = DEFAULT_GRIDLINE_STROKE ;

  /**
   * <zh>ȱʡʮPaint.</zh>
   * <en>The default crosshair paint.</en>
   */
  public static final Paint DEFAULT_CROSSHAIR_PAINT = Color.BLUE ;

  /**
   * <zh>ͼ.</zh>
   * <en>The chart orientation.</en>
   */
  private ChartOrientation orientation ;

  /**
   * <zh>֮Ŀհ.</zh>
   * <en>The offset between the data area and the axes.</en>
   */
  private RectangleInsets axisOffset ;

  /**
   * <zh>ڴ洢domain.</zh>
   * <en>Storage for the domain axes.</en>
   */
  protected List domainAxes ;

  /**
   * <zh>ڴ洢value.</zh>
   * <en>Storage for the value axes.</en>
   */
  protected List valueAxes ;

  /**
   * <zh>renderdomainĶӦϵ.</zh>
   * <en>Storage for keys that map render to domain axis.</en>
   */
  protected Hashtable renderToDomainAxis ;

  /**
   * <zh>rendervalueĶӦϵ.</zh>
   * <en>Storage for keys that map render to value axis.</en>
   */
  protected Hashtable renderToValueAxis ;

  /**
   * <zh>ڴ洢valueλõļ.</zh>
   * <en>Storage for the value axis locations.</en>
   */
  private List valueAxisLocations ;

  /**
   * <zh>ƥvalueᵽrenderͼμ.</zh>
   * <en>Storage for keys that map value axis to render.</en>
   */
  private Map valueAxisToRenderMap ;

  /**
   * <zh>domainǷɼ.</zh>
   * <en>A flag that controls whether the grid-lines for the domain axis are visible.</en>
   */
  private boolean domainGridlineVisible ;

  /**
   * <zh>domainߵstroke.</zh>
   * <en>The stroke used to draw the domain grid-lines.</en>
   */
  private transient Stroke domainGridlineStroke ;

  /**
   * <zh>domainߵpaint.</zh>
   * <en>The paint used to draw the domain grid-lines.</en>
   */
  private transient Paint domainGridlinePaint ;

  /**
   * <zh>valueǷɼ.</zh>
   * <en>A flag that controls whether the grid-lines for the value axis are visible.</en>
   */
  private boolean valueGridlineVisible ;

  /**
   * <zh>valueߵstroke.</zh>
   * <en>The stroke used to draw the value axis grid-lines.</en>
   */
  private transient Stroke valueGridlineStroke ;

  /**
   * <zh>valueߵpaint.</zh>
   * <en>The paint used to draw the value axis grid-lines.</en>
   */
  private transient Paint valueGridlinePaint ;

  /**
   * <zh>domain᷽ʮǷɼ.</zh>
   * <en>A flag that controls whether or not a domain crosshair is drawn.</en>
   */
  private boolean domainCrosshairVisible ;

  /**
   * <zh>domainʮߵStroke.</zh>
   * <en>The pen/brush used to draw the crosshair (if any).</en>
   */
  private transient Stroke domainCrosshairStroke ;

  /**
   * <zh>domain᷽ʮߵPaint.</zh>
   * <en>The color used to draw the crosshair (if any).</en>
   */
  private transient Paint domainCrosshairPaint ;

  /**
   * <zh>ʮǷԶݵ.</zh>
   * <en>A flag that controls whether or not the crosshair locks onto actual data points.</en>
   */
  private boolean domainCrosshairLockedOnData ;

  /**
   * <zh>value᷽ʮǷɼ.</zh>
   * <en>A flag that controls whether or not a value crosshair is drawn.</en>
   */
  private boolean valueCrosshairVisible ;

  /**
   * <zh>valueʮߵStroke.</zh>
   * <en>The pen/brush used to draw the crosshair (if any).</en>
   */
  private transient Stroke valueCrosshairStroke ;

  /**
   * <zh>value᷽ʮߵPaint.</zh>
   * <en>The color used to draw the crosshair (if any).</en>
   */
  private transient Paint valueCrosshairPaint ;

  /**
   * <zh>ʮǷԶݵ.</zh>
   * <en>A flag that controls whether or not the crosshair locks onto actual data points.</en>
   */
  private boolean valueCrosshairLockedOnData ;

  /**
   * <zh>ȱʡĹ.</zh>
   * <en>The default constructor.</en>
   */
  public AbstractAxisPlot() {
    this.orientation = DEFAULT_CHART_ORIENTATION ;
    this.axisOffset = RectangleInsets.INSETS_ZERO ;

    this.domainAxes = new ArrayList() ;
    this.valueAxes = new ArrayList() ;
    this.valueAxisLocations = new ArrayList() ;

    this.renderToDomainAxis = new Hashtable() ;
    this.renderToValueAxis = new Hashtable() ;

    this.domainGridlineVisible = DEFAULT_DOMAIN_GRIDLINE_VISIBLE ;
    this.domainGridlineStroke = DEFAULT_GRIDLINE_STROKE ;
    this.domainGridlinePaint = DEFAULT_GRIDLINE_PAINT ;

    this.valueGridlineVisible = DEFAULT_VALUE_GRIDLINE_VISIBLE ;
    this.valueGridlineStroke = DEFAULT_GRIDLINE_STROKE ;
    this.valueGridlinePaint = DEFAULT_GRIDLINE_PAINT ;

    this.domainCrosshairVisible = DEFAULT_DOMAIN_CROSSHAIR_VISIBLE ;
    this.domainCrosshairStroke = DEFAULT_CROSSHAIR_STROKE ;
    this.domainCrosshairPaint = DEFAULT_CROSSHAIR_PAINT ;
    this.domainCrosshairLockedOnData = DEFAULT_DOMAIN_CROSSHAIR_LOCKED_ON_DATA ;

    this.valueCrosshairVisible = DEFAULT_VALUE_CROSSHAIR_VISIBLE ;
    this.valueCrosshairStroke = DEFAULT_CROSSHAIR_STROKE ;
    this.valueCrosshairPaint = DEFAULT_CROSSHAIR_PAINT ;
    this.valueCrosshairLockedOnData = DEFAULT_VALUE_CROSSHAIR_LOCKED_ON_DATA ;

    this.renderShapesList = new ArrayList() ;
    this.valueAxisToRenderMap = new HashMap() ;
  }

  /**
   * <zh>ͼķ.</zh>
   * <en>Returns the orientation of the plot.</en>
   *
   * @return <zh>ͼķ (Ϊnull).</zh>
   * <en>The orientation of the plot (never null).</en>
   */
  public ChartOrientation getOrientation() {
    return this.orientation ;
  }

  /**
   * <zh>ͼķ,еļ{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the orientation for the plot and sends a {@link PlotChangeEvent} to
   * all registered listeners.</en>
   *
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The orientation (null not permitted).</en>
   */
  public void setOrientation(ChartOrientation orientation) {
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    this.orientation = orientation ;
    notifyListeners(new PlotChangeEvent(this)) ;
  }

  /**
   * <zh>֮Ŀհ.</zh>
   * <en>Returns the axis offset.</en>
   *
   * @return <zh>հ(Ϊnull).</zh>
   * <en>The axis offset (never null).</en>
   */
  public RectangleInsets getAxisOffset() {
    return this.axisOffset ;
  }

  /**
   * <zh>֮Ŀհ,еļ{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the axis offsets (gap between the data area and the axes) and
   * sends a {@link PlotChangeEvent} to all registered listeners.</en>
   *
   * @param offset RectangleInsets <zh>֮Ŀհ (Ϊnull).</zh>
   * <en>The offset (null not permitted).</en>
   */
  public void setAxisOffset(RectangleInsets offset) {
    if (offset == null) {
      throw new IllegalArgumentException("Null 'offset' argument.") ;
    }

    this.axisOffset = offset ;
    notifyListeners(new PlotChangeEvent(this)) ;
  }

  /**
   * <zh>ɾеdomain,м{@link PlotChangeEvent}¼..</zh>
   * <en>Clears the domain axes from the plot and sends a {@link PlotChangeEvent}
   * to all registered listeners.</en>
   */
  public void clearDomainAxes() {
    for (int i = 0 ; i < this.domainAxes.size() ; i++) {
      AbstractAxis axis = (AbstractAxis)this.domainAxes.get(i) ;
      if (axis != null) {
        axis.removeChangeListener(this) ;
      }
    }

    this.domainAxes.clear() ;
    notifyListeners(new PlotChangeEvent(this)) ;
  }

  /**
   * <zh>value.</zh>
   * <en>Returns the value axis for the plot.</en>
   *
   * @return AbstractValueAxis <zh>value (Ϊnull).</zh>
   * <en>The value axis (possibly null).</en>
   */
  public AbstractValueAxis getValueAxis() {
    return getValueAxis(0) ;
  }

  /**
   * <zh>ֵָvalue.</zh>
   * <en>Returns a value axis.</en>
   *
   * @param index int <zh>ֵ,ڵ0.</zh>
   * <en>The axis index,must be greater than or equal to 0.</en>
   *
   * @return AbstractValueAxis <zh>value.</zh>
   * <en>The axis (null possible).</en>
   */
  public AbstractValueAxis getValueAxis(int index) {
    if (index < 0) {
      throw new IllegalArgumentException("Must be 'index'>=0.") ;
    }

    AbstractValueAxis result = null ;
    if (index < this.valueAxes.size()) {
      result = (AbstractValueAxis)this.valueAxes.get(index) ;
    }

    return result ;
  }

  /**
   * <zh>value.</zh>
   * <en>Returns a value axis.</en>
   *
   * @param renderIndex <zh>renderֵ.</zh>
   * <en>The index of render.</en>
   *
   * @return <zh>value (Ϊnull).</zh>
   * <en>The value axis (null possible).</en>
   */
  public AbstractValueAxis getValueAxisByRenderIndex(int renderIndex) {
    Integer axisIndex = (Integer)this.renderToValueAxis.get(new Integer(renderIndex)) ;
    if (axisIndex != null) {
      int idx = axisIndex.intValue() ;
      if (idx < this.valueAxes.size()) {
        return (AbstractValueAxis)this.valueAxes.get(idx) ;
      } else {
        return (AbstractValueAxis)this.valueAxes.get(this.valueAxes.size() - 1) ;
      }
    }

    if (renderIndex < this.valueAxes.size()) {
      return (AbstractValueAxis)this.valueAxes.get(renderIndex) ;
    } else {
      return (AbstractValueAxis)this.valueAxes.get(this.valueAxes.size() - 1) ;
    }
  }

  /**
   * <zh>value,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the value axis for the plot and sends a {@link PlotChangeEvent} to
   * all registered listeners.</en>
   *
   * @param axis AbstractValueAxis <zh>value (Ϊnull).</zh>
   * <en>The axis (null not permitted).</en>
   */
  public void setValueAxis(AbstractValueAxis axis) {
    setValueAxis(0, axis, true) ;
  }

  /**
   * <zh>value,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the value axis for the plot and sends a {@link PlotChangeEvent} to
   * all registered listeners.</en>
   *
   * @param index int <zh>ֵ,ڵ0.</zh>
   * <en>The axis index,must be greater than or equal to 0.</en>
   * @param axis AbstractValueAxis <zh>value (Ϊnull).</zh>
   * <en>The axis (null not permitted).</en>
   */
  public void setValueAxis(int index, AbstractValueAxis axis) {
    setValueAxis(index, axis, true) ;
  }

  /**
   * <zh>ֵvalue,notifyֵȷǷм{@link PlotChangeEvent}¼.</zh>
   * <en>Sets a value axis and, if requested, sends a {@link PlotChangeEvent} to
   * all registered listeners.</en>
   *
   * @param index int <zh>ֵ,ڵ0.</zh>
   * <en>The axis index,must be greater than or equal to 0.</en>
   * @param axis AbstractValueAxis <zh>value (Ϊnull).</zh>
   * <en>The value axis (null not permitted).</en>
   * @param notify boolean <zh>Ƿ֪ͨ.</zh>
   * <en>Whether or not notify listeners.</en>
   */
  public void setValueAxis(int index, AbstractValueAxis axis, boolean notify) {
    if (index < this.valueAxes.size()) {
      AbstractValueAxis existing = (AbstractValueAxis)this.valueAxes.get(index) ;
      if (existing != null) {
        existing.removeChangeListener(this) ;
      }
    }

    setListObject(valueAxes, index, axis) ;
    if (axis != null) {
      axis.addChangeListener(this) ;
    }

    if (notify) {
      notifyListeners(new PlotChangeEvent(this)) ;
    }
  }

  /**
   * <zh>plotvalue,ѾڵĶᱻ,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the value axes for this plot and sends a {@link PlotChangeEvent}
   * to all registered listeners.</en>
   *
   * @param axes AbstractValueAxis[] <zh>value (Ϊnull).</zh>
   * <en>The axes (null not permitted).</en>
   */
  public void setValueAxes(AbstractValueAxis[] axes) {
    setValueAxes(axes, 0) ;
  }

  /**
   * <zh>plotvalue,ѾڵĶᱻ,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the value axes for this plot and sends a {@link PlotChangeEvent}
   * to all registered listeners.</en>
   *
   * @param axes AbstractValueAxis[] <zh>value (Ϊnull).</zh>
   * <en>The axes (null not permitted).</en>
   * @param startIndex int <zh>Ŀʼֵ,startIndex֮ǰĶ󲻻ᱻ,ڵ0.</zh>
   * <en>The beginning of the index value, startIndex before the target will not be covered.
   * must be greater than or equal to 0.</en>
   */
  public void setValueAxes(AbstractValueAxis[] axes, int startIndex) {
    if (axes == null) {
      throw new IllegalArgumentException("Null 'axes' argument.") ;
    }
    if (startIndex < 0) {
      throw new IllegalArgumentException("Must be 'startIndex'>=0.") ;
    }

    if (axes.length > 0) {
      for (int i = 0 ; i < axes.length ; i++) {
        setValueAxis(startIndex + i, axes[i], false) ;
      }

      notifyListeners(new PlotChangeEvent(this)) ;
    }
  }

  /**
   * <zh>valueֵ,û򷵻-1.</zh>
   * <en>Returns the index of the specified axis, or -1 if the axis
   * is not assigned to the plot.</en>
   *
   * @param axis <zh>value.</zh>
   * <en>The value axis (null not permitted).</en>
   *
   * @return int <zh>ֵ.</zh>
   * <en>The axis index.</en>
   */
  public int getValueAxisIndex(AbstractValueAxis axis) {
    if (axis == null) {
      throw new IllegalArgumentException("Null 'axis' argument.") ;
    }

    return this.valueAxes.indexOf(axis) ;
  }

  /**
   * <zh>ص0valueλ.</zh>
   * <en>Returns the value axis location.</en>
   *
   * @return AxisLocation <zh>λ (Ϊnull).</zh>
   * <en>The location (never null).</en>
   */
  public AxisLocation getValueAxisLocation() {
    return getValueAxisLocation(0) ;
  }

  /**
   * <zh>ָvalueλ.</zh>
   * <en>Returns the location for a value axis.</en>
   *
   * @param index <zh>ֵ,ڵ0.</zh>
   * <en>The axis index,must be greater than or equal to 0.</en>
   *
   * @return AxisLocation <zh>λ.</zh>
   * <en>The location.</en>
   */
  public AxisLocation getValueAxisLocation(int index) {
    if (index < 0) {
      throw new IllegalArgumentException("Must be 'index'>=0.") ;
    }

    AxisLocation result = null ;
    if (index < this.valueAxisLocations.size()) {
      result = (AxisLocation)this.valueAxisLocations.get(index) ;
    }
    if (result == null) {
      result = AxisLocation.getOpposite(getValueAxisLocation(0)) ;
    }

    return result ;
  }

  /**
   * <zh>valueλ,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the location of the value axis and sends a {@link PlotChangeEvent}
   * to all registered listeners.</en>
   *
   * @param location AxisLocation <zh>λ (Ϊnull).</zh>
   * <en>The location (null not permitted).</en>
   */
  public void setValueAxisLocation(AxisLocation location) {
    setValueAxisLocation(0, location, true) ;
  }

  /**
   * <zh>ƥrenderdomain.</zh>
   * <en>Maps a render to a domain axis.</en>
   *
   * @param renderIndex int <zh>render.</zh>
   * <en>The index of render object.</en>
   * @param domainAxisIndex int <zh>domain.</zh>
   * <en>The index of domain axis.</en>
   */
  public void mapRenderToDomainAxis(int renderIndex, int domainAxisIndex) {
    this.renderToDomainAxis.put(new Integer(renderIndex), new Integer(domainAxisIndex)) ;
  }

  /**
   * <zh>ƥrenderdomain.</zh>
   * <en>Maps a render to a domain axis.</en>
   *
   * @param renderIndex int <zh>render.</zh>
   * <en>The index of render object.</en>
   * @param valueAxisIndex int <zh>value.</zh>
   * <en>The index of value axis.</en>
   */
  public void mapRenderToValueAxis(int renderIndex, int valueAxisIndex) {
    this.renderToValueAxis.put(new Integer(renderIndex), new Integer(valueAxisIndex)) ;
  }

  /**
   * <zh>valueλ,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the location of the value axis and sends a {@link PlotChangeEvent}
   * to all registered listeners.</en>
   *
   * @param location AxisLocation <zh>λ (Ϊnull).</zh>
   * <en>The location (null not permitted).</en>
   * @param notify <zh>Ƿ֪ͨ.</zh>
   * <en>Whether or not notify listeners.</en>
   */
  public void setValueAxisLocation(AxisLocation location, boolean notify) {
    setValueAxisLocation(0, location, notify) ;
  }

  /**
   * <zh>valueλ,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the location of the value axis and sends a {@link PlotChangeEvent}
   * to all registered listeners.</en>
   *
   * @param index int <zh>ֵ,ڵ0.</zh>
   * <en>The axis index,must be greater than or equal to 0.</en>
   * @param location AxisLocation <zh>λ (Ϊnull).</zh>
   * <en>The location (null not permitted).</en>
   */
  public void setValueAxisLocation(int index, AxisLocation location) {
    setValueAxisLocation(index, location, true) ;
  }

  /**
   * <zh>valueλ,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the location of the value axis and sends a {@link PlotChangeEvent}
   * to all registered listeners.</en>
   *
   * @param index int <zh>ֵ,ڵ0.</zh>
   * <en>The axis index,must be greater than or equal to 0.</en>
   * @param location AxisLocation <zh>λ (Ϊnull).</zh>
   * <en>The location (null not permitted).</en>
   * @param notify <zh>Ƿ֪ͨ.</zh>
   * <en>Whether or not notify listeners.</en>
   */
  public void setValueAxisLocation(int index, AxisLocation location, boolean notify) {
    if (index < 0) {
      throw new IllegalArgumentException("Must be 'index'>=0") ;
    }
    if (location == null) {
      throw new IllegalArgumentException("Null 'location' argument.") ;
    }

    setListObject(valueAxisLocations, index, location) ;
    if (notify) {
      notifyListeners(new PlotChangeEvent(this)) ;
    }
  }

  /**
   * <zh>ص0valueı߽.</zh>
   * <en>Returns the edge where the primary value axis is located.</en>
   *
   * @return <zh>ֵ߽ (Ϊnull).</zh>
   * <en>The edge (never null).</en>
   */
  public RectangleEdge getValueAxisEdge() {
    return getValueAxisEdge(0) ;
  }

  /**
   * <zh>ֵָvalueı߽.</zh>
   * <en>Returns the edge for a value axis.</en>
   *
   * @param index int <zh>value.</zh>
   * <en>The axis index.</en>
   *
   * @return RectangleEdge <zh>ֵ߽.</zh>
   * <en>The edge.</en>
   */
  public RectangleEdge getValueAxisEdge(int index) {
    AxisLocation location = getValueAxisLocation(index) ;
    if (location == null) {
      location = AxisLocation.BOTTOM_OR_LEFT ; //default value
    }

    return AbstractPlot.resolveValueAxisLocation(location, getOrientation()) ;
  }

  /**
   * <zh>value.</zh>
   * <en>Returns the number of value axes.</en>
   *
   * @return int <zh>value.</zh>
   * <en>The axis count.</en>
   */
  public int getValueAxisCount() {
    return this.valueAxes.size() ;
  }

  /**
   * <zh>ɾеvalue,м{@link PlotChangeEvent}¼.</zh>
   * <en>Clears the value axes from the plot and sends a {@link PlotChangeEvent}
   * to all registered listeners.</en>
   */
  public void clearValueAxes() {
    for (int i = 0 ; i < this.valueAxes.size() ; i++) {
      AbstractValueAxis axis = (AbstractValueAxis)this.valueAxes.get(i) ;
      if (axis != null) {
        axis.removeChangeListener(this) ;
      }
    }

    this.valueAxes.clear() ;
    notifyListeners(new PlotChangeEvent(this)) ;
  }

  /**
   * <zh>domainǷɼ.</zh>
   * <en>Returns the flag that controls whether the grid-lines
   * for the domain axis are visible.</en>
   *
   * @return boolean <zh>truefalse.</zh>
   * <en>The true or false.</en>
   */
  public boolean isDomainGridlineVisible() {
    return this.domainGridlineVisible ;
  }

  /**
   * <zh>domainǷɼ,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the flag that controls whether or not grid-lines are drawn against
   * the domain axis.
   * <p>
   * If the flag value changes, a {@link PlotChangeEvent} is sent to all
   * registered listeners.</en>
   *
   * @param visible boolean <zh>booleanֵ.</zh>
   * <en>The new value of the flag.</en>
   */
  public void setDomainGridlineVisible(boolean visible) {
    if (this.domainGridlineVisible != visible) {
      this.domainGridlineVisible = visible ;
      notifyListeners(new PlotChangeEvent(this)) ;
    }
  }

  /**
   * <zh>ػdomainߵstroke.</zh>
   * <en>Returns the stroke used to draw the domain grid-lines.</en>
   *
   * @return Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The stroke (never null).</en>
   */
  public Stroke getDomainGridlineStroke() {
    return this.domainGridlineStroke ;
  }

  /**
   * <zh>ûdomainߵstroke,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the stroke used to draw grid-lines against the domain axis and
   * sends a {@link PlotChangeEvent} to all registered listeners.</en>
   *
   * @param stroke Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The stroke (null not permitted).</en>
   */
  public void setDomainGridlineStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' not permitted.") ;
    }

    this.domainGridlineStroke = stroke ;
    notifyListeners(new PlotChangeEvent(this)) ;
  }

  /**
   * <zh>ػdomainߵpaint.</zh>
   * <en>Returns the paint used to draw grid-lines against the domain axis.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getDomainGridlinePaint() {
    return this.domainGridlinePaint ;
  }

  /**
   * <zh>ûdomainߵpaint,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw the grid-lines (if any) against the domain
   * axis and sends a {@link PlotChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setDomainGridlinePaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.domainGridlinePaint = paint ;
    notifyListeners(new PlotChangeEvent(this)) ;
  }

  /**
   * <zh>valueǷɼ.</zh>
   * <en>Returns the flag that controls whether the grid-lines for the value axis are visible.</en>
   *
   * @return boolean <zh>booleanֵ.</zh>
   * <en>The flag.</en>
   */
  public boolean isValueGridlineVisible() {
    return this.valueGridlineVisible ;
  }

  /**
   * <zh>valueǷɼ,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the flag that controls whether or not grid-lines are drawn against
   * the value axis.  If the flag changes value, a {@link PlotChangeEvent} is
   * sent to all registered listeners.</en>
   *
   * @param visible boolean <zh>booleanֵ.</zh>
   * <en>The new value of the flag.</en>
   */
  public void setValueGridlineVisible(boolean visible) {
    if (this.valueGridlineVisible != visible) {
      this.valueGridlineVisible = visible ;
      notifyListeners(new PlotChangeEvent(this)) ;
    }
  }

  /**
   * <zh>ػvalueߵstroke.</zh>
   * <en>Returns the stroke used to draw the value axis grid-lines.</en>
   *
   * @return Stroke <zh>stroke.</zh>
   * <en>The stroke (never null).</en>
   */
  public Stroke getValueGridlineStroke() {
    return this.valueGridlineStroke ;
  }

  /**
   * <zh>ûvalueߵstroke,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the stroke used to draw the grid-lines against the value axis and
   * sends a {@link PlotChangeEvent} to all registered listeners.</en>
   *
   * @param stroke Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The stroke (null not permitted).</en>
   */
  public void setValueGridlineStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }
    this.valueGridlineStroke = stroke ;
    notifyListeners(new PlotChangeEvent(this)) ;
  }

  /**
   * <zh>ػvalueߵpaint.</zh>
   * <en>Returns the paint used to draw the value axis grid-lines.</en>
   *
   * @return Paint <zh>paint.</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getValueGridlinePaint() {
    return this.valueGridlinePaint ;
  }

  /**
   * <zh>ûvalueߵpaint,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw the grid lines against the value axis and
   * sends a {@link PlotChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setValueGridlinePaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.valueGridlinePaint = paint ;
    notifyListeners(new PlotChangeEvent(this)) ;
  }

  /**
   * <zh>domain᷽ʮǷɼ.</zh>
   * <en>Returns a flag indicating whether or not the domain crosshair is visible.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public boolean isDomainCrosshairVisible() {
    return this.domainCrosshairVisible ;
  }

  /**
   * <zh>domain᷽ʮǷɼ,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the flag indicating whether or not the domain crosshair is visible
   * and, if the flag changes, sends a {@link PlotChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The new value of the flag.</en>
   */
  public void setDomainCrosshairVisible(boolean flag) {
    if (this.domainCrosshairVisible != flag) {
      this.domainCrosshairVisible = flag ;
      notifyListeners(new PlotChangeEvent(this)) ;
    }
  }

  /**
   * <zh>ʮǷԶݵ.</zh>
   * <en>Returns a flag indicating whether or not the crosshair should "lock-on"
   * to actual data values.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public boolean isDomainCrosshairLockedOnData() {
    return this.domainCrosshairLockedOnData ;
  }

  /**
   * <zh>ʮǷԶݵ,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the flag indicating whether or not the domain crosshair should
   * "lock-on" to actual data values.  If the flag value changes, this
   * method sends a {@link PlotChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public void setDomainCrosshairLockedOnData(boolean flag) {
    if (this.domainCrosshairLockedOnData != flag) {
      this.domainCrosshairLockedOnData = flag ;
      notifyListeners(new PlotChangeEvent(this)) ;
    }
  }

  /**
   * <zh>domainʮߵStroke.</zh>
   * <en>Returns the {@link Stroke} used to draw the crosshair (if visible).</en>
   *
   * @return Stroke <zh>,Ϊnull.</zh>
   * <en>The crosshair stroke (never null).</en>
   */
  public Stroke getDomainCrosshairStroke() {
    return this.domainCrosshairStroke ;
  }

  /**
   * <zh>domainʮߵStroke,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the Stroke used to draw the crosshairs (if visible) and notifies
   * registered listeners that the axis has been modified.</en>
   *
   * @param stroke Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The new crosshair stroke (null not permitted).</en>
   */
  public void setDomainCrosshairStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }

    this.domainCrosshairStroke = stroke ;
    notifyListeners(new PlotChangeEvent(this)) ;
  }

  /**
   * <zh>domainʮߵPaint.</zh>
   * <en>Returns the {@link Paint} used to draw the crosshair.</en>
   *
   * @return The crosshair paint (never null).
   */
  public Paint getDomainCrosshairPaint() {
    return this.domainCrosshairPaint ;
  }

  /**
   * <zh>domainʮߵPaint,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw the crosshairs (if visible) and sends a
   * {@link PlotChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The new crosshair paint (null not permitted).</en>
   */
  public void setDomainCrosshairPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.domainCrosshairPaint = paint ;
    notifyListeners(new PlotChangeEvent(this)) ;
  }

  /**
   * <zh>ؿvalue᷽ʮǷɼ.</zh>
   * <en>Returns a flag indicating whether or not the value crosshair is visible.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public boolean isValueCrosshairVisible() {
    return this.valueCrosshairVisible ;
  }

  /**
   * <zh>ÿvalue᷽ʮǷɼ,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the flag indicating whether or not the value crosshair is visible
   * and, if the flag changes, sends a {@link PlotChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The new value of the flag.</en>
   */
  public void setValueCrosshairVisible(boolean flag) {
    if (this.valueCrosshairVisible != flag) {
      this.valueCrosshairVisible = flag ;
      notifyListeners(new PlotChangeEvent(this)) ;
    }
  }

  /**
   * <zh>ؿʮǷԶݵ.</zh>
   * <en>Returns a flag indicating whether or not the crosshair should "lock-on"
   * to actual data values.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public boolean isValueCrosshairLockedOnData() {
    return this.valueCrosshairLockedOnData ;
  }

  /**
   * <zh>ÿʮǷԶݵ,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the flag indicating whether or not the value crosshair should
   * "lock-on" to actual data values.  If the flag value changes, this
   * method sends a {@link PlotChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public void setValueCrosshairLockedOnData(boolean flag) {
    if (this.valueCrosshairLockedOnData != flag) {
      this.valueCrosshairLockedOnData = flag ;
      notifyListeners(new PlotChangeEvent(this)) ;
    }
  }

  /**
   * <zh>valueʮߵStroke.</zh>
   * <en>Returns the {@link Stroke} used to draw the crosshair (if visible).</en>
   *
   * @return Stroke <zh>,Ϊnull.</zh>
   * <en>The crosshair stroke (never null).</en>
   */
  public Stroke getValueCrosshairStroke() {
    return this.valueCrosshairStroke ;
  }

  /**
   * <zh>valueʮߵStroke,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the Stroke used to draw the crosshairs (if visible) and notifies
   * registered listeners that the axis has been modified.</en>
   *
   * @param stroke Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The new crosshair stroke (null not permitted).</en>
   */
  public void setValueCrosshairStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }

    this.valueCrosshairStroke = stroke ;
    notifyListeners(new PlotChangeEvent(this)) ;
  }

  /**
   * <zh>valueʮߵPaint.</zh>
   * <en>Returns the {@link Paint} used to draw the crosshair.</en>
   *
   * @return The crosshair paint (never null).
   */
  public Paint getValueCrosshairPaint() {
    return this.valueCrosshairPaint ;
  }

  /**
   * <zh>valueʮߵPaint,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw the crosshairs (if visible) and sends a
   * {@link PlotChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The new crosshair paint (null not permitted).</en>
   */
  public void setValueCrosshairPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.valueCrosshairPaint = paint ;
    notifyListeners(new PlotChangeEvent(this)) ;
  }

  /**
   * <zh>ƥvalueᵽrender.</zh>
   * <en>Maps a value axis to a render.</en>
   *
   * @param valueAxisIndex int <zh>valueֵ.</zh>
   * <en>The value axis index.</en>
   * @param renderIndex int <zh>renderֵ.</zh>
   * <en>The render index.</en>
   */
  public void mapValueAxisToRender(int valueAxisIndex, int renderIndex) {
    this.valueAxisToRenderMap.put(new Integer(valueAxisIndex), new Integer(renderIndex)) ;
  }

  /**
   * <zh>ָvalueضӦrender.</zh>
   * <en>Returns the render index for a value axis index.</en>
   *
   * @param valueAxisIndex int <zh>value.</zh>
   * <en>The value axis index.</en>
   * @return int <zh>render.</zh>
   * <en>The render index.</en>
   */
  public int getRenderIndexForValueAxis(int valueAxisIndex) {
    Object value = this.valueAxisToRenderMap.get(new Integer(valueAxisIndex)) ;
    if (value != null) {
      return ((Integer) value).intValue() ;
    }

    return -1 ;
  }

  /**
   * <zh>ʮ.</zh>
   * <en>Draws the crosshair.</en>
   *
   *
   * @param mouseClickPoint Point2D <zh>ʱ (Ϊnull).</zh>
   * <en>The mouse click point (null permitted).</en>
   *
   * @return boolean <zh>Ƿʮ߷˱仯.</zh>
   * <en>Returns whether or not crosshair is changed.</en>
   */
  public boolean drawCrosshair(Point2D mouseClickPoint) {
    if (mouseClickPoint == null || this.plotShapes == null || this.dataArea == null
        || !this.dataArea.contains(mouseClickPoint)) {
      if (this.plotShapes == null) {
        return false ;
      }

      IChartShape oldXCross = plotShapes.getXCrossShape() ;
      IChartShape oldYCross = plotShapes.getYCrossShape() ;

      plotShapes.setXCrossShape(null) ;
      plotShapes.setYCrossShape(null) ;
      return!(oldXCross == null && oldYCross == null) ;
    }

    //ȡеlock point
    List shapeLockPoints = null ;
    if (isDomainCrosshairLockedOnData() || isValueCrosshairLockedOnData()) {
      shapeLockPoints = new ArrayList() ;
      List renderShapeList = this.plotShapes.getAllRenderShapes() ;
      for (int i = 0 ; i < renderShapeList.size() ; i++) {
        RenderShapeCollection renderShapes = (RenderShapeCollection) renderShapeList.get(i) ;
        List dataItems = renderShapes.getDataItemShapes() ;
        for (int j = 0 ; j < dataItems.size() ; j++) {
          IChartShape shape = (IChartShape) dataItems.get(j) ;
          if (shape != null) {
            Point2D lockPoint = shape.getLockPoint() ;
            if (lockPoint != null) {
              shapeLockPoints.add(lockPoint) ;
            }
          }
        }
      }
    }

    double mouseX = mouseClickPoint.getX() ;
    double mouseY = mouseClickPoint.getY() ;
    double findX = Double.MAX_VALUE ;
    if ((isDomainCrosshairVisible() && isDomainCrosshairLockedOnData())
        || (isValueCrosshairVisible() && isValueCrosshairLockedOnData())) { //findX
      double minDis = Double.MAX_VALUE ;
      int size = shapeLockPoints.size() ;
      for (int i = 0 ; i < size ; i++) {
        Point2D point = (Point2D) shapeLockPoints.get(i) ;
        double distince = Math.abs(mouseX - point.getX()) ;
        if (distince < minDis) {
          minDis = distince ;
          findX = point.getX() ;
          if (minDis <= 0.5) { //Ѿܽ,ûҪ
            break ;
          }
        }
      }
    }

    if (isDomainCrosshairVisible()) {
      Line2D line = null ;
      if (isDomainCrosshairLockedOnData() && findX != Double.MAX_VALUE) {
        line = new Line2D.Double(findX, dataArea.getMinY(), findX, dataArea.getMaxY()) ;
      }
      if (line == null) {
        line = new Line2D.Double(mouseX, dataArea.getMinY(), mouseX, dataArea.getMaxY()) ;
      }

      LineChartShape lineShape = new LineChartShape(line) ;
      lineShape.setBorderPaint(getDomainCrosshairPaint()) ;
      lineShape.setBorderStroke(getDomainCrosshairStroke()) ;
      plotShapes.setXCrossShape(lineShape) ;
    } else {
      plotShapes.setXCrossShape(null) ;
    }

    double findY = Double.MAX_VALUE ;
    if (isValueCrosshairVisible()) {
      if (isValueCrosshairLockedOnData()) {
        if (findX == Double.MAX_VALUE) {
          findY = mouseY ;
        } else {
          double minDis = Double.MAX_VALUE ;
          int size = shapeLockPoints.size() ;
          for (int i = 0 ; i < size ; i++) {
            Point2D point = (Point2D) shapeLockPoints.get(i) ;
            if (Math.abs(point.getX() - findX) <= 0.1) { //ȷƥ,ƥ䲻
              double distince = Math.abs(mouseY - point.getY()) ;
              if (distince < minDis) {
                minDis = distince ;
                findY = point.getY() ;
                if (minDis <= 0.5) { //Ѿܽ,ûҪ
                  break ;
                }
              }
            }
          }
        }
      }

      Line2D line = null ;
      if (findY != Double.MAX_VALUE) {
        line = new Line2D.Double(dataArea.getMinX(), findY, dataArea.getMaxX(), findY) ;
      }
      if (line == null) {
        line = new Line2D.Double(dataArea.getMinX(), mouseY, dataArea.getMaxX(), mouseY) ;
      }
      LineChartShape lineShape = new LineChartShape(line) ;
      lineShape.setBorderPaint(getValueCrosshairPaint()) ;
      lineShape.setBorderStroke(getValueCrosshairStroke()) ;
      plotShapes.setYCrossShape(lineShape) ;
    } else {
      plotShapes.setYCrossShape(null) ;
    }

    return true ;
  }

  /**
   * <zh>ʮ߶Ӧĵ.</zh>
   * <en>Returns the crosshair point.</en>
   *
   * @param mouseClickPoint Point2D<zh>ʱ (Ϊnull).</zh>
   * <en>The mouse click point (null permitted).</en>
   *
   * @return Point2D <zh> (Ϊnull).</zh>
   * <en>The point (maybe null).</en>
   */
  public Point2D getCrosshairPoint(Point2D mouseClickPoint) {
    if (mouseClickPoint == null || this.plotShapes == null || this.dataArea == null
        || !this.dataArea.contains(mouseClickPoint)) {
      return null ;
    }

    //ȡеlock point
    List shapeLockPoints = null ;
    if (isDomainCrosshairLockedOnData() || isValueCrosshairLockedOnData()) {
      shapeLockPoints = new ArrayList() ;
      List renderShapeList = this.plotShapes.getAllRenderShapes() ;
      for (int i = 0 ; i < renderShapeList.size() ; i++) {
        RenderShapeCollection renderShapes = (RenderShapeCollection) renderShapeList.get(i) ;
        List dataItems = renderShapes.getDataItemShapes() ;
        for (int j = 0 ; j < dataItems.size() ; j++) {
          IChartShape shape = (IChartShape) dataItems.get(j) ;
          if (shape != null) {
            Point2D lockPoint = shape.getLockPoint() ;
            if (lockPoint != null) {
              shapeLockPoints.add(lockPoint) ;
            }
          }
        }
      }
    }

    double mouseX = mouseClickPoint.getX() ;
    double mouseY = mouseClickPoint.getY() ;
    double findX = Double.MAX_VALUE ;
    if ((isDomainGridlineVisible() && isDomainCrosshairLockedOnData())
        || (isValueCrosshairVisible() && isValueCrosshairLockedOnData())) { //findX
      double minDis = Double.MAX_VALUE ;
      int size = shapeLockPoints.size() ;
      for (int i = 0 ; i < size ; i++) {
        Point2D point = (Point2D) shapeLockPoints.get(i) ;
        double distince = Math.abs(mouseX - point.getX()) ;
        if (distince < minDis) {
          minDis = distince ;
          findX = point.getX() ;
          if (minDis <= 0.5) { //Ѿܽ,ûҪ
            break ;
          }
        }
      }
    }

    double pointX = Double.NaN, pointY = Double.NaN ;
    if (isDomainCrosshairLockedOnData() && findX != Double.MAX_VALUE) {
      pointX = findX ;
    } else {
      pointX = mouseX ;
    }

    double findY = Double.MAX_VALUE ;
    if (isValueCrosshairLockedOnData()) {
      if (findX == Double.MAX_VALUE) {
        findY = mouseY ;
      } else {
        double minDis = Double.MAX_VALUE ;
        int size = shapeLockPoints.size() ;
        for (int i = 0 ; i < size ; i++) {
          Point2D point = (Point2D) shapeLockPoints.get(i) ;
          if (Math.abs(point.getX() - findX) <= 0.1) { //ȷƥ,ƥ䲻
            double distince = Math.abs(mouseY - point.getY()) ;
            if (distince < minDis) {
              minDis = distince ;
              findY = point.getY() ;
              if (minDis <= 0.5) { //Ѿܽ,ûҪ
                break ;
              }
            }
          }
        }
      }
    }

    if (findY != Double.MAX_VALUE) {
      pointY = findY ;
    } else {
      pointY = Double.NaN ;
    }

    if (Double.isNaN(pointX) || Double.isNaN(pointY)) {
      return mouseClickPoint ;
    } else {
      return new Point2D.Double(pointX, pointY) ;
    }
  }

  /**
   * <zh>value.</zh>
   * <en>Zoom value axes.</en>
   *
   * @param selectArea Rectangle2D <zh>ѡ,Ϊnullʾָǰ״̬.</zh>
   * <en>The selected area,null to restore chart.</en>
   * @param dataArea Rectangle2D <zh>,Ϊnullʾָǰ״̬.</zh>
   * <en>The data area,null to restore chart.</en>
   * @param source Point2D <zh>Դ (Ϊnull).</zh>
   * <en>The source mouse point (null not permitted).</en>
   */
  public void zoomValueAxes(Rectangle2D selectArea, Rectangle2D dataArea, Point2D source) {
    if (source == null) {
      throw new IllegalArgumentException("Null 'source' argument.") ;
    }

    boolean restore = (selectArea == null || dataArea == null) ;
    for (int i = 0 ; i < this.valueAxes.size() ; i++) {
      AbstractAxis valueAxis = (AbstractAxis)this.valueAxes.get(i) ;
      if (valueAxis != null && valueAxis instanceof NumberValueAxis) {
        NumberValueAxis numberAxis = (NumberValueAxis) valueAxis ;
        if (restore) {
          numberAxis.setZoomDatasetRange(null) ;
          continue ;
        }

        NumberRange datasetRange = numberAxis.getTickRange() ;
        if (datasetRange == null) {
          continue ;
        }

        double lowerPercent = 0, upperPercent = 0 ;
        if (getOrientation().equals(ChartOrientation.VERTICAL)) {
          double lowerMarginLen = numberAxis.getLowerMargin() * dataArea.getHeight() ;
          double upperMarginLen = numberAxis.getUpperMargin() * dataArea.getHeight() ;

          double upperDataAreaY = dataArea.getY() + upperMarginLen ;
          double lowerDataAreaY = dataArea.getMaxY() - lowerMarginLen ;

          double selectAreaMinY = Math.max(selectArea.getMinY(), upperDataAreaY) ;
          double selectAreaMaxY = Math.min(selectArea.getMaxY(), lowerDataAreaY) ;
          double dataAreaDrawHeight = dataArea.getHeight() - lowerMarginLen - upperMarginLen ;

          lowerPercent = Math.abs(selectAreaMaxY - lowerDataAreaY) / dataAreaDrawHeight ;
          upperPercent = Math.abs(selectAreaMinY - upperDataAreaY) / dataAreaDrawHeight ;
        } else if (getOrientation().equals(ChartOrientation.HORIZONTAL)) {
          double lowerMarginLen = numberAxis.getLowerMargin() * dataArea.getWidth() ;
          double upperMarginLen = numberAxis.getUpperMargin() * dataArea.getWidth() ;

          double lowerDataAreaX = dataArea.getX() + lowerMarginLen ;
          double upperDataAreaX = dataArea.getMaxX() - upperMarginLen ;

          double selectAreaMinX = Math.max(selectArea.getMinX(), lowerDataAreaX) ;
          double selectAreaMaxX = Math.min(selectArea.getMaxX(), upperDataAreaX) ;
          double dataAreaDrawWidth = dataArea.getWidth() - lowerMarginLen - upperMarginLen ;

          lowerPercent = Math.abs(selectAreaMinX - lowerDataAreaX) / dataAreaDrawWidth ;
          upperPercent = Math.abs(selectAreaMaxX - upperDataAreaX) / dataAreaDrawWidth ;
        }

        INumberTickUnit tickUnit = numberAxis.getTickUnit() ;
        if (tickUnit instanceof LogTickUnit) {
          LogTickUnit logTickUnit = (LogTickUnit) tickUnit ;
          String[] labels = logTickUnit.getExponentLabels() ;
          NumberRange range = new NumberRange(Double.parseDouble(labels[0]),
                                              Double.parseDouble(labels[labels.length - 1])) ;
          NumberRange tmpZoomRange = range.zoomRange(lowerPercent, upperPercent,
                                                     numberAxis.isInverted()) ;
          double rangeLower = Math.pow(logTickUnit.getLogarithmBase(), tmpZoomRange.getLowerBound()) ;
          double rangeUpper = Math.pow(logTickUnit.getLogarithmBase(), tmpZoomRange.getUpperBound()) ;
          NumberRange zoomRange = new NumberRange(rangeLower, rangeUpper) ;
          numberAxis.setZoomDatasetRange(zoomRange) ;
        } else {
          NumberRange zoomRange = datasetRange.zoomRange(lowerPercent, upperPercent,
                                                         numberAxis.isInverted()) ;
          numberAxis.setZoomDatasetRange(zoomRange) ;
        }

      }
    }
  }

  /**
   * <zh>ָvalueֵrenderͼμ.</zh>
   * <en>Returns the RenderShapeCollection for a value axis index.</en>
   *
   * @param valueAxisIndex int <zh>value.</zh>
   * <en>The value axis index.</en>
   * @return RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>The render shape collection (maybe null).</en>
   */
  protected RenderShapeCollection getRenderShapeCollection(int valueAxisIndex) {
    Integer renderIndex = (Integer)this.valueAxisToRenderMap.get(new Integer(valueAxisIndex)) ;
    if (renderIndex != null) {
      int index = renderIndex.intValue() ;
      if (index >= 0 && index < this.renderShapesList.size()) {
        return (RenderShapeCollection)this.renderShapesList.get(renderIndex.intValue()) ;
      }
    }

    if (this.renderShapesList.size() > 0) {
      return (RenderShapeCollection)this.renderShapesList.get(0) ;
    }

    return null ;
  }

  /**
   * <zh>renderͼμ.</zh>
   * <en>Allocate render shapes.</en>
   *
   * @param renderSize int <zh>renderĸ.</zh>
   * <en>The size of renders.</en>
   */
  protected void allocateRenderShapesList(int renderSize) {
    this.renderShapesList.clear() ;
    for (int i = 0 ; i < renderSize ; i++) {
      this.renderShapesList.add(new RenderShapeCollection()) ;
    }

    //һrenderͼμ
    if (this.renderShapesList.size() == 0) {
      this.renderShapesList.add(new RenderShapeCollection()) ;
    }
  }

  /**
   * <zh>Ƶ0value᷽.</zh>
   * <en>Draws the first value axis gridlines for the plot.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param dataArea Rectangle2D <zh>ͼdata() (Ϊnull).</zh>
   * <en>The area within which the data should be drawn (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param valueAxesSize int <zh>valueĸ.</zh>
   * <en>The count of value axes.</en>
   */
  protected void drawValueGridlines(Graphics2D g2, Rectangle2D dataArea,
                                    RenderShapeCollection renderShapes, int valueAxesSize) {
    if (isValueGridlineVisible() && valueAxesSize > 0) {
      RectangleEdge axisEdge = getValueAxisEdge() ;
      Stroke gridStroke = getValueGridlineStroke() ;
      Paint gridPaint = getValueGridlinePaint() ;

      AbstractAxis axis = getValueAxis() ;
      axis.drawGridLine(g2, dataArea, axisEdge, renderShapes, gridStroke, gridPaint) ;
    }
  }

  /**
   * <zh>3D͵chart,Ҫƶmarkerͼ.</zh>
   * <en>Translate marker shape for 3D chart.</en>
   */
  protected void translateMarkerShapes() {
    if (renderShapesList != null && this instanceof IEffect3D) {
      IEffect3D effect = (IEffect3D)this ;
      double offsetX = orientation.equals(ChartOrientation.VERTICAL) ? effect.getXOffset() : 0 ;
      double offsetY = orientation.equals(ChartOrientation.HORIZONTAL) ? effect.getYOffset() : 0 ;

      for (int j = 0 ; j < renderShapesList.size() ; j++) {
        RenderShapeCollection renderShapes = (RenderShapeCollection) renderShapesList.get(j) ;

        List markerShapes = renderShapes.getBackgroundMarkerShapes() ;
        for (int i = 0 ; i < markerShapes.size() ; i++) {
          ((IChartShape) markerShapes.get(i)).translate( -offsetX, offsetY) ;
        }

        markerShapes = renderShapes.getForegroundMarkerShapes() ;
        for (int i = 0 ; i < markerShapes.size() ; i++) {
          ((IChartShape) markerShapes.get(i)).translate( -offsetX, offsetY) ;
        }

        markerShapes = renderShapes.getBackgroundIntervalMarkerShapes() ;
        for (int i = 0 ; i < markerShapes.size() ; i++) {
          ((IChartShape) markerShapes.get(i)).translate( -offsetX, offsetY) ;
        }

        markerShapes = renderShapes.getForegroundIntervalMarkerShapes() ;
        for (int i = 0 ; i < markerShapes.size() ; i++) {
          ((IChartShape) markerShapes.get(i)).translate( -offsetX, offsetY) ;
        }
      }
    }
  }

  /**
   * Returns a clone of the plot.
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException  if the cloning is not supported.
   */
  public Object clone() throws CloneNotSupportedException {
    AbstractAxisPlot clone = (AbstractAxisPlot)super.clone() ;

    return clone ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests the plot for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪȽϵĶ (Ϊnull).</zh>
   * <en>The object to test against (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof AbstractAxisPlot)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    AbstractAxisPlot that = (AbstractAxisPlot) obj ;

    if (!EqualsUtil.equals(this.orientation, that.orientation)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.axisOffset, that.axisOffset)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.valueAxisToRenderMap, that.valueAxisToRenderMap)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.domainAxes, that.domainAxes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.valueAxes, that.valueAxes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.valueAxisLocations, that.valueAxisLocations)) {
      return false ;
    }
    if (this.domainGridlineVisible != that.domainGridlineVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.domainGridlineStroke, that.domainGridlineStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.domainGridlinePaint, that.domainGridlinePaint)) {
      return false ;
    }
    if (this.valueGridlineVisible != that.valueGridlineVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.valueGridlineStroke, that.valueGridlineStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.valueGridlinePaint, that.valueGridlinePaint)) {
      return false ;
    }
    if (this.domainCrosshairVisible != that.domainCrosshairVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.domainCrosshairStroke, that.domainCrosshairStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.domainCrosshairPaint, that.domainCrosshairPaint)) {
      return false ;
    }
    if (this.domainCrosshairLockedOnData != that.domainCrosshairLockedOnData) {
      return false ;
    }
    if (this.valueCrosshairVisible != that.valueCrosshairVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.valueCrosshairStroke, that.valueCrosshairStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.valueCrosshairPaint, that.valueCrosshairPaint)) {
      return false ;
    }
    if (this.valueCrosshairLockedOnData != that.valueCrosshairLockedOnData) {
      return false ;
    }
    if (this.renderShapesList != that.renderShapesList) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code for this object.</en>
   *
   * @return <zh>hashcode.</zh>
   * <en>A hash code.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 29 * result + HashcodeUtil.hashCode(this.orientation) ;
    result = 29 * result + HashcodeUtil.hashCode(this.axisOffset) ;
    result = 29 * result + HashcodeUtil.hashCode(this.valueAxisToRenderMap) ;
    result = 29 * result + HashcodeUtil.hashCode(this.valueAxisLocations) ;
    result = 29 * result + HashcodeUtil.hashCode(this.domainAxes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.valueAxes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.domainGridlineVisible) ;
    result = 29 * result + HashcodeUtil.hashCode(this.domainGridlineStroke) ;
    result = 29 * result + HashcodeUtil.hashCode(this.domainGridlinePaint) ;
    result = 29 * result + HashcodeUtil.hashCode(this.valueGridlineVisible) ;
    result = 29 * result + HashcodeUtil.hashCode(this.valueGridlineStroke) ;
    result = 29 * result + HashcodeUtil.hashCode(this.valueGridlinePaint) ;
    result = 29 * result + HashcodeUtil.hashCode(this.domainCrosshairVisible) ;
    result = 29 * result + HashcodeUtil.hashCode(this.domainCrosshairStroke) ;
    result = 29 * result + HashcodeUtil.hashCode(this.domainCrosshairPaint) ;
    result = 29 * result + HashcodeUtil.hashCode(this.domainCrosshairLockedOnData) ;
    result = 29 * result + HashcodeUtil.hashCode(this.valueCrosshairVisible) ;
    result = 29 * result + HashcodeUtil.hashCode(this.valueCrosshairStroke) ;
    result = 29 * result + HashcodeUtil.hashCode(this.valueCrosshairPaint) ;
    result = 29 * result + HashcodeUtil.hashCode(this.valueCrosshairLockedOnData) ;
    result = 29 * result + HashcodeUtil.hashCode(this.renderShapesList) ;

    return result ;
  }

  /**
   * <zh>toString().</zh>
   * <en>Override toString method.</en>
   *
   * @return String
   */
  public String toString() {
    return "[AbstractAxisPlot]" ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the output stream.
   *
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;
    SerialUtil.writeStroke(stream, this.domainGridlineStroke) ;
    SerialUtil.writePaint(stream, this.domainGridlinePaint) ;
    SerialUtil.writeStroke(stream, this.valueGridlineStroke) ;
    SerialUtil.writePaint(stream, this.valueGridlinePaint) ;
    SerialUtil.writeStroke(stream, this.domainCrosshairStroke) ;
    SerialUtil.writePaint(stream, this.domainCrosshairPaint) ;
    SerialUtil.writeStroke(stream, this.valueCrosshairStroke) ;
    SerialUtil.writePaint(stream, this.valueCrosshairPaint) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;
    this.domainGridlineStroke = SerialUtil.readStroke(stream) ;
    this.domainGridlinePaint = SerialUtil.readPaint(stream) ;
    this.valueGridlineStroke = SerialUtil.readStroke(stream) ;
    this.valueGridlinePaint = SerialUtil.readPaint(stream) ;
    this.domainCrosshairStroke = SerialUtil.readStroke(stream) ;
    this.domainCrosshairPaint = SerialUtil.readPaint(stream) ;
    this.valueCrosshairStroke = SerialUtil.readStroke(stream) ;
    this.valueCrosshairPaint = SerialUtil.readPaint(stream) ;

    for (int i = 0 ; i < this.domainAxes.size() ; i++) {
      CategoryAxis xAxis = (CategoryAxis)this.domainAxes.get(i) ;
      if (xAxis != null) {
        xAxis.removeAllChangeListener() ;
        xAxis.addChangeListener(this) ;
      }
    }
    for (int i = 0 ; i < this.valueAxes.size() ; i++) {
      AbstractValueAxis yAxis = (AbstractValueAxis)this.valueAxes.get(i) ;
      if (yAxis != null) {
        yAxis.removeAllChangeListener() ;
        yAxis.addChangeListener(this) ;
      }
    }
  }

}
