package com.ijchart.xychart.plot ;

import java.awt.Color ;
import java.awt.Graphics2D ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.Collection ;
import java.util.Collections ;
import java.util.Iterator ;
import java.util.List ;

import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.chartinfo.AxisShapeCollection ;
import com.ijchart.xychart.chartinfo.ChartShapeCollection ;
import com.ijchart.xychart.chartinfo.PlotShapeCollection ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.PlotType ;
import com.ijchart.xychart.constant.RenderDrawOrder ;
import com.ijchart.xychart.constant.XYValueAxisType ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.event.PlotChangeEvent ;
import com.ijchart.xychart.render.AbstractRender ;
import com.ijchart.xychart.render.noaxis.AbstractNoAxisRender ;
import com.ijchart.xychart.render.noaxis.PolarRender ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.RectangleInsets ;
import com.ijchart.xychart.ui.supplier.IDrawingSupplier ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ûplot.</zh>
 * <en>The plot with no axis.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class NoAxisPlot extends AbstractPlot implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 1166180130999173352L ;

  /**
   * <zh>render.</zh>
   * <en>Storage for the renders.</en>
   */
  private List renders ;

  /**
   * <zh>ȱʡĹ.</zh>
   * <en>Default constructor.</en>
   */
  public NoAxisPlot() {
    this(null) ;
  }

  /**
   * <zh>һµĶ.</zh>
   * <en>Creates a new plot.</en>
   *
   * @param render AbstractNoAxisRender <zh> (Ϊnull).</zh>
   * <en>The item render (null permitted).</en>
   *
   */
  public NoAxisPlot(AbstractNoAxisRender render) {
    this.renders = new ArrayList() ;
    setListObject(renders, 0, render) ;
    if (render != null) {
      render.addChangeListener(this) ;
    }

    this.renderShapesList = new ArrayList() ;
    setBackgroundPaint(Color.lightGray) ;
  }

  /**
   * <zh>ص0render.</zh>
   * <en>Returns a reference to the render for the plot.</en>
   *
   * @return AbstractNoAxisRender <zh>0render</zh>
   * <en>The render.</en>
   */
  public AbstractNoAxisRender getRender() {
    return getRender(0) ;
  }

  /**
   * <zh>ֵȡrender.</zh>
   * <en>Returns the render at the given index.</en>
   *
   * @param index <zh>ֵ,ڵ0.</zh>
   * <en>The render index (zero-based).</en>
   *
   * @return AbstractNoAxisRender <zh>render(Ϊnull).</zh>
   * <en>The render (possibly null).</en>
   */
  public AbstractNoAxisRender getRender(int index) {
    if (index < 0) {
      throw new IllegalArgumentException("Must be 'index'>=0.") ;
    }

    AbstractNoAxisRender result = null ;
    if (this.renders.size() > index) {
      result = (AbstractNoAxisRender)this.renders.get(index) ;
    }

    return result ;
  }

  /**
   * <zh>render,˼ϲ޸.</zh>
   * <en>Returns the collection of renders,and this collection is unmodifiable.</en>
   *
   * @return Collection <zh>render.</zh>
   * <en>The collection of renders.</en>
   */
  public Collection getRenders() {
    return Collections.unmodifiableCollection(this.renders) ;
  }

  /**
   * <zh>õ0render.</zh>
   * <en>Sets the render at index 0 (sometimes referred to as the "primary" render) and sends
   * a {@link PlotChangeEvent} to all registered listeners.</en>
   *
   * @param render AbstractNoAxisRender <zh>render (Ϊnull).</zh>
   * <en>The render (null not permitted.</en>
   */
  public void setRender(AbstractNoAxisRender render) {
    setRender(0, render, true) ;
  }

  /**
   * <zh>õ0render,Ҫм{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the render at index 0 (sometimes referred to as the "primary" render) and,
   * if requested, sends a {@link PlotChangeEvent} to all registered listeners.</en>
   *
   * @param render  AbstractNoAxisRender <zh>render (Ϊnull).</zh>
   * <en>The render (null not permitted).</en>
   * @param notify  boolean <zh>Ƿ֪ͨ.</zh>
   * <en>Whether or not notify listeners.</en>
   */
  public void setRender(AbstractNoAxisRender render, boolean notify) {
    setRender(0, render, notify) ;
  }

  /**
   * <zh>趨ֵָrender,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the render at the specified index and sends a
   * {@link PlotChangeEvent} to all registered listeners.</en>
   *
   * @param index int <zh>ֵ,ڵ0.</zh>
   * <en>The index (zero-based).</en>
   * @param render AbstractNoAxisRender <zh>render (Ϊnull).</zh>
   * <en>The render (null not permitted).</en>
   */
  public void setRender(int index, AbstractNoAxisRender render) {
    setRender(index, render, true) ;
  }

  /**
   * <zh>趨ֵָrender,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets a render.  A {@link PlotChangeEvent} is sent to all registered listeners.</en>
   *
   * @param index int <zh>ֵ,ڵ0.</zh>
   * <en>The dataset index (zero-based).</en>
   * @param render AbstractNoAxisRender <zh>render (Ϊnull).</zh>
   * <en>The render (null not permitted).</en>
   * @param notify  boolean <zh>Ƿ֪ͨ.</zh>
   * <en>Whether or not notify listeners.</en>
   */
  public void setRender(int index, AbstractNoAxisRender render, boolean notify) {
    if (render == null) {
      throw new IllegalArgumentException("Null 'render' argument.") ;
    }

    // stop listening to the existing render...
    if (index < this.renders.size()) {
      AbstractNoAxisRender existing = (AbstractNoAxisRender)this.renders.get(index) ;
      if (existing != null) {
        existing.removeChangeListener(this) ;
      }
    }

    // register the new render...
    setListObject(renders, index, render) ;
    render.addChangeListener(this) ;

    if (notify) {
      notifyListeners(new PlotChangeEvent(this)) ;
    }
  }

  /**
   * <zh>render,м{@link PlotChangeEvent}¼.</zh>
   * <en>Sets the renders for this plot and sends a {@link PlotChangeEvent}
   * to all registered listeners.</en>
   *
   * @param renders AbstractNoAxisRender[] <zh>render (Ϊnull).</zh>
   * <en>The renders (null not permitted).</en>
   * @param startIndex int <zh>Ŀʼֵ,startIndex֮ǰĶ󲻻ᱻ,ڵ0.</zh>
   * <en>The beginning of the index value, startIndex before the target will not be covered.
   * must be greater than or equal to 0.</en>
   */
  public void setRenders(AbstractNoAxisRender[] renders, int startIndex) {
    if (renders == null) {
      throw new IllegalArgumentException("Null 'renders' argument.") ;
    }
    if (startIndex < 0) {
      throw new IllegalArgumentException("Must be 'startIndex'>=0.") ;
    }

    if (renders.length > 0) {
      for (int i = 0 ; i < renders.length ; i++) {
        setRender(startIndex + i, renders[i], false) ;
      }
      notifyListeners(new PlotChangeEvent(this)) ;
    }
  }

  /**
   * <zh>ָrender,ûҵ򷵻-1.</zh>
   * <en>Returns the index of the specified render, or -1 if the
   * render is not assigned to this plot.</en>
   *
   * @param render AbstractNoAxisRender <zh>render (Ϊnull).</zh>
   * <en>The render (null permitted).</en>
   *
   * @return int <zh>ֵ.</zh>
   * <en>The render index.</en>
   */
  public int getRenderIndex(AbstractNoAxisRender render) {
    return this.renders.indexOf(render) ;
  }

  /**
   * <zh>ͼķ.</zh>
   * <en>Returns the orientation of the plot.</en>
   *
   * @return <zh>null.</zh>
   * <en>Returns null.</en>
   */
  public ChartOrientation getOrientation() {
    return null ;
  }

  /**
   * <zh>ͼplot,Ҫʵִ˷.</zh>
   * <en>Draws the plot within the specified area.Subclasses need to provide an
   * implementation of this method, obviously.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param plotArea Rectangle2D <zh>plot (Ϊnull).</zh>
   * <en>The plot draw area (null not permitted).</en>
   * @param chartShapes ChartShapeCollection <zh>chartͼμ (Ϊnull).</zh>
   * <en>Collects information about the chart (null not permitted).</en>
   */
  public void draw(Graphics2D g2, Rectangle2D plotArea, ChartShapeCollection chartShapes) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (plotArea == null) {
      throw new IllegalArgumentException("Null 'plotArea' argument.") ;
    }
    if (chartShapes == null) {
      throw new IllegalArgumentException("Null 'chartShapes' argument.") ;
    }

    // if the plot area is too small, just return...
    if (plotArea.getWidth() <= DEFAULT_MINIMUM_WIDTH_TO_DRAW ||
        plotArea.getHeight() <= DEFAULT_MINIMUM_HEIGHT_TO_DRAW) {
      return ;
    }

    plotShapes = new PlotShapeCollection() ;
    plotShapes.setDrawArea(plotArea) ;
    chartShapes.addPlotShapes(plotShapes) ;

    ChartOrientation orientation = ChartOrientation.VERTICAL ;

    // adjust the drawing area for the plot insets (if any)...
    RectangleInsets insets = getInsets() ;
    insets.trim(plotArea) ;

    drawBorder(plotArea, plotShapes) ;
    drawBackground(plotArea, orientation, plotShapes) ;
    drawBackgroundImage(plotArea, plotShapes) ;

    //еݼΪ,ʾݼʱϢ
    boolean isAllDatasetEmpty = true ;
    Iterator iter = getRenders().iterator() ;
    while (iter.hasNext()) {
      AbstractRender render = (AbstractRender) iter.next() ;
      if (!render.getDataset().isEmpty()) {
        isAllDatasetEmpty = false ;
        break ;
      }
    }

    if (isAllDatasetEmpty) {
      drawNoDataMessage(plotArea, plotShapes) ;
      return ;
    }

    allocateRenderShapesList(this.renders.size()) ;
    RenderShapeCollection renderShapes = (RenderShapeCollection)this.renderShapesList.get(0) ;
    plotShapes.addRenderShape(renderShapes) ;

    dataArea = plotArea ;
    renderShapes.setDrawArea(dataArea) ;

    if (getRenderDrawOrder().equals(RenderDrawOrder.FORWARD)) {
      IDrawingSupplier supplier = getDrawingSupplier() ;
      supplier.reset() ;
      for (int i = 0 ; i < this.renders.size() ; i++) {
        if (this.renders.get(i) == null
            || !(this.renders.get(i) instanceof AbstractNoAxisRender)) {
          continue ;
        }

        AbstractNoAxisRender render = (AbstractNoAxisRender)this.renders.get(i) ;
        render.setDrawingSupplier(supplier) ;
        if (render instanceof PolarRender) {
          XYNumberValueAxis axis = drawPolarAxis(g2, dataArea, (PolarRender) render, renderShapes) ;
          ((PolarRender) render).setAxis(axis) ;
        }

        if (i == 0) { //0renderƱͼƬ,Ĳ
          render.drawBackground(plotArea, orientation, renderShapes) ;
          render.drawDrawAreaBorder(dataArea, renderShapes) ;
          render.drawBackgroundImage(dataArea, renderShapes) ;

          renderShapes.setDataset(render.getDataset()) ;
          render.drawAllItems(dataArea, renderShapes, getForegroundAlpha()) ;
        } else {
          RenderShapeCollection tmpRenderShapes = (RenderShapeCollection) renderShapesList.get(i) ;
          if (tmpRenderShapes == null) {
            return ;
          }
          tmpRenderShapes.setDrawArea(dataArea) ;
          plotShapes.addRenderShape(tmpRenderShapes) ;

          tmpRenderShapes.setDataset(render.getDataset()) ;
          render.drawAllItems(dataArea, tmpRenderShapes, getForegroundAlpha()) ;
        }

      }
    } else {
      for (int i = this.renders.size() - 1 ; i >= 0 ; i--) {
        if (this.renders.get(i) == null
            || !(this.renders.get(i) instanceof AbstractNoAxisRender)) {
          continue ;
        }

        AbstractNoAxisRender render = (AbstractNoAxisRender)this.renders.get(i) ;
        IDrawingSupplier supplier = getDrawingSupplier() ;
        supplier.reset() ;
        render.setDrawingSupplier(supplier) ;
        if (render instanceof PolarRender) {
          XYNumberValueAxis axis = drawPolarAxis(g2, dataArea, (PolarRender) render, renderShapes) ;
          ((PolarRender) render).setAxis(axis) ;
        }

        if (i == this.renders.size() - 1) { //0renderƱͼƬ,Ĳ
          render.drawBackground(dataArea, orientation, renderShapes) ;
          render.drawDrawAreaBorder(dataArea, renderShapes) ;
          render.drawBackgroundImage(dataArea, renderShapes) ;

          renderShapes.setDataset(render.getDataset()) ;
          render.drawAllItems(dataArea, renderShapes, getForegroundAlpha()) ;
        } else {
          RenderShapeCollection tmpRenderShapes = (RenderShapeCollection) renderShapesList.get(i) ;
          if (tmpRenderShapes == null) {
            return ;
          }
          tmpRenderShapes.setDrawArea(dataArea) ;
          plotShapes.addRenderShape(tmpRenderShapes) ;

          tmpRenderShapes.setDataset(render.getDataset()) ;
          render.drawAllItems(dataArea, tmpRenderShapes, getForegroundAlpha()) ;
        }
      }
    }

  }

  /**
   * <zh>Ƽͼ.</zh>
   * <en>The the polar axis.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param render PolarRender <zh>PolarRender (Ϊnull).</zh>
   * <en>The PolarRender object (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   *
   * @return XYNumberValueAxis <zh>.</zh>
   * <en>The axis.</en>
   */
  protected XYNumberValueAxis drawPolarAxis(Graphics2D g2, Rectangle2D dataArea, PolarRender render,
                                            RenderShapeCollection renderShapes) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument") ;
    }
    if (render == null) {
      throw new IllegalArgumentException("Null 'render' argument") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument") ;
    }

    double margin = Math.max(0, render.getMargin()) ;
    double areaSize = Math.min(dataArea.getWidth(), dataArea.getHeight()) * 0.5 - margin ;
    Rectangle2D axisArea = new Rectangle2D.Double(dataArea.getCenterX(), dataArea.getCenterY(),
                                                  areaSize, areaSize) ;

    RectangleEdge edge = RectangleEdge.TOP ;
    AxisShapeCollection axisShapes = new AxisShapeCollection() ;
    axisShapes.setEdge(edge) ;
    axisShapes.setDrawArea(axisArea) ;

    IMatrixDataset dataset = render.getDataset() ;
    XYNumberValueAxis axis = new XYNumberValueAxis(dataset, XYValueAxisType.XY_VALUE_AXIS_Y) ;
    axis.setAxisLabelVisible(true) ;
    axis.setTickMarkVisible(false) ;
    axis.setAxisLineVisible(false) ;
    axis.setGridLineVisible(false) ;

    axis.draw(g2, axisArea, axisArea, edge, axisShapes, true) ;
    renderShapes.addAllTickLabelShape(axisShapes.getTickLabelShapes()) ;

    return axis ;
  }

  /**
   * <zh>ʮ.</zh>
   * <en>Draws the crosshair,this method is not implemented.</en>
   *
   * @param mouseClickPoint Point2D <zh>ʱ (Ϊnull).</zh>
   * <en>The mouse click point (null permitted).</en>
   *
   * @return boolean <zh>Ƿʮ߷˱仯.</zh>
   * <en>Returns whether or not crosshair is changed.</en>
   */
  public boolean drawCrosshair(Point2D mouseClickPoint) {
    throw new RuntimeException("Method not implented.") ;
  }

  /**
   * <zh>domainǷ.</zh>
   * <en>Returns true if the plot's domain axis is zoomable, and false otherwise.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isDomainZoomable() {
    return false ;
  }

  /**
   * <zh>valueǷ.</zh>
   * <en>Returns true if the plot's value is zoomable, and false otherwise.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isValueZoomable() {
    return false ;
  }

  /**
   * <zh>renderͼμ.</zh>
   * <en>Allocate render shapes.</en>
   *
   * @param renderSize int <zh>renderĸ.</zh>
   * <en>The size of renders.</en>
   */
  protected void allocateRenderShapesList(int renderSize) {
    this.renderShapesList.clear() ;
    for (int i = 0 ; i < renderSize ; i++) {
      this.renderShapesList.add(new RenderShapeCollection()) ;
    }

    //һrenderͼμ
    if (this.renderShapesList.size() == 0) {
      this.renderShapesList.add(new RenderShapeCollection()) ;
    }
  }

  /**
   * <zh>plotַ.</zh>
   * <en>Returns a short string describing the plot type.</en>
   *
   * @return <zh>plot.</zh>
   * <en>The plot type.</en>
   */
  public PlotType getPlotType() {
    return PlotType.PLOT_TYPE_NO_AXIS ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests the plot for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪȽϵĶ (Ϊnull).</zh>
   * <en>The object to test against (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof NoAxisPlot)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    NoAxisPlot that = (NoAxisPlot) obj ;
    if (!EqualsUtil.equals(this.renders, that.renders)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code for this object.</en>
   *
   * @return <zh>hashcode.</zh>
   * <en>A hash code.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 29 * result + HashcodeUtil.hashCode(this.renders) ;

    return result ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Returns a clone of the plot.</en>
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException  if the cloning is not supported.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString().</zh>
   * <en>Override toString method.</en>
   *
   * @return String
   */
  public String toString() {
    return "[NoAxisPlot]" ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    int renderCount = this.renders.size() ;
    for (int i = 0 ; i < renderCount ; i++) {
      AbstractNoAxisRender render = (AbstractNoAxisRender)this.renders.get(i) ;
      if (render != null) {
        render.removeChangeListener(this) ;
        render.addChangeListener(this) ;
      }
    }

  }

  /**
   * Provides serialization support.
   *
   * @param stream  the output stream.
   *
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;
  }

}
