package com.ijchart.xychart.render.noaxis.pie ;

import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.Collections ;
import java.util.List ;

/**
 *
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>бͼⲿǩ.</zh>
 * <en>This class distributes the section labels for one side of a pie chart so
 * that they do not overlap.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class PieLabelDistributor implements Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -9122096507434908527L ;

  /**
   * <zh>PieLabelRecord󼯺.</zh>
   * <en>The label records.</en>
   */
  private List labels ;

  /**
   * <zh>С.</zh>
   * <en> The minimum gap.</en>
   */
  private final double minGap = 4.0 ;

  /**
   * <zh>.</zh>
   * <en>Creates a new distributor.</en>
   */
  public PieLabelDistributor() {
    this.labels = new ArrayList() ;
  }

  /**
   * <zh>PieLabelRecord.</zh>
   * <en>Returns a label record from the list.</en>
   *
   * @param index <zh>.</zh>
   * <en>The index.</en>
   *
   * @return <zh>PieLabelRecord(Ϊnull).</zh>
   * <en>The PieLabelRecord (maybe null).</en>
   */
  public PieLabelRecord getPieLabelRecord(int index) {
    return (PieLabelRecord)this.labels.get(index) ;
  }

  /**
   * <zh>һPieLabelRecord.</zh>
   * <en>Adds a label record.</en>
   *
   * @param record <zh>PieLabelRecord.</zh>
   * <en>The label record (null not permitted).</en>
   */
  public void addPieLabelRecord(PieLabelRecord record) {
    if (record == null) {
      throw new IllegalArgumentException("Null 'record' argument.") ;
    }

    this.labels.add(record) ;
  }

  /**
   * <zh>PieLabelRecord.</zh>
   * <en>Returns the number of items in the list.</en>
   *
   * @return The item count.
   */
  public int getItemCount() {
    return this.labels.size() ;
  }

  /**
   * <zh>ɾеĶ.</zh>
   * <en>Clears the list of labels.</en>
   */
  public void clear() {
    this.labels.clear() ;
  }

  /**
   * <zh>бǩ.</zh>
   * <en>Distributes the labels.</en>
   *
   * @param minY  <zh>Сyֵ.</zh>
   * <en>the minimum y-coordinate in Java2D-space.</en>
   * @param height  <zh>߶.</zh>
   * <en>The available height (in Java2D units).</en>
   */
  public void distributeLabels(double minY, double height) {
    sort() ;

    if (isOverlap()) {
      adjustInwards() ;
    }

    if (isOverlap()) {
      adjustDownwards(minY, height) ;
    }

    if (isOverlap()) {
      adjustUpwards(minY, height) ;
    }

    if (isOverlap()) {
      spreadEvenly(minY, height) ;
    }
  }

  /**
   * <zh>жǷصıǩ.</zh>
   * <en>Returns true if there are overlapping labels in the list,
   * and false otherwise.</en>
   *
   * @return <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  private boolean isOverlap() {
    double y = 0.0 ;
    for (int i = 0 ; i < this.labels.size() ; i++) {
      PieLabelRecord plr = (PieLabelRecord)this.labels.get(i) ;
      if (y > plr.getLowerY()) {
        return true ;
      }
      y = plr.getUpperY() ;
    }

    return false ;
  }

  /**
   * <zh>ڱǩصʱ,ķy.</zh>
   * <en>Adjusts the y-coordinate for the labels in towards the center in an
   * attempt to fix overlapping.</en>
   */
  protected void adjustInwards() {
    int lower = 0 ;
    int upper = this.labels.size() - 1 ;
    while (upper > lower) {
      if (lower < upper - 1) {
        PieLabelRecord r0 = (PieLabelRecord)this.labels.get(lower) ;
        PieLabelRecord r1 = (PieLabelRecord)this.labels.get(lower + 1) ;
        if (r1.getLowerY() < r0.getUpperY()) {
          double adjust = r0.getUpperY() - r1.getLowerY()
                          + this.minGap ;
          r1.setAllocatedY(r1.getAllocatedY() + adjust) ;
        }
      }

      PieLabelRecord r2 = (PieLabelRecord)this.labels.get(upper - 1) ;
      PieLabelRecord r3 = (PieLabelRecord)this.labels.get(upper) ;
      if (r2.getUpperY() > r3.getLowerY()) {
        double adjust = (r2.getUpperY() - r3.getLowerY()) + this.minGap ;
        r3.setAllocatedY(r3.getAllocatedY() + adjust) ;
      }

      lower++ ;
      upper-- ;
    }
  }

  /**
   * <zh>ڱǩصʱ,·y.</zh>
   * <en>Any labels that are overlapping are moved down in an attempt to
   * eliminate the overlaps.</en>
   *
   * @param minY <zh>Сyֵ.</zh>
   * <en>The minimum y value.</en>
   * @param height <zh>߶.</zh>
   * <en>The height available for all labels.</en>
   */
  protected void adjustDownwards(double minY, double height) {
    for (int i = 0 ; i < this.labels.size() - 1 ; i++) {
      PieLabelRecord record0 = (PieLabelRecord)this.labels.get(i) ;
      PieLabelRecord record1 = (PieLabelRecord)this.labels.get(i + 1) ;
      if (record1.getLowerY() < record0.getUpperY()) {
        record1.setAllocatedY(Math.min(minY + height, record0.getUpperY() + this.minGap
                                       + record1.getLabelHeight() * 0.5)) ;
      }
    }
  }

  /**
   * <zh>ڱǩصʱ,Ϸy.</zh>
   * <en>Any labels that are overlapping are moved up in an attempt to eliminate
   * the overlaps.</en>
   *
   * @param minY <zh>Сyֵ.</zh>
   * <en>The minimum y value.</en>
   * @param height <zh>߶.</zh>
   * <en>The height available for all labels.</en>
   */
  protected void adjustUpwards(double minY, double height) {
    for (int i = this.labels.size() - 1 ; i > 0 ; i--) {
      PieLabelRecord record0 = (PieLabelRecord)this.labels.get(i) ;
      PieLabelRecord record1 = (PieLabelRecord)this.labels.get(i - 1) ;
      if (record1.getUpperY() > record0.getLowerY()) {
        record1.setAllocatedY(Math.max(minY, record0.getLowerY() - this.minGap
                                       - record1.getLabelHeight() * 0.5)) ;
      }
    }
  }

  /**
   * <zh>ѱǩо.</zh>
   * <en>Labels are spaced evenly in the available space in an attempt to
   * eliminate the overlaps..</en>
   *
   * @param minY <zh>Сyֵ.</zh>
   * <en>The minimum y value.</en>
   * @param height <zh>߶.</zh>
   * <en>The height available for all labels.</en>
   */
  protected void spreadEvenly(double minY, double height) {
    double y = minY ;
    double sumOfLabelHeights = 0.0 ;
    for (int i = 0 ; i < this.labels.size() ; i++) {
      sumOfLabelHeights += ((PieLabelRecord)this.labels.get(i)).getLabelHeight() ;
    }

    double gap = Math.max(0, height - sumOfLabelHeights) ;
    if (this.labels.size() > 1) {
      gap /= (this.labels.size() - 1) ;
    }

    for (int i = 0 ; i < this.labels.size() ; i++) {
      PieLabelRecord record = (PieLabelRecord)this.labels.get(i) ;
      y += record.getLabelHeight() * 0.5 ;
      record.setAllocatedY(y) ;
      y = y + record.getLabelHeight() * 0.5 + gap ;
    }
  }

  /**
   * <zh>yֵ.</zh>
   * <en>Sorts the label records into ascending order by y-value.</en>
   */
  public void sort() {
    Collections.sort(this.labels) ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "PieLabelDistributor" ;
  }

}
