package com.ijchart.xychart.render.xy ;

import java.awt.geom.GeneralPath ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.YIntervalChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.shape.GeneralPathChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ƫXYͼͼ.</zh>
 * <en>A specialised subclass of the {@link com.ijchart.xychart.render.xy.XYLineAndShapeRender}
 *  that represents the y-interval by shading an area behind the y-values on the chart.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DeviationRender extends XYLineAndShapeRender implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -8497752197307397614L ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public DeviationRender(IMatrixDataset dataset) {
    super(dataset) ;
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param xAxis AbstractValueAxis <zh>xֵ (Ϊnull).</zh>
   * <en>The x value axis (null not permitted).</en>
   * @param xAxisEdge RectangleEdge <zh>xλ(Ϊnull).</zh>
   * <en>The x axis edge.</en>
   * @param yAxis AbstractValueAxis <zh>yֵ (Ϊnull).</zh>
   * <en>The y value axis (null not permitted).</en>
   * @param yAxisEdge RectangleEdge <zh>yλ (Ϊnull).</zh>
   * <en>The y axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(AbstractValueAxis xAxis, RectangleEdge xAxisEdge,
                           AbstractValueAxis yAxis, RectangleEdge yAxisEdge,
                           Rectangle2D dataArea, ChartOrientation orientation,
                           RenderShapeCollection renderShapes, float alpha) {
    super.drawAllItems(xAxis, xAxisEdge, yAxis, yAxisEdge, dataArea,
                       orientation, renderShapes, alpha) ;

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }

    int rows = dataset.getRowCount() ;
    this.fillPaints = getFillPaints(rows) ;
    getBorderPaints(rows) ;
    getBorderStrokes(rows) ;

    //ߺͼ
    for (int i = 0 ; i < rows ; i++) {
      int currRowCols = dataset.getColumnCount(i) ;
      for (int j = 0 ; j < currRowCols - 1 ; j++) {
        IChartData currData = dataset.getData(i, j) ;
        IChartData nextData = dataset.getData(i, j + 1) ;
        if (currData != null && currData instanceof YIntervalChartData
            && nextData != null && nextData instanceof YIntervalChartData) {
          double currX = ((YIntervalChartData) currData).getX() ;
          double currYLow = ((YIntervalChartData) currData).getYLow() ;
          double currYHigh = ((YIntervalChartData) currData).getYHigh() ;
          double nextX = ((YIntervalChartData) nextData).getX() ;
          double nextYLow = ((YIntervalChartData) nextData).getYLow() ;
          double nextYHigh = ((YIntervalChartData) nextData).getYHigh() ;
          if (Double.isNaN(currX) || Double.isNaN(currYLow) || Double.isNaN(currYHigh)
              || Double.isNaN(nextX) || Double.isNaN(nextYLow) || Double.isNaN(nextYHigh)) {
            renderShapes.addDeviationShape(null) ;
            continue ;
          }

          double currXPos = xAxis.valueToJava2D(dataArea, xAxisEdge, currX, false) ;
          double currYLowPos = yAxis.valueToJava2D(dataArea, yAxisEdge, currYLow, false) ;
          double currYHighPos = yAxis.valueToJava2D(dataArea, yAxisEdge, currYHigh, false) ;
          double nextXPos = xAxis.valueToJava2D(dataArea, xAxisEdge, nextX, false) ;
          double nextYLowPos = yAxis.valueToJava2D(dataArea, yAxisEdge, nextYLow, false) ;
          double nextYHighPos = yAxis.valueToJava2D(dataArea, yAxisEdge, nextYHigh, false) ;

          //ֵ᷶Χֵ
          if (currXPos == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || currYLowPos == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || currYHighPos == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || nextXPos == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || nextYLowPos == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || nextYHighPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
            renderShapes.addDeviationShape(null) ;
            continue ;
          }

          GeneralPath path = new GeneralPath() ;
          if (orientation.equals(ChartOrientation.VERTICAL)) { //ֱ
            path.moveTo((float) currXPos, (float) Math.min(currYLowPos, currYHighPos)) ;
            path.lineTo((float) currXPos, (float) Math.max(currYLowPos, currYHighPos)) ;
            path.lineTo((float) nextXPos, (float) Math.max(nextYLowPos, nextYHighPos)) ;
            path.lineTo((float) nextXPos, (float) Math.min(nextYLowPos, nextYHighPos)) ;
            path.lineTo((float) currXPos, (float) Math.min(currYLowPos, currYHighPos)) ;
          } else if (orientation.equals(ChartOrientation.HORIZONTAL)) { //ˮƽ
            path.moveTo((float) Math.min(currYLowPos, currYHighPos), (float) currXPos) ;
            path.lineTo((float) Math.max(currYLowPos, currYHighPos), (float) currXPos) ;
            path.lineTo((float) Math.max(nextYLowPos, nextYHighPos), (float) nextXPos) ;
            path.lineTo((float) Math.min(nextYLowPos, nextYHighPos), (float) nextXPos) ;
            path.lineTo((float) Math.min(currYLowPos, currYHighPos), (float) currXPos) ;
          }
          path.closePath() ;

          GeneralPathChartShape deviationShape = new GeneralPathChartShape(path) ;
          IChartShape chartShape = createItemShape(orientation, yAxis, i, j, null,
                                                   fillPaints[i], borderPaints[i],
                                                   borderStrokes[i], alpha, deviationShape) ;
          chartShape.setBorderPaint(chartShape.getFillPaint()) ;
          chartShape.setDrawBorder(true) ;

          renderShapes.addDeviationShape(chartShape) ;
        } else {
          renderShapes.addDeviationShape(null) ;
        }
      }
    }

  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof XYLineAndShapeRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    return super.hashCode() ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "DeviationRender" ;
  }

}
