package com.ijchart.xychart.render.xy ;

import java.awt.geom.GeneralPath ;
import java.awt.geom.Line2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.VectorChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.shape.GeneralPathChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>XYͼ.</zh>
 * <en>Draw xy vector shapes.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class VectorRender extends AbstractXYRender implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -8354530224829336944L ;

  /**
   * <zh>ȱʡļͷĳ.</zh>
   * <en>The default length of the arrow head.</en>
   */
  public static final double DEFAULT_ARROW_HEAD_LENGTH = 0.14D ;

  /**
   * <zh>ȱʡļĳ.</zh>
   * <en>The default length of the arrow shaft.</en>
   */
  public static final double DEFAULT_ARROW_SHAFT_LENGTH = 0.10D ;

  /**
   * <zh>ͷĳ.</zh>
   * <en>The length of the arrow head.</en>
   */
  private double arrowHeadLength ;

  /**
   * <zh>ĳ.</zh>
   * <en>The length of the arrow shaft.</en>
   */
  private double arrowShaftLength ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public VectorRender(IMatrixDataset dataset) {
    super(dataset) ;

    setBaseItemDrawBorder(true) ;
  }

  /**
   * <zh>ؼͷĳ.</zh>
   * <en>Returns the length of the arrow head.</en>
   *
   * @return double <zh>.</zh>
   * <en>The length.</en>
   */
  public double getArrowHeadLength() {
    return this.arrowHeadLength ;
  }

  /**
   * <zh>üͷĳ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the length of the arrow head and sends a
   *  {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param length double <zh>.</zh>
   * <en>The length.</en>
   */
  public void setArrowHeadLength(double length) {
    if (length <= 0) {
      throw new IllegalArgumentException("Argument 'length' must be >0.") ;
    }

    if (this.arrowHeadLength != length) {
      this.arrowHeadLength = length ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ؼĳ.</zh>
   * <en>Returns the length of the arrow shaft.</en>
   *
   * @return double <zh>.</zh>
   * <en>The length.</en>
   */
  public double getArrowShaftLength() {
    return this.arrowShaftLength ;
  }

  /**
   * <zh>üĳ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the length of the arrow shaft and sends a
   *  {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param length double <zh>.</zh>
   * <en>The length.</en>
   */
  public void setArrowShaftLength(double length) {
    if (length <= 0) {
      throw new IllegalArgumentException("Argument 'length' must be >0.") ;
    }

    if (this.arrowShaftLength != length) {
      this.arrowShaftLength = length ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param xAxis AbstractValueAxis <zh>xֵ (Ϊnull).</zh>
   * <en>The x value axis (null not permitted).</en>
   * @param xAxisEdge RectangleEdge <zh>xλ(Ϊnull).</zh>
   * <en>The x axis edge.</en>
   * @param yAxis AbstractValueAxis <zh>yֵ (Ϊnull).</zh>
   * <en>The y value axis (null not permitted).</en>
   * @param yAxisEdge RectangleEdge <zh>yλ (Ϊnull).</zh>
   * <en>The y axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(AbstractValueAxis xAxis, RectangleEdge xAxisEdge,
                           AbstractValueAxis yAxis, RectangleEdge yAxisEdge,
                           Rectangle2D dataArea, ChartOrientation orientation,
                           RenderShapeCollection renderShapes, float alpha) {
    if (xAxis == null) {
      throw new IllegalArgumentException("Null 'xAxis' argument.") ;
    }
    if (xAxisEdge == null) {
      throw new IllegalArgumentException("Null 'xAxisEdge' argument.") ;
    }
    if (yAxis == null) {
      throw new IllegalArgumentException("Null 'yAxis' argument.") ;
    }
    if (yAxisEdge == null) {
      throw new IllegalArgumentException("Null 'yAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    if (getDataset().isEmpty()) {
      return ;
    }

    IMatrixDataset dataset = getDataset() ;
    int rows = dataset.getRowCount() ;

    this.fillPaints = getFillPaints(rows) ;
    this.borderPaints = getBorderPaints(rows) ;
    this.borderStrokes = getBorderStrokes(rows) ;

    for (int i = 0 ; i < rows ; i++) {
      int currRowCols = dataset.getColumnCount(i) ;
      for (int j = 0 ; j < currRowCols ; j++) {
        IChartData data = dataset.getData(i, j) ;
        if (data != null && data instanceof VectorChartData) {
          VectorChartData vectorData = (VectorChartData) data ;
          double x = vectorData.getX() ;
          double y = vectorData.getY() ;
          double vectorX = vectorData.getDeltaX() ;
          double vectorY = vectorData.getDeltaY() ;

          double xx0 = xAxis.valueToJava2D(dataArea, xAxisEdge, x, false) ;
          double yy0 = yAxis.valueToJava2D(dataArea, yAxisEdge, y, false) ;
          double xx1 = xAxis.valueToJava2D(dataArea, xAxisEdge, x + vectorX, false) ;
          double yy1 = yAxis.valueToJava2D(dataArea, yAxisEdge, y + vectorY, false) ;

          //ֵ᷶Χֵ
          if (xx0 == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || yy0 == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || xx1 == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || yy1 == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
            renderShapes.addDataItemShape(null) ;
            continue ;
          }

          Line2D line = null ;
          if (orientation.equals(ChartOrientation.HORIZONTAL)) {
            line = new Line2D.Double(yy0, xx0, yy1, xx1) ;
          } else {
            line = new Line2D.Double(xx0, yy0, xx1, yy1) ;
          }

          double dxx = (xx1 - xx0) ;
          double dyy = (yy1 - yy0) ;
          double bx = xx0 + (1.0 - this.arrowShaftLength) * dxx ;
          double by = yy0 + (1.0 - this.arrowShaftLength) * dyy ;

          double cx = xx0 + (1.0 - this.arrowHeadLength) * dxx ;
          double cy = yy0 + (1.0 - this.arrowHeadLength) * dyy ;

          double angle = 0.0 ;
          if (dxx != 0.0) {
            angle = Math.PI / 2.0 - Math.atan(dyy / dxx) ;
          }
          double deltaX = 2.0 * Math.cos(angle) ;
          double deltaY = 2.0 * Math.sin(angle) ;

          double leftx = cx + deltaX ;
          double lefty = cy - deltaY ;
          double rightx = cx - deltaX ;
          double righty = cy + deltaY ;

          GeneralPath path = new GeneralPath() ;
          GeneralPath hotspotPath = null ;
          if (orientation.equals(ChartOrientation.VERTICAL)) {
            path.moveTo((float) xx1, (float) yy1) ;
            path.lineTo((float) rightx, (float) righty) ;
            path.lineTo((float) bx, (float) by) ;
            path.lineTo((float) leftx, (float) lefty) ;

            hotspotPath = (GeneralPath) path.clone() ;
          } else {
            path.moveTo((float) yy1, (float) xx1) ;
            path.lineTo((float) righty, (float) rightx) ;
            path.lineTo((float) by, (float) bx) ;
            path.lineTo((float) lefty, (float) leftx) ;

            hotspotPath = (GeneralPath) path.clone() ;
          }
          path.append(line, false) ;
          hotspotPath.append(line, true) ;

          GeneralPathChartShape chartShape = new GeneralPathChartShape(path, hotspotPath) ;
          IChartShape pathShape = createItemShape(orientation, yAxis, i, j, null, fillPaints[i],
                                                  fillPaints[i], borderStrokes[i], alpha,
                                                  chartShape) ;
          renderShapes.addDataItemShape(pathShape) ;
        } else {
          renderShapes.addDataItemShape(null) ;
        }
      }
    }
  }

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public LegendType getLegendType() {
    return LegendType.ROW ;
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    return (IChartShape) appendParam ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates if some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof VectorRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    VectorRender that = (VectorRender) obj ;
    if (this.arrowHeadLength != that.arrowHeadLength) {
      return false ;
    }
    if (this.arrowShaftLength != that.arrowShaftLength) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double result = super.hashCode() ;
    result = result * 37 + 100 * this.arrowHeadLength ;
    result = result * 37 + 100 * this.arrowShaftLength ;

    return (int) result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "VectorRender" ;
  }

}
