package com.ijchart.xychart.shape ;

import java.awt.AlphaComposite ;
import java.awt.Graphics2D ;
import java.awt.Shape ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Line2D ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;
import com.ijchart.xychart.util.StrokeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>˫ͷͼ.</zh>
 * <en>A chart shape composited by a line and two shapes.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DblHeadMatchstickChartShape extends AbstractRectangleTextChartShape implements
  Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 1771132206661071501L ;

  /**
   * <zh>СΪ0ľ.</zh>
   * <en>A constant representing zero rectangle.</en>
   */
  public static final Rectangle2D ZERO_RECT = new Rectangle2D.Double(0, 0, 0, 0) ;

  /**
   * <zh>ͼζ.</zh>
   * <en>A shape.</en>
   */
  private transient Shape shape ;

  /**
   * <zh>һֱ.</zh>
   * <en>A line.</en>
   */
  private transient Line2D line ;

  /**
   * <zh>.</zh>
   * <en>A constructor.</en>
   *
   * @param shape Shape <zh>ͼζ (Ϊnull).</zh>
   * <en>A shape (null permitted).</en>
   * @param line Line2D <zh> (Ϊnull).</zh>
   * <en>A line (null not permitted).</en>
   */
  public DblHeadMatchstickChartShape(Shape shape, Line2D line) {
    this(shape, line, null) ;
  }

  /**
   * <zh>.</zh>
   * <en>A constructor.</en>
   *
   * @param shape Shape <zh>ͼζ (Ϊnull).</zh>
   * <en>A shape (null permitted).</en>
   * @param line Line2D <zh> (Ϊnull).</zh>
   * <en>A line (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   */
  public DblHeadMatchstickChartShape(Shape shape, Line2D line, String text) {
    super((shape == null) ? ZERO_RECT : shape.getBounds2D(), text) ;
    if (line == null) {
      throw new IllegalArgumentException("Null 'line' argument") ;
    }

    this.shape = shape ;
    this.line = line ;
    setForceShowAllText(true) ;
    setTextPosition(TextPosition.OUTSIDE_TOP) ;
    setTextHorizontalAlign(HorizontalAlignment.CENTER) ;
  }

  /**
   * <zh>ͼζ.</zh>
   * <en>Returns the shape.</en>
   *
   * @return Shape <zh>ͼζ (Ϊnull).</zh>
   * <en>A shape (maybe null).</en>
   */
  public Shape getShape() {
    return this.shape ;
  }

  /**
   * <zh>ͼζ,м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the shape,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param shape Shape <zh>ͼζ (Ϊnull).</zh>
   * <en>A shape (null permitted).</en>
   */
  public void setShape(Shape shape) {
    if (!(this.shape == null && shape == null)) {
      this.shape = shape ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh> (Ϊnull).</zh>
   * <en>A line (null permitted).</en>
   *
   * @return Line2D <zh> (Ϊnull).</zh>
   * <en>A line (maybe null).</en>
   */
  public Line2D getLine() {
    return this.line ;
  }

  /**
   * <zh>,м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the line,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param line Line2D <zh> (Ϊnull).</zh>
   * <en>A line (null not permitted).</en>
   */
  public void setLine(Line2D line) {
    if (line == null) {
      throw new IllegalArgumentException("Null 'line' argument") ;
    }

    this.line = line ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ</zh>
   * <en>Draw the ChartShape on a Java 2d graphics device(such as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    g2.translate(this.line.getX1(), this.line.getY1()) ;
    g2.rotate(getRotate()) ;
    beforeDraw(g2) ;
//  g2.setClip(cloneArea) ;

    Line2D newLine = new Line2D.Double(0, 0, this.line.getX2() - this.line.getX1(),
                                       this.line.getY2() - this.line.getY1()) ;
    drawBorder(g2, newLine) ;
    drawBackground(g2, this.shape) ;
    drawBorder(g2, this.shape) ;
    drawText(g2, getDrawArea()) ;

    g2.translate(newLine.getX2(), newLine.getY2()) ;
    drawBackground(g2, this.shape) ;
    drawBorder(g2, this.shape) ;

    afterDraw(g2) ;
    state.restoreOldState() ;
    drawSelect(g2) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics device(such as the screen or the
   * printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  protected void drawSelect(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }

    if (isSelected()) {
      double x1 = this.line.getX1() ;
      double y1 = this.line.getY1() ;
      double x2 = this.line.getX2() ;
      double y2 = this.line.getY2() ;

      Graphics2DState state = new Graphics2DState(g2) ;
      state.saveOldState() ;

      g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getSelectRectAlpha())) ;

      double rectWidth = getSelectRectWidth() ;
      final double halfRect = rectWidth / 2 ;

      g2.setPaint(getSelectAnchorPaint()) ;
      g2.translate(x1, y1) ;
      g2.rotate(getRotate()) ;

      //ê
      if (y1 == y2) { //ˮƽ
        g2.fill(new Rectangle2D.Double(0, -halfRect, rectWidth, rectWidth)) ;
        g2.fill(new Rectangle2D.Double(x2 - x1 - rectWidth, -halfRect,
                                       rectWidth, rectWidth)) ;
      } else if (x1 == x2) { //ֱ
        g2.fill(new Rectangle2D.Double( -halfRect, 0, rectWidth, rectWidth)) ;
        g2.fill(new Rectangle2D.Double( -halfRect, y2 - y1 - rectWidth,
                                       rectWidth, rectWidth)) ;
      } else {
        //ˮƽֱê㾭ƫתбê
        double theta = Math.atan((y2 - y1) / (x2 - x1)) ; //бߵƫת
        if (y1 > y2 && x1 > x2) {
          theta -= Math.PI ;
        }

        double lineLen = Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2)) ; //֮ľ
        g2.rotate(theta) ;

        g2.fill(new Rectangle2D.Double(0, -halfRect, rectWidth, rectWidth)) ;
        g2.fill(new Rectangle2D.Double(lineLen - rectWidth, -halfRect, rectWidth, rectWidth)) ;

        g2.rotate( -theta) ;
      }

      g2.rotate( -getRotate()) ;
      state.restoreOldState() ;
    }

  }

  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  protected void calculateTransformedShape() {
    if (transformedShape == null) {
      AffineTransform transform = new AffineTransform() ;
      transform.translate(this.line.getX1(), this.line.getY1()) ;
      transform.rotate(getRotate()) ;

      GeneralPath path = new GeneralPath() ;
      Line2D newLine = new Line2D.Double(0, 0, this.line.getX2() - this.line.getX1(),
                                         this.line.getY2() - this.line.getY1()) ;
      if (this.shape != null) {
        path.append(this.shape.getPathIterator(null), false) ;
      }
      Shape shape = StrokeUtil.lineToRectangle(newLine, getBorderStroke(), 0) ;
      path.append(shape, false) ;
      path.lineTo((float) newLine.getX2(), (float) newLine.getY2()) ;
      if (this.shape != null) {
        path.append(this.shape.getPathIterator(null), false) ;
      }

      transformedShape = transform.createTransformedShape(path) ;
    }
  }

  /**
   * <zh>ƶͼ,X᷽ƫtx,Y᷽ƫty,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Concatenates the current Graphics2D Transform with a translation transform.and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.
   * Subsequent rendering is translated by the specified
   * distance relative to the previous position.
   * This is equivalent to calling transform(T), where T is an
   * AffineTransform represented by the following matrix:
   * <pre>
   *		[   1    0    tx  ]
   *		[   0    1    ty  ]
   *		[   0    0    1   ]
   * </pre>.</en>
   *
   * @param tx double <zh>X᷽ľ.</zh>
   * <en>The distance to translate along the x-axis.</en>
   * @param ty double <zh>Y᷽ľ.</zh>
   * <en>The distance to translate along the y-axis.</en>
   */
  public void translate(double tx, double ty) {
    if (!(tx == 0 && ty == 0)) {
      if (this.line != null) {
        this.line.setLine(this.line.getX1() + tx, this.line.getY1() + ty,
                          this.line.getX2() + tx, this.line.getY2() + ty) ;
      }

      fireChartShapeChanged() ;
    }

  }

  /**
   * <zh>ͼϽǵ(ͼεת).</zh>
   * <en>Returns the top left corner coordinates of this shape (not consider rotation).</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getTopLeftPoint() {
    return new Point2D.Double(Math.min(this.line.getX1(), this.line.getX2()),
                              Math.min(this.line.getY1(), this.line.getY2())) ;
  }

  /**
   * <zh>ʱͼεĵ.</zh>
   * <en>Returns the lock coordinates of this shape.</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getLockPoint() {
    AffineTransform transform = AffineTransform.getTranslateInstance(this.line.getX1(),
                                                                     this.line.getY1()) ;
    transform.rotate(getRotate()) ;
    Line2D newLine = new Line2D.Double(0, 0, this.line.getX2() - this.line.getX1(),
                                       this.line.getY2() - this.line.getY1()) ;
    Shape rotateShape = transform.createTransformedShape(newLine) ;
    Rectangle2D rotateBound = rotateShape.getBounds2D() ;

    return new Point2D.Double(rotateBound.getCenterX(), rotateBound.getCenterY()) ;
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    AffineTransform transform = AffineTransform.getTranslateInstance(this.line.getX1(),
                                                                     this.line.getY1()) ;
    transform.rotate(getRotate()) ;

    GeneralPath path = new GeneralPath() ;
    path.moveTo( -1, 0) ;
    path.lineTo((float) (this.line.getX2() - this.line.getX1()) - 1,
                (float) (this.line.getY2() - this.line.getY1())) ;
    path.lineTo((float) (this.line.getX2() - this.line.getX1() + 1),
                (float) (this.line.getY2() - this.line.getY1())) ;
    path.lineTo(1, 0) ;
    path.closePath() ;

    return transform.createTransformedShape(path) ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   * @return ShapeType <zh>{@link com.ijchart.xychart.constant.ShapeType}ඨĳ
   * (Ϊnull).</zh>
   * <en>The constant value in class {@link com.ijchart.xychart.constant.ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_DBL_HEAD_MATCHSTICK ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument;
   * false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!(obj instanceof DblHeadMatchstickChartShape)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    DblHeadMatchstickChartShape that = (DblHeadMatchstickChartShape) obj ;
    if (!EqualsUtil.equals(this.shape, that.shape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.line, that.line)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double hashCode = super.hashCode() ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.shape) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.line) ;

    return (int) hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[shape=" + this.shape + ",line=" + this.line + "]" ;
  }

  /**
   * <zh>л</zh>
   * <en>Provides serialization support</en>
   * @param stream <zh></zh>
   * <en>the output stream</en>
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writeShape(stream, this.shape) ;
    SerialUtil.writeShape(stream, this.line) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.shape = SerialUtil.readShape(stream) ;
    this.line = (Line2D) SerialUtil.readShape(stream) ;
  }
}
