package com.ijchart.xychart.shape ;

import java.awt.AlphaComposite ;
import java.awt.Graphics2D ;
import java.awt.Shape ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.PathIterator ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.GraphicsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>һGeneralPathͼ.</zh>
 * <en>Draw a GeneralPath chart shape.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class GeneralPathChartShape extends AbstractRectangleTextChartShape implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -160035045166140879L ;

  /**
   * <zh>ҪƵͼ.</zh>
   * <en>The shape.</en>
   */
  private transient GeneralPath shape ;

  /**
   * <zh>ȵͼ.</zh>
   * <en>The hotspot shape.</en>
   */
  private transient GeneralPath hotspot ;

  /**
   * <zh>һGeneralPathͼι챾.</zh>
   * <en>Construct by a GeneralPath shape.</en>
   *
   * @param shape GeneralPath <zh>GeneralPathͼ (Ϊnull).</zh>
   * <en>A GeneralPath shape (null not permitted).</en>
   */
  public GeneralPathChartShape(GeneralPath shape) {
    this(shape, new Rectangle2D.Double(0, 0, 0, 0), null) ;
  }

  /**
   * <zh>һGeneralPathͼι챾.</zh>
   * <en>Construct by a GeneralPath shape.</en>
   *
   * @param shape GeneralPath <zh>GeneralPathͼ (Ϊnull).</zh>
   * <en>A GeneralPath shape (null not permitted).</en>
   * @param hotspot GeneralPath <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (null permitted).</en>
   */
  public GeneralPathChartShape(GeneralPath shape, GeneralPath hotspot) {
    this(shape, hotspot, new Rectangle2D.Double(0, 0, 0, 0), null) ;
  }

  /**
   * <zh>drawAreaʾtext.</zh>
   * <en>Draw text in area drawArea.</en>
   *
   * @param shape GeneralPath <zh>GeneralPathͼ (Ϊnull).</zh>
   * <en>A GeneralPath shape (null not permitted).</en>
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   */
  public GeneralPathChartShape(GeneralPath shape, Rectangle2D drawArea, String text) {
    super(drawArea, text, null) ;

    this.shape = shape ;
  }

  /**
   * <zh>drawAreaʾtext.</zh>
   * <en>Draw text in area drawArea.</en>
   *
   * @param shape GeneralPath <zh>GeneralPathͼ (Ϊnull).</zh>
   * <en>A GeneralPath shape (null not permitted).</en>
   * @param hotspot GeneralPath <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (null permitted).</en>
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   */
  public GeneralPathChartShape(GeneralPath shape, GeneralPath hotspot,
                               Rectangle2D drawArea, String text) {
    super(drawArea, text, null) ;

    this.shape = shape ;
    this.hotspot = hotspot ;
  }

  /**
   * <zh>ҪƵͼ.</zh>
   * <en>Returns the shape.</en>
   *
   * @return GeneralPath <zh>ҪƵͼ (Ϊnull).</zh>
   * <en>The shape (never null).</en>
   */
  public GeneralPath getShape() {
    return this.shape ;
  }

  /**
   * <zh>ҪƵͼ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the shape and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param shape GeneralPath <zh>ҪƵͼ.</zh>
   * <en>The shape.</en>
   */
  public void setShape(GeneralPath shape) {
    if (shape == null) {
      throw new IllegalArgumentException("Null 'shape' argument.") ;
    }

    this.shape = shape ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return GeneralPath <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  public GeneralPath getHotspot() {
    return this.hotspot ;
  }

  /**
   * <zh>ȵͼ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the hotspot shape and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param hotspot GeneralPath <zh>ȵͼ.</zh>
   * <en>The hotspot shape.</en>
   */
  public void setHotspot(GeneralPath hotspot) {
    if (!(this.hotspot == null && hotspot == null)) {
      this.hotspot = hotspot ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics device(such as the screen or the
   * printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    Point2D firstPoint = new Point2D.Double() ;
    Shape newPath = getNewPathShape(this.shape, firstPoint) ;
    g2.translate(firstPoint.getX(), firstPoint.getY()) ;
    g2.rotate(getRotate()) ;
    beforeDraw(g2) ;
//    g2.setClip(cloneArea) ;

    drawBackground(g2, newPath) ;
    drawBorder(g2, newPath) ;

    g2.translate(getDrawArea().getX(), getDrawArea().getY()) ;
    Rectangle2D cloneArea = new Rectangle2D.Double(0, 0, getDrawArea().getWidth(),
                                                   getDrawArea().getHeight()) ;
    drawTextWithTrimMarginRect(g2, cloneArea) ;

    afterDraw(g2) ;
    g2.rotate( -getRotate()) ;
    state.restoreOldState() ;
    drawSelect(g2) ;
  }

  /**
   * <zh>ͼεĵһƶ(0,0).</zh>
   * <en>The shape first point moved to the coordinates of the points (0,0).</en>
   *
   * @param pathShape Shape
   * @param firstPoint Point2D
   * @return Shape
   */
  private static Shape getNewPathShape(Shape pathShape, Point2D firstPoint) {
    double[] coords = new double[6] ; //
    PathIterator pathIterator = pathShape.getPathIterator(null, 1.0) ;
    double startX = 0, startY = 0 ; //ʼλ(startX,startY)
    while (!pathIterator.isDone()) {
      int type = pathIterator.currentSegment(coords) ;
      if (type == PathIterator.SEG_MOVETO) {
        startX = coords[0] ;
        startY = coords[1] ;
        firstPoint.setLocation(startX, startY) ;

        break ;
      }
    }

    return GraphicsUtil.translateShapeByPath(pathShape, (float) startX, (float) startY) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics
   * device(such as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  protected void drawSelect(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }

    if (isSelected()) {
      Graphics2DState state = new Graphics2DState(g2) ;
      state.saveOldState() ;
      g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getSelectRectAlpha())) ;
      g2.setPaint(getSelectAnchorPaint()) ;

      double rectWidth = getSelectRectWidth() ;
      double halfWidth = 0.5 * rectWidth ;

      boolean first = true ; //һ
      double[] coords = new double[6] ; //
      PathIterator pathIterator = this.shape.getPathIterator(null, 1.0) ;
      double startX = 0, startY = 0 ; //ʼλ(startX,startY)
      while (!pathIterator.isDone()) {
        if (first) { //һڵ
          first = false ;
          int type = pathIterator.currentSegment(coords) ;
          if (type == PathIterator.SEG_MOVETO) {
            startX = coords[0] ;
            startY = coords[1] ;
            g2.translate(startX, startY) ;
            g2.rotate(getRotate()) ;
            g2.fillRect((int) - halfWidth, (int) - halfWidth, (int) rectWidth, (int) rectWidth) ;
          }
        } else { //ڵ
          int type = pathIterator.currentSegment(coords) ;
          if (type == PathIterator.SEG_LINETO) {
            g2.fillRect((int) (coords[0] - startX - halfWidth),
                        (int) (coords[1] - startY - halfWidth), (int) rectWidth, (int) rectWidth) ;
          }
        }

        pathIterator.next() ;
      }

      if (!first) {
        g2.rotate( -getRotate()) ;
      }
      state.restoreOldState() ;
    }
  }

  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  protected void calculateTransformedShape() {
    if (transformedShape == null) {
      AffineTransform transform = new AffineTransform() ;
      Point2D firstPoint = new Point2D.Double() ;
      Shape newPath = getNewPathShape(this.shape, firstPoint) ;

      transform.translate(firstPoint.getX(), firstPoint.getY()) ;
      transform.rotate(getRotate()) ;
      transformedShape = transform.createTransformedShape(newPath) ;
    }
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    Point2D firstPoint = new Point2D.Double() ;
    Shape newPath = getNewPathShape((this.hotspot == null) ? this.shape : this.hotspot, firstPoint) ;
    AffineTransform transform = AffineTransform.getTranslateInstance(firstPoint.getX(),
                                                                     firstPoint.getY()) ;
    transform.rotate(getRotate()) ;

    return transform.createTransformedShape(newPath) ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   * @return ShapeType <zh>{@link com.ijchart.xychart.constant.ShapeType}ඨĳ
   * (Ϊnull).</zh>
   * <en>The constant value in class {@link com.ijchart.xychart.constant.ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_GENERALPATH ;
  }

  /**
   * <zh>ͼϽǵ(ͼεת).</zh>
   * <en>Returns the top left corner coordinates of this shape (not consider rotation).</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getTopLeftPoint() {
    Point2D result = new Point2D.Double() ;

    boolean first = true ; //һ
    double[] coords = new double[6] ; //
    PathIterator pathIterator = this.shape.getPathIterator(null, 1.0) ;
    while (!pathIterator.isDone()) {
      if (first) { //һڵ
        first = false ;
        int type = pathIterator.currentSegment(coords) ;
        if (type == PathIterator.SEG_MOVETO) {
          result.setLocation(coords[0], coords[1]) ;
          break ;
        }
      }

      pathIterator.next() ;
    }

    return result ;
  }

  /**
   * <zh>ʱͼεĵ.</zh>
   * <en>Returns the lock coordinates of this shape.</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getLockPoint() {
    Point2D firstPoint = new Point2D.Double() ;
    Shape newPath = getNewPathShape(this.shape, firstPoint) ;

    AffineTransform transform = AffineTransform.getTranslateInstance(firstPoint.getX(),
                                                                     firstPoint.getY()) ;
    transform.rotate(getRotate()) ;
    Shape rotateShape = transform.createTransformedShape(newPath) ;
    Rectangle2D rotateBound = rotateShape.getBounds2D() ;

    return new Point2D.Double(rotateBound.getCenterX(), rotateBound.getCenterY()) ;
  }

  /**
   * <zh>ƶͼ,X᷽ƫtx,Y᷽ƫty,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Concatenates the current Graphics2D Transform with a translation transform.and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.
   * Subsequent rendering is translated by the specified
   * distance relative to the previous position.
   * This is equivalent to calling transform(T), where T is an
   * AffineTransform represented by the following matrix:
   * <pre>
   *		[   1    0    tx  ]
   *		[   0    1    ty  ]
   *		[   0    0    1   ]
   * </pre>.</en>
   *
   * @param tx double <zh>X᷽ľ.</zh>
   * <en>The distance to translate along the x-axis.</en>
   * @param ty double <zh>Y᷽ľ.</zh>
   * <en>The distance to translate along the y-axis.</en>
   */
  public void translate(double tx, double ty) {
    if (!(tx == 0 && ty == 0)) {
      boolean first = true ; //һ
      double[] coords = new double[6] ; //
      PathIterator pathIterator = this.shape.getPathIterator(null, 1.0) ;
      GeneralPath newPath = new GeneralPath() ;
      while (!pathIterator.isDone()) {
        if (first) { //һڵ
          first = false ;
          int type = pathIterator.currentSegment(coords) ;
          if (type == PathIterator.SEG_MOVETO) {
            newPath.moveTo((float) (coords[0] + tx), (float) (coords[1] + ty)) ;
          }
        } else { //ڵ
          int type = pathIterator.currentSegment(coords) ;
          if (type == PathIterator.SEG_LINETO) {
            newPath.lineTo((float) (coords[0] + tx), (float) (coords[1] + ty)) ;
          } else if (type == PathIterator.SEG_CLOSE) {
            newPath.closePath() ;
          }
        }

        pathIterator.next() ;
      }
      this.shape = newPath ;

      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument;
   * false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!(obj instanceof GeneralPathChartShape)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    GeneralPathChartShape that = (GeneralPathChartShape) obj ;
    if (!EqualsUtil.equals(this.shape, that.shape)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int hashCode = super.hashCode() ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.shape) ;

    return hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[shape=" + this.shape + "]" ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the output stream.
   *
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;
    SerialUtil.writeShape(stream, this.shape) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;
    this.shape = (GeneralPath) SerialUtil.readShape(stream) ;
  }

}
