package com.ijchart.xychart.shape ;

import java.awt.AlphaComposite ;
import java.awt.Graphics2D ;
import java.awt.Shape ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.Line2D ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.StrokeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ͼ.</zh>
 * <en>Draw a line {@link com.ijchart.xychart.shape.IChartShape}.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class LineChartShape extends AbstractChartShape implements Serializable, Cloneable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -6721432195276103174L ;

  /**
   * <zh>һx.</zh>
   * <en>The first point's x coordinate.</en>
   */
  private double x1 ;

  /**
   * <zh>һy.</zh>
   * <en>The first point's y coordinate.</en>
   */
  private double y1 ;

  /**
   * <zh>ڶx.</zh>
   * <en>The second point's x coordinate.</en>
   */
  private double x2 ;

  /**
   * <zh>ڶy.</zh>
   * <en>The second point's y coordinate.</en>
   */
  private double y2 ;

  /**
   * <zh>һӵ(x1,y1)(x2,y2)ֱ.</zh>
   * <en>Draw a line from point (x1,y1) to point (x2,y2).</en>
   *
   * @param x1 double <zh>һx.</zh>
   * <en>The first point's x coordinate.</en>
   * @param y1 double <zh>һy.</zh>
   * <en>The first point's y coordinate.</en>
   * @param x2 double <zh>ڶx.</zh>
   * <en>The second point's x coordinate.</en>
   * @param y2 double <zh>ڶy.</zh>
   * <en>The second point's y coordinate.</en>
   */
  public LineChartShape(double x1, double y1, double x2, double y2) {
    this.x1 = x1 ;
    this.y1 = y1 ;
    this.x2 = x2 ;
    this.y2 = y2 ;
  }

  /**
   * <zh>һӵp1p2ֱ.</zh>
   * <en>Draw a line from point (p1) to point (p2).</en>
   *
   * @param p1 Point2D <zh>һ (Ϊnull).</zh>
   * <en>The first point (null not permitted).</en>
   * @param p2 Point2D <zh>ڶ (Ϊnull).</zh>
   * <en>The second point (null not permitted).</en>
   */
  public LineChartShape(Point2D p1, Point2D p2) {
    if (p1 == null) {
      throw new IllegalArgumentException("Null 'p1' point.") ;
    }
    if (p2 == null) {
      throw new IllegalArgumentException("Null 'p2' point.") ;
    }

    this.x1 = p1.getX() ;
    this.y1 = p1.getY() ;
    this.x2 = p2.getX() ;
    this.y2 = p2.getY() ;
  }

  /**
   * <zh>һֱ.</zh>
   * <en>Draw a line.</en>
   *
   * @param line Line2D <zh>ֱ (Ϊnull).</zh>
   * <en>The line (null not permitted).</en>
   */
  public LineChartShape(Line2D line) {
    if (line == null) {
      throw new IllegalArgumentException("Null 'line' point.") ;
    }

    this.x1 = line.getX1() ;
    this.y1 = line.getY1() ;
    this.x2 = line.getX2() ;
    this.y2 = line.getY2() ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ</zh>
   * <en>Draw the ChartShape on a Java 2d graphics device(such as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull).</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    g2.translate(this.x1, this.y1) ;
    g2.rotate(getRotate()) ;
    beforeDraw(g2) ;

    g2.setStroke(getBorderStroke()) ;
    g2.setPaint(getBorderPaint()) ;
    g2.draw(new Line2D.Double(0, 0, this.x2 - this.x1, this.y2 - this.y1)) ;

    afterDraw(g2) ;
    g2.rotate( -getRotate()) ;
    state.restoreOldState() ;

    drawSelect(g2) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics device(such
   * as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  protected void drawSelect(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }

    if (isSelected()) {
      Graphics2DState state = new Graphics2DState(g2) ;
      state.saveOldState() ;

      g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getSelectRectAlpha())) ;

      double rectWidth = getSelectRectWidth() ;
      final double halfRect = rectWidth / 2 ;

      g2.setPaint(getSelectAnchorPaint()) ;
      g2.translate(this.x1, this.y1) ;
      g2.rotate(getRotate()) ;

      //ê
      if (y1 == y2) { //ˮƽ
        g2.fill(new Rectangle2D.Double(0, -halfRect, rectWidth, rectWidth)) ;
        g2.fill(new Rectangle2D.Double(x2 - x1 - rectWidth, -halfRect,
                                       rectWidth, rectWidth)) ;
      } else if (x1 == x2) { //ֱ
        g2.fill(new Rectangle2D.Double( -halfRect, 0, rectWidth, rectWidth)) ;
        g2.fill(new Rectangle2D.Double( -halfRect, y2 - y1 - rectWidth,
                                       rectWidth, rectWidth)) ;
      } else {
        //ˮƽֱê㾭ƫתбê
        double theta = Math.atan((y2 - y1) / (x2 - x1)) ; //бߵƫת
        if (x1 > x2 && y1 > y2) {
          theta -= Math.PI ;
        }

        double lineLen = Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2)) ; //֮ľ
        g2.rotate(theta) ;

        g2.fill(new Rectangle2D.Double(0, -halfRect, rectWidth, rectWidth)) ;
        g2.fill(new Rectangle2D.Double(lineLen - rectWidth, -halfRect, rectWidth, rectWidth)) ;

        g2.rotate( -theta) ;
      }

      g2.rotate( -getRotate()) ;
      state.restoreOldState() ;
    }

  }

  /**
   * <zh>ƶͼ,X᷽ƫtx,Y᷽ƫty,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Concatenates the current Graphics2D Transform with a translation transform.and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.
   * Subsequent rendering is translated by the specified
   * distance relative to the previous position.
   * This is equivalent to calling transform(T), where T is an
   * AffineTransform represented by the following matrix:
   * <pre>
   *		[   1    0    tx  ]
   *		[   0    1    ty  ]
   *		[   0    0    1   ]
   * </pre>.</en>
   *
   * @param tx double <zh>X᷽ľ.</zh>
   * <en>The distance to translate along the x-axis.</en>
   * @param ty double <zh>Y᷽ľ.</zh>
   * <en>The distance to translate along the y-axis.</en>
   */
  public void translate(double tx, double ty) {
    if (!(tx == 0 && ty == 0)) {
      this.x1 += tx ;
      this.y1 += ty ;
      this.x2 += tx ;
      this.y2 += ty ;

      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼϽǵ(ͼεת).</zh>
   * <en>Returns the top left corner coordinates of this shape (not consider rotation).</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getTopLeftPoint() {
    return new Point2D.Double(this.x1, this.y1) ;
  }

  /**
   * <zh>ʱͼεĵ.</zh>
   * <en>Returns the lock coordinates of this shape.</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getLockPoint() {
    Line2D newLine = new Line2D.Double(0, 0, this.x2 - this.x1, this.y2 - this.y1) ;
    AffineTransform transform = AffineTransform.getTranslateInstance(this.x1, this.y1) ;
    transform.rotate(getRotate()) ;
    Shape rotateShape = transform.createTransformedShape(newLine) ;
    Rectangle2D rotateBound = rotateShape.getBounds2D() ;

    return new Point2D.Double(rotateBound.getCenterX(), rotateBound.getCenterY()) ;
  }

  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  protected void calculateTransformedShape() {
    if (this.transformedShape == null) {
      Line2D line = new Line2D.Double(this.x1, this.y1, this.x2, this.y2) ;
      this.transformedShape = StrokeUtil.lineToRectangle(line, getBorderStroke(), getRotate()) ;
    }
  }

  /**
   * <zh>tooltipͼӦƫƵλ.</zh>
   * <en>Returns the translate point of tooltip shape.</en>
   *
   * @return Point2D <zh> (Ϊnull).</zh>
   * <en>The translate point (null permitted).</en>
   */
  protected Point2D getTooltipShapeTranslatePoint() {
    return new Point2D.Double(this.x1, this.y1) ;
  }

  /**
   * <zh>tooltipͼӦתĽǶ.</zh>
   * <en>Returns the angle of rotation of the tooltip shape.</en>
   *
   * @return double <zh>Ƕ(λǻ).</zh>
   * <en>The angle of rotation (in radians).</en>
   */
  protected double getTooltipShapeRotate() {
    double theta = 0 ; //бߵƫת
    if (x1 == x2) {
      theta = IAngle.ANGLE_DOWN_90 ;
    } else {
      theta = Math.atan((y2 - y1) / (x2 - x1)) ;
    }

    if (y1 > y2 && x1 > x2) {
      theta -= Math.PI ;
    }

    return theta ;
  }

  /**
   * <zh>õһx,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the first point's x coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param x1 double
   */
  public void setX1(double x1) {
    if (this.x1 != x1) {
      this.x1 = x1 ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>صһx.</zh>
   * <en>Returns the first point's x coordinate.</en>
   *
   * @return double
   */
  public double getX1() {
    return x1 ;
  }

  /**
   * <zh>õһy,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the first point's y coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param y1 double
   */
  public void setY1(double y1) {
    if (this.y1 != y1) {
      this.y1 = y1 ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>صһy.</zh>
   * <en>Returns the first point's y coordinate.</en>
   *
   * @return double
   */
  public double getY1() {
    return y1 ;
  }

  /**
   * <zh>õڶx,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the second point's x coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param x2 double
   */
  public void setX2(double x2) {
    if (this.x2 != x2) {
      this.x2 = x2 ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>صڶx.</zh>
   * <en>Returns the second point's x coordinate.</en>
   *
   * @return double
   */
  public double getX2() {
    return x2 ;
  }

  /**
   * <zh>õڶy,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the second point's y coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param y2 double
   */
  public void setY2(double y2) {
    if (this.y2 != y2) {
      this.y2 = y2 ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>صڶy.</zh>
   * <en>Returns the second point's y coordinate.</en>
   * @return double
   */
  public double getY2() {
    return y2 ;
  }

  /**
   * <zh>õһ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the first point coordinate and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param x double <zh>һx.</zh>
   * <en>The first point's x coordinate.</en>
   * @param y double <zh>һy.</zh>
   * <en>The first point's y coordinate.</en>
   */
  public void setPoint1(double x, double y) {
    this.x1 = x ;
    this.y1 = y ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>õڶ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the second point coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param x double <zh>ڶx.</zh>
   * <en>The second point's x coordinate.</en>
   * @param y double <zh>ڶy.</zh>
   * <en>The second point's y coordinate.</en>
   */
  public void setPoint2(double x, double y) {
    this.x2 = x ;
    this.y2 = y ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>õһ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the first point coordinate,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param p Point2D <zh>һ (Ϊnull).</zh>
   * <en>The first point coordinate (null not permitted).</en>
   */
  public void setPoint1(Point2D p) {
    if (p == null) {
      throw new IllegalArgumentException("Null 'p' argument.") ;
    }

    this.x1 = p.getX() ;
    this.y1 = p.getY() ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>صһ.</zh>
   * <en>Returns the first point coordinate.</en>
   *
   * @return Point2D <zh>һ (Ϊnull).</zh>
   * <en>Returns the first point coordinate (never null).</en>
   */
  public Point2D getPoint1() {
    return new Point2D.Double(this.x1, this.y1) ;
  }

  /**
   * <zh>õڶ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the second point coordinate and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param p Point2D <zh>ڶ (Ϊnull).</zh>
   * <en>The second point coordinate (null not permitted).</en>
   */
  public void setPoint2(Point2D p) {
    if (p == null) {
      throw new IllegalArgumentException("Null 'p' argument.") ;
    }

    this.x2 = p.getX() ;
    this.y2 = p.getY() ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>صڶ.</zh>
   * <en>Returns the second point coordinate.</en>
   *
   * @return Point2D <zh>ڶ (Ϊnull).</zh>
   * <en>Returns the second point coordinate (never null).</en>
   */
  public Point2D getPoint2() {
    return new Point2D.Double(this.x2, this.y2) ;
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    calculateTransformedShape() ;
    return transformedShape ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   *
   * @return ShapeType <zh>{@link ShapeType}ඨĳ (Ϊnull).</zh>
   * <en>The constant value in class {@link ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_LINE ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false
   *  otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    if (!(obj instanceof LineChartShape)) {
      return false ;
    }

    final double theta = 0.00001 ;
    LineChartShape that = (LineChartShape) obj ;
    if (!EqualsUtil.equals(this.x1, that.x1, theta)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.y1, that.y1, theta)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.x2, that.x2, theta)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.y2, that.y2, theta)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double hashCode = super.hashCode() ;
    hashCode = 37 * hashCode + x1 ;
    hashCode = 37 * hashCode + y1 ;
    hashCode = 37 * hashCode + x2 ;
    hashCode = 37 * hashCode + y2 ;

    return (int) hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[x1=" + x1 + ",y1=" + y1 + ",x2=" + x2 + ",y2=" + y2 + "]" ;
  }
}
