package com.ijchart.xychart.shape ;

import java.awt.AlphaComposite ;
import java.awt.Font ;
import java.awt.FontMetrics ;
import java.awt.Graphics2D ;
import java.awt.Shape ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.Line2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.ui.Rectangle ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.RectangleInsets ;
import com.ijchart.xychart.util.AnchorUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.StrokeUtil ;
import com.ijchart.xychart.util.TextUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>һڲⲿʾ.</zh>
 * <en>Draw a text inside or outside a Rectangle,with or not have a border.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class RectangleTextChartShape extends AbstractRectangleTextChartShape implements Cloneable,
  Serializable {

  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 5332038187514490656L ;

  /**
   * <zh>߿Ƿɼ.</zh>
   * <en>A boolean flag controls whether or not the left border is visible.</en>
   */
  private boolean leftBorderVisible ;

  /**
   * <zh>ұ߿Ƿɼ.</zh>
   * <en>A boolean flag controls whether or not the right border is visible.</en>
   */
  private boolean rightBorderVisible ;

  /**
   * <zh>ϱ߿Ƿɼ.</zh>
   * <en>A boolean flag controls whether or not the top border is visible.</en>
   */
  private boolean topBorderVisible ;

  /**
   * <zh>±߿Ƿɼ.</zh>
   * <en>A boolean flag controls whether or not the bottom border is visible.</en>
   */
  private boolean bottomBorderVisible ;

  /**
   * <zh>drawAreaʾtext.</zh>
   * <en>Draw text in area drawArea.</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   */
  public RectangleTextChartShape(Rectangle2D drawArea, String text) {
    this(drawArea, text, null) ;
  }

  /**
   * <zh>drawAreaʾtext.</zh>
   * <en>Draw text in area drawArea.</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param font <zh> (Ϊnull),fontΪnull,
   * Ϊ{@link com.ijchart.xychart.shape.IChartShape#DEFAULT_TEXT_FONT}.</zh>
   * <en>The specified font (null permitted).If font is null,
   * {@link com.ijchart.xychart.shape.IChartShape#DEFAULT_TEXT_FONT} is used.</en>
   */
  public RectangleTextChartShape(Rectangle2D drawArea, String text, Font font) {
    super(drawArea, text, font) ;

    this.leftBorderVisible = true ;
    this.rightBorderVisible = true ;
    this.topBorderVisible = true ;
    this.bottomBorderVisible = true ;
  }

  /**
   * <zh>,ַҪС,ȲdrawArea.getWidth().</zh>
   * <en>Returns the minimum bounds of the specified string in the specified Graphics context,
   * the width is less or equal than drawArea.getWidth() (width<=drawArea.getWidth()).</en>
   *
   * @param g2 Graphics2D <zh>ƻ (Ϊnull).</zh>
   * <en>The specified Graphics context (null not permitted).</en>
   * @return Rectangle
   */
  public Rectangle getMinimumBound(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }

    Rectangle2D cloneArea = new Rectangle2D.Double(0, 0, getDrawArea().getWidth(),
                                                   getDrawArea().getHeight()) ;
    Rectangle2D trimArea = getMargin().getTrimArea(cloneArea) ;
    trimArea.setRect(0, 0, trimArea.getWidth(), trimArea.getHeight()) ;
    if (isDrawBorder()) { //߽
      float strokeWidth = StrokeUtil.getStrokeWidth(getBorderStroke()) ;
      trimArea = new Rectangle2D.Double(0, 0, trimArea.getWidth() - 2 * strokeWidth,
                                        trimArea.getHeight() - 2 * strokeWidth) ;
    }

    String[] splitArr = splitString(g2, trimArea) ;

    RectangleInsets margin = getMargin() ;
    double width = 0, height = 0 ;
    if (margin != null) {
      width = margin.getLeft() + margin.getRight() ;
      height = margin.getTop() + margin.getBottom() ;
    }

    if (splitArr != null) {
      double maxWidth = 0 ; //
      double totalHeight = 0 ; //߶֮

      Graphics2DState state = new Graphics2DState(g2) ;
      state.saveOldState() ;
      g2.setFont(getTextFont()) ;
      FontMetrics fm = g2.getFontMetrics() ;
      int descent = fm.getDescent() ;

      for (int i = 0 ; i < splitArr.length ; i++) {
        Rectangle2D bound = TextUtil.getStringBoundWithFm(g2, splitArr[i]) ;
        maxWidth = Math.max(maxWidth, bound.getWidth()) ;
        totalHeight += bound.getHeight() ;
      }
      totalHeight += (splitArr.length - 1) * descent ;
      state.restoreOldState() ;

      width += Math.min(maxWidth, getDrawArea().getWidth()) ;
      height += totalHeight ;
    }

    if (isDrawBorder()) { //߽
      float strokeWidth = StrokeUtil.getStrokeWidth(getBorderStroke()) ;
      width += 2 * strokeWidth ;
      height += 2 * strokeWidth ;
    }

    return new Rectangle(width + 0.5, height) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ</zh>
   * <en>Draw the ChartShape on a Java 2d graphics device(such as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    Rectangle2D cloneArea = new Rectangle2D.Double(0, 0, getDrawArea().getWidth(),
                                                   getDrawArea().getHeight()) ;
    g2.translate(getDrawArea().getX(), getDrawArea().getY()) ;
    g2.rotate(getRotate()) ;
    beforeDraw(g2) ;
//    g2.setClip(cloneArea) ;

    float strokeWidth = 0 ;
    if (isDrawBorder()) {
      strokeWidth = StrokeUtil.getStrokeWidth(getBorderStroke()) ;
    }

    drawBackground(g2, cloneArea) ;
    drawBorder(g2, new Rectangle2D.Double(strokeWidth / 2, strokeWidth / 2,
                                          getDrawArea().getWidth() - strokeWidth,
                                          getDrawArea().getHeight() - strokeWidth)) ;

    drawTextWithTrimMarginRect(g2, cloneArea) ;

    afterDraw(g2) ;
    g2.rotate( -getRotate()) ;
    state.restoreOldState() ;
    drawSelect(g2) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics device(such as the screen or the
   * printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  protected void drawSelect(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }

    if (isSelected()) {
      Graphics2DState state = new Graphics2DState(g2) ;
      state.saveOldState() ;
      g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getSelectRectAlpha())) ;

      Rectangle2D drawArea = getDrawArea() ;
      double rectWidth = getSelectRectWidth() ;
      g2.translate(drawArea.getX(), drawArea.getY()) ;
      g2.rotate(getRotate()) ;

      g2.setPaint(getSelectAnchorPaint()) ;
      //ê
      Rectangle2D newArea = new Rectangle2D.Double(0, 0, drawArea.getWidth(),
                                                   drawArea.getHeight()) ;
      AnchorUtil.drawAnchor(g2, newArea, rectWidth) ;

      g2.rotate( -getRotate()) ;
      state.restoreOldState() ;
    }
  }

  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  protected void calculateTransformedShape() {
    if (transformedShape == null) {
      AffineTransform transform = new AffineTransform() ;
      transform.translate(getDrawArea().getX(), getDrawArea().getY()) ;
      transform.rotate(getRotate()) ;
      Rectangle2D newArea = new Rectangle2D.Double(0, 0, getDrawArea().getWidth(),
                                                   getDrawArea().getHeight()) ;

      transformedShape = transform.createTransformedShape(newArea) ;
    }
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    calculateTransformedShape() ;
    return transformedShape ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   * @return ShapeType <zh>{@link com.ijchart.xychart.constant.ShapeType}ඨĳ
   * (Ϊnull).</zh>
   * <en>The constant value in class {@link com.ijchart.xychart.constant.ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_TEXT_IN_RECTANGLE ;
  }

  /**
   * <zh>ĳ߿Ƿɼ,֪ͨ仯.</zh>
   * <en>Sets whether or not a border is visible.</en>
   *
   * @param edge RectangleEdge <zh>߽ (Ϊnull).</zh>
   * <en>The rectangle edge (null not permitted).</en>
   * @param visible boolean <zh>ֵ,trueɼ,falseɼ.</zh>
   * <en>A boolean.</en>
   */
  public void setBorderVisible(RectangleEdge edge, boolean visible) {
    if (edge == null) {
      throw new IllegalArgumentException("Null 'edge' argument.") ;
    }

    if (edge.equals(RectangleEdge.LEFT) && this.leftBorderVisible != visible) {
      this.leftBorderVisible = visible ;
      fireChartShapeChanged() ;
    } else if (edge.equals(RectangleEdge.RIGHT) && this.rightBorderVisible != visible) {
      this.rightBorderVisible = visible ;
      fireChartShapeChanged() ;
    } else if (edge.equals(RectangleEdge.TOP) && this.topBorderVisible != visible) {
      this.topBorderVisible = visible ;
      fireChartShapeChanged() ;
    } else if (edge.equals(RectangleEdge.BOTTOM) && this.bottomBorderVisible != visible) {
      this.bottomBorderVisible = visible ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ĳ߽Ƿɼ.</zh>
   * <en>Returns whether or not the specified border is visible.</en>
   *
   * @param edge RectangleEdge <zh>߽ (Ϊnull).</zh>
   * <en>The rectangle edge (null not permitted).</en>
   * @return boolean <zh>ɼtrue,򷵻false.</zh>
   * <en>Returns true if the border visible,otherwise return false.</en>
   */
  public boolean isBorderVisible(RectangleEdge edge) {
    if (edge == null) {
      throw new IllegalArgumentException("Null 'edge' argument.") ;
    }

    if (edge.equals(RectangleEdge.LEFT)) {
      return this.leftBorderVisible ;
    } else if (edge.equals(RectangleEdge.RIGHT)) {
      return this.rightBorderVisible ;
    } else if (edge.equals(RectangleEdge.TOP)) {
      return this.topBorderVisible ;
    } else if (edge.equals(RectangleEdge.BOTTOM)) {
      return this.bottomBorderVisible ;
    }

    return true ;
  }

  /**
   * <zh>߿(Ǹķ).</zh>
   * <en>Draw border(override parent class method).</en>
   *
   * @param g2 Graphics2D <zh>ͼλ.</zh>
   * <en>The specified Graphics context.</en>
   * @param drawArea Shape <zh>ƷΧ.</zh>
   * <en>The draw area.</en>
   */
  protected void drawBorder(Graphics2D g2, Shape drawArea) {
    if (!isDrawBorder()) {
      return ;
    }

    if (this.leftBorderVisible && this.rightBorderVisible
        && this.topBorderVisible && this.bottomBorderVisible) { //б߿򶼿ɼ
      super.drawBorder(g2, drawArea) ;
    } else {
      if (drawArea instanceof Rectangle2D) {
        Rectangle2D rect = (Rectangle2D) drawArea ;

        Graphics2DState state = new Graphics2DState(g2) ;
        state.saveOldState() ;

        if (getBorderStroke() != null) {
          g2.setStroke(getBorderStroke()) ;
        }
        g2.setPaint(getBorderPaint()) ;

        if (this.leftBorderVisible) { //߽
          g2.draw(new Line2D.Double(rect.getX(), rect.getY(), rect.getX(), rect.getMaxY())) ;
        }
        if (this.rightBorderVisible) { //ұ߽
          g2.draw(new Line2D.Double(rect.getMaxX(), rect.getY(), rect.getMaxX(), rect.getMaxY())) ;
        }
        if (this.topBorderVisible) { //ϱ߽
          g2.draw(new Line2D.Double(rect.getX(), rect.getY(), rect.getMaxX(), rect.getY())) ;
        }
        if (this.bottomBorderVisible) { //±߽
          g2.draw(new Line2D.Double(rect.getX(), rect.getMaxY(), rect.getMaxX(), rect.getMaxY())) ;
        }

        state.restoreOldState() ;
      }
    }

  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument;
   * false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!(obj instanceof RectangleTextChartShape)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    RectangleTextChartShape that = (RectangleTextChartShape) obj ;
    if (this.leftBorderVisible != that.leftBorderVisible) {
      return false ;
    }
    if (this.rightBorderVisible != that.rightBorderVisible) {
      return false ;
    }
    if (this.topBorderVisible != that.topBorderVisible) {
      return false ;
    }
    if (this.bottomBorderVisible != that.bottomBorderVisible) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int hashCode = super.hashCode() ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.leftBorderVisible) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.rightBorderVisible) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.topBorderVisible) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.bottomBorderVisible) ;

    return hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[text=" + getText() + "]" ;
  }

}
