package com.ijchart.xychart.shape ;

import java.awt.Graphics2D ;
import java.awt.Shape ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.util.EqualsUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Statisticalͼ.</zh>
 * <en>Draw a statistical bar {@link com.ijchart.xychart.shape.IChartShape}.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class StatisticalBarChartShape extends AbstractRectangleTextChartShape implements
  Serializable, Cloneable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 3402055772412767823L ;

  /**
   * <zh>½ǵĵX.</zh>
   * <en>The x coordinate of left-hand corner.</en>
   */
  private double iShapedLeftX ;

  /**
   * <zh>½ǵĵY.</zh>
   * <en>The y coordinate of left-hand corner.</en>
   */
  private double iShapedBottomY ;

  /**
   * <zh>εĿ.</zh>
   * <en>The width of I-shaped shape.</en>
   */
  private double iShapedWidth ;

  /**
   * <zh>εĸ߶.</zh>
   * <en>The height of I-shaped shape.</en>
   */
  private double iShapedHeight ;

  /**
   * <zh>ƹεͼ.</zh>
   * <en>Draw a I-shaped chart shape.</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param iShapedLeftX double <zh>½ǵĵX.</zh>
   * <en>The x coordinate of left-hand corner.</en>
   * @param iShapedBottomY double <zh>½ǵĵY.</zh>
   * <en>The y coordinate of left-hand corner.</en>
   * @param iShapedWidth double <zh>εĿ.</zh>
   * <en>The width of I-shaped shape.</en>
   * @param iShapedHeight double <zh>εĸ߶.</zh>
   * <en>The height of I-shaped shape.</en>
   */
  public StatisticalBarChartShape(Rectangle2D drawArea, String text, double iShapedLeftX,
                                  double iShapedBottomY, double iShapedWidth, double iShapedHeight) {
    super(drawArea, text) ;

    this.iShapedLeftX = iShapedLeftX ;
    this.iShapedBottomY = iShapedBottomY ;
    this.iShapedWidth = iShapedWidth ;
    this.iShapedHeight = iShapedHeight ;
  }

  /**
   * <zh>½ǵĵX.</zh>
   * <en>Returns the x coordinate of left-hand corner.</en>
   *
   * @return double
   */
  public double getIShapedLeftX() {
    return this.iShapedLeftX ;
  }

  /**
   * <zh>½ǵĵX,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the x coordinate of left-hand corner,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param iShapedLeftX double
   */
  public void setIShapedLeftX(double iShapedLeftX) {
    if (this.iShapedLeftX != iShapedLeftX) {
      this.iShapedLeftX = iShapedLeftX ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>½ǵĵY.</zh>
   * <en>Returns the y coordinate of left-hand corner,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @return double
   */
  public double getIShapedBottomY() {
    return this.iShapedBottomY ;
  }

  /**
   * <zh>½ǵĵY,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the y coordinate of left-hand corner,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param iShapedBottomY double
   */
  public void setIShapedBottomY(double iShapedBottomY) {
    if (this.iShapedBottomY != iShapedBottomY) {
      this.iShapedBottomY = iShapedBottomY ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ùεĿ.</zh>
   * <en>Sets the width of I-shaped shape.</en>
   *
   * @return double
   */
  public double getIShapedWidth() {
    return this.iShapedWidth ;
  }

  /**
   * <zh>ùεĿ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the width of I-shaped shape,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param iShapedWidth double
   */
  public void setIShapedWidth(double iShapedWidth) {
    if (this.iShapedWidth != iShapedWidth) {
      this.iShapedWidth = iShapedWidth ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>عεĸ߶.</zh>
   * <en>Returns the height of I-shaped shape.</en>
   *
   * @return double
   */
  public double getIShapedHeight() {
    return this.iShapedHeight ;
  }

  /**
   * <zh>ùεĸ߶,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the height of I-shaped shape,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param iShapedHeight double
   */
  public void setIShapedHeight(double iShapedHeight) {
    if (this.iShapedHeight != iShapedHeight) {
      this.iShapedHeight = iShapedHeight ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ</zh>
   * <en>Draw the ChartShape on a Java 2d graphics
   * device(such as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    Rectangle2D drawArea = getDrawArea() ;
    Rectangle2D cloneArea = new Rectangle2D.Double(drawArea.getX() - this.iShapedLeftX,
                                                   drawArea.getY() - this.iShapedBottomY,
                                                   drawArea.getWidth(), drawArea.getHeight()) ;

    //ƹͼ--ʼ
    GeneralPath iShapedPath = new GeneralPath() ;
    iShapedPath.moveTo(0, 0) ;
    iShapedPath.lineTo((float)this.iShapedWidth, 0) ;
    iShapedPath.lineTo((float) (0.5 * this.iShapedWidth), 0) ;
    iShapedPath.lineTo((float) (0.5 * this.iShapedWidth), (float) - this.iShapedHeight) ;
    iShapedPath.lineTo(0, (float) - this.iShapedHeight) ;
    iShapedPath.lineTo((float)this.iShapedWidth, (float) - this.iShapedHeight) ;
    iShapedPath.lineTo((float) (0.5 * this.iShapedWidth), (float) - this.iShapedHeight) ;
    iShapedPath.lineTo((float) (0.5 * this.iShapedWidth), 0) ;
    iShapedPath.closePath() ;

    g2.translate(this.iShapedLeftX, this.iShapedBottomY) ;
    g2.rotate(getRotate()) ;
    beforeDraw(g2) ;

    drawBackground(g2, cloneArea) ;
    drawBorder(g2, cloneArea) ;

    g2.setPaint(getBorderPaint()) ;
    g2.draw(iShapedPath) ;

    g2.translate(drawArea.getX() - this.iShapedLeftX, drawArea.getY() - this.iShapedBottomY) ;
    cloneArea.setRect(0, 0, cloneArea.getWidth(), cloneArea.getHeight()) ;
    drawTextWithTrimMarginRect(g2, cloneArea) ;

    afterDraw(g2) ;
    g2.rotate( -getRotate()) ;
    state.restoreOldState() ;
    drawSelect(g2) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics device(such
   * as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  protected void drawSelect(Graphics2D g2) {
    if (isSelected()) {
      double rectWidth = getSelectRectWidth() ;
      Rectangle2D drawArea = getDrawArea() ;
      double drawAreaX = drawArea.getX() - this.iShapedLeftX ;
      double drawAreaY = drawArea.getY() - this.iShapedBottomY ;
      double drawAreaWidth = drawArea.getWidth() ;
      double drawAreaHeight = drawArea.getHeight() ;
      boolean drawRectWidthCenter = drawAreaWidth > 3 * rectWidth ;
      boolean drawRectHeightCenter = drawAreaHeight > 3 * rectWidth ;

      double halfWidth = 0.5 * this.iShapedWidth ;
      boolean drawCenter = this.iShapedWidth > 2 * rectWidth ;

      Point2D.Double[] points = new Point2D.Double[] {
                                //β
                                new Point2D.Double(drawAreaX, drawAreaY), //top left
                                new Point2D.Double(drawAreaX + drawAreaWidth, drawAreaY), //top right
                                new Point2D.Double(drawAreaX, drawAreaY + drawAreaHeight), //bottom left
                                new Point2D.Double(drawAreaX + drawAreaWidth,
                                                   drawAreaY + drawAreaHeight), //bottom right
                                drawRectWidthCenter
                                ? (new Point2D.Double(drawAreaX + 0.5 * drawAreaWidth, drawAreaY)) : null, //top center
                                drawRectWidthCenter
                                ? (new Point2D.Double(drawAreaX + 0.5 * drawAreaWidth,
                                                      drawAreaY + drawAreaHeight)) : null, //bottom center
                                drawRectHeightCenter
                                ? (new Point2D.Double(drawAreaX, drawAreaY + 0.5 * drawAreaHeight)) : null, //left center
                                drawRectWidthCenter
                                ? (new Point2D.Double(drawAreaX + drawAreaWidth,
                                                      drawAreaY + 0.5 * drawAreaHeight)) : null, //right center
                                //ǹβ
                                new Point2D.Double(0, 0),
                                drawCenter ? new Point2D.Double(halfWidth, 0) : null,
                                new Point2D.Double(this.iShapedWidth, 0),

                                new Point2D.Double(0, -this.iShapedHeight),
                                drawCenter ? new Point2D.Double(halfWidth, -this.iShapedHeight) : null,
                                new Point2D.Double(this.iShapedWidth, -this.iShapedHeight)} ;

      drawAnchorByPoints(g2, this.iShapedLeftX, this.iShapedBottomY, points) ;
    }

  }

  /**
   * <zh>ƶͼ,X᷽ƫtx,Y᷽ƫty,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Concatenates the current Graphics2D Transform with a translation transform.and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.
   * Subsequent rendering is translated by the specified
   * distance relative to the previous position.
   * This is equivalent to calling transform(T), where T is an
   * AffineTransform represented by the following matrix:
   * <pre>
   *		[   1    0    tx  ]
   *		[   0    1    ty  ]
   *		[   0    0    1   ]
   * </pre>.</en>
   *
   * @param tx double <zh>X᷽ľ.</zh>
   * <en>The distance to translate along the x-axis.</en>
   * @param ty double <zh>Y᷽ľ.</zh>
   * <en>The distance to translate along the y-axis.</en>
   */
  public void translate(double tx, double ty) {
    if (!(tx == 0 && ty == 0)) {
      Rectangle2D rect = getDrawArea() ;
      rect.setRect(rect.getX() + tx, rect.getY() + ty, rect.getWidth(), rect.getHeight()) ;

      this.iShapedLeftX += tx ;
      this.iShapedBottomY += ty ;

      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼϽǵ(ͼεת).</zh>
   * <en>Returns the top left corner coordinates of this shape (not consider rotation).</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getTopLeftPoint() {
    Rectangle2D rect = getDrawArea() ;
    double minX = Math.min(rect.getX(), this.iShapedLeftX) ;
    double minY = Math.min(rect.getY(), this.iShapedBottomY - this.iShapedHeight) ;

    return new Point2D.Double(minX, minY) ;
  }

  /**
   * <zh>ʱͼεĵ.</zh>
   * <en>Returns the lock coordinates of this shape.</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getLockPoint() {
    Rectangle2D drawArea = getDrawArea() ;
    if (drawArea == null) {
      return null ;
    }

    Rectangle2D cloneArea = new Rectangle2D.Double(drawArea.getX() - this.iShapedLeftX,
                                                   drawArea.getY() - this.iShapedBottomY,
                                                   drawArea.getWidth(), drawArea.getHeight()) ;

    AffineTransform transform = AffineTransform.getTranslateInstance(this.iShapedLeftX,
                                                                     this.iShapedBottomY) ;
    transform.rotate(getRotate()) ;
    Shape rotateShape = transform.createTransformedShape(cloneArea) ;
    Rectangle2D rotateBound = rotateShape.getBounds2D() ;

    return new Point2D.Double(rotateBound.getCenterX(), rotateBound.getCenterY()) ;
  }

  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  protected void calculateTransformedShape() {
    if (transformedShape == null) {
      AffineTransform transform = new AffineTransform() ;
      transform.translate(this.iShapedLeftX, this.iShapedBottomY) ;
      transform.rotate(getRotate()) ;

      float halfWidth = (float) (0.5 * this.iShapedWidth) ;
      float fWidth = (float)this.iShapedWidth ;
      float fHeight = (float)this.iShapedHeight ;
      final float expandWidth = 1.5F ;

      GeneralPath path = new GeneralPath() ;
      path.moveTo(0, -expandWidth) ;
      path.lineTo(halfWidth - expandWidth, -expandWidth) ;
      path.lineTo(halfWidth - expandWidth, -fHeight + expandWidth) ;
      path.lineTo(0, -fHeight + expandWidth) ;
      path.lineTo(0, -fHeight - expandWidth) ;
      path.lineTo(fWidth, -fHeight - expandWidth) ;
      path.lineTo(fWidth, -fHeight + expandWidth) ;
      path.lineTo(halfWidth + expandWidth, -fHeight + expandWidth) ;
      path.lineTo(halfWidth + expandWidth, -expandWidth) ;
      path.lineTo(fWidth, -expandWidth) ;
      path.lineTo(fWidth, expandWidth) ;
      path.lineTo(0, expandWidth) ;
      path.closePath() ;

      Rectangle2D drawArea = getDrawArea() ;
      Rectangle2D cloneArea = new Rectangle2D.Double(drawArea.getX() - this.iShapedLeftX,
                                                     drawArea.getY() - this.iShapedBottomY,
                                                     drawArea.getWidth(), drawArea.getHeight()) ;
      GeneralPath cloneAreaPath = new GeneralPath(cloneArea) ;
      path.append(cloneAreaPath, false) ;

      transformedShape = transform.createTransformedShape(path) ;
    }

  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    AffineTransform transform = new AffineTransform() ;
    transform.translate(this.iShapedLeftX, this.iShapedBottomY) ;
    transform.rotate(getRotate()) ;
    Rectangle2D drawArea = getDrawArea() ;
    Rectangle2D newArea = new Rectangle2D.Double(drawArea.getX() - this.iShapedLeftX,
                                                 drawArea.getY() - this.iShapedBottomY,
                                                 drawArea.getWidth(), drawArea.getHeight()) ;

    return transform.createTransformedShape(newArea) ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   *
   * @return ShapeType <zh>{@link ShapeType}ඨĳ (Ϊnull).</zh>
   * <en>The constant value in class {@link ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_STATISTICAL_BAR ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false
   *  otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    if (!(obj instanceof StatisticalBarChartShape)) {
      return false ;
    }

    final double theta = 0.00001 ;
    StatisticalBarChartShape that = (StatisticalBarChartShape) obj ;
    if (!EqualsUtil.equals(this.iShapedLeftX, that.iShapedLeftX, theta)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.iShapedBottomY, that.iShapedBottomY, theta)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.iShapedWidth, that.iShapedWidth, theta)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.iShapedHeight, that.iShapedHeight, theta)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double hashCode = super.hashCode() ;
    hashCode = 37 * hashCode + this.iShapedLeftX ;
    hashCode = 37 * hashCode + this.iShapedBottomY ;
    hashCode = 37 * hashCode + this.iShapedWidth ;
    hashCode = 37 * hashCode + this.iShapedHeight ;

    return (int) hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[iShapedLeftX=" + this.iShapedLeftX
      + ",iShapedBottomY=" + this.iShapedBottomY
      + ",iShapedWidth=" + this.iShapedWidth
      + ",iShapedHeight=" + this.iShapedHeight
      + "]" ;
  }

}
