package com.ijchart.xychart.util;

import java.awt.Graphics2D ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ڻͼѡʱêĹ.</zh>
 * <en>Util class for draw the anchor of selected ChartShape.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class AnchorUtil {

  /**
   * <zh>ƾͼѡʱê.</zh>
   * <en>Draw the anchor of selected rectangle shape.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param anchorWidth double <zh>êĿ.</zh>
   * <en>The anchor rectangle width.</en>
   */
  public static void drawAnchor(Graphics2D g2, Rectangle2D drawArea, double anchorWidth) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (drawArea == null) {
      throw new IllegalArgumentException("Null 'drawArea' argument.") ;
    }

    if (drawArea.getWidth() < anchorWidth && drawArea.getHeight() < anchorWidth) { //̫СͲê
      return ;
    }

    double rectWidth = anchorWidth ;
    final double halfRect = rectWidth / 2 ;
    //top left
    g2.fill(new Rectangle2D.Double( -halfRect, -halfRect, rectWidth, rectWidth)) ;
    //top right
    g2.fill(new Rectangle2D.Double(drawArea.getWidth() - halfRect, -halfRect,
                                   rectWidth, rectWidth)) ;

    //bottom left
    g2.fill(new Rectangle2D.Double( -halfRect, drawArea.getHeight() - halfRect,
                                   rectWidth, rectWidth)) ;
    //bottom right
    g2.fill(new Rectangle2D.Double(drawArea.getWidth() - halfRect,
                                   drawArea.getHeight() - halfRect, rectWidth, rectWidth)) ;

    if (drawArea.getWidth() > 3 * rectWidth) {
      //top center
      g2.fill(new Rectangle2D.Double((drawArea.getWidth() - rectWidth) / 2,
                                     -halfRect, rectWidth, rectWidth)) ;
      //bottom center
      g2.fill(new Rectangle2D.Double((drawArea.getWidth() - rectWidth) / 2,
                                     drawArea.getHeight() - halfRect, rectWidth, rectWidth)) ;
    }

    if (drawArea.getHeight() > 3 * rectWidth) {
      //center left
      g2.fill(new Rectangle2D.Double( -halfRect, (drawArea.getHeight() - rectWidth) / 2,
                                     rectWidth, rectWidth)) ;
      //center right
      g2.fill(new Rectangle2D.Double(drawArea.getWidth() - halfRect,
                                     (drawArea.getHeight() - rectWidth) / 2,
                                     rectWidth, rectWidth)) ;
    }
  }

  /**
   * <zh>ݵλ,êľ.</zh>
   * <en>Calculate the anchor rect by a point.</en>
   *
   * @param anchorCenterPoint Point2D <zh>êλ (Ϊnull).</zh>
   * <en>The center point of anchor rect (null not permitted).</en>
   * @param anchorWidth double <zh>êĿ.</zh>
   * <en>The anchor width.</en>
   *
   * @return Rectangle2D <zh>ê.</zh>
   * <en>The anchor rect.</en>
   */
  public static Rectangle2D getAnchorRect(Point2D anchorCenterPoint, double anchorWidth) {
    if (anchorCenterPoint == null) {
      throw new IllegalArgumentException("Null 'point' argument") ;
    }

    double halfWidth = 0.5 * anchorWidth ;
    return new Rectangle2D.Double(anchorCenterPoint.getX() - halfWidth,
                                  anchorCenterPoint.getY() - halfWidth,
                                  anchorWidth, anchorWidth) ;
  }
}
