package com.ijchart.xychart.util ;

import java.awt.GradientPaint ;
import java.awt.Image ;
import java.awt.Paint ;
import java.awt.Polygon ;
import java.awt.Shape ;
import java.awt.geom.Arc2D ;
import java.awt.geom.Ellipse2D ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Line2D ;
import java.awt.geom.PathIterator ;
import java.util.Arrays ;
import java.util.Iterator ;
import java.util.List ;
import java.util.Map ;
import java.util.Set ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ڱȽ϶Ĺ.</zh>
 * <en>Utilise class form object compare.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class EqualsUtil {

  /**
   * <zh>{@link java.awt.Paint}Ƿ.</zh>
   * <en>Returns true if the two Paint objects are equal
   * OR both null.This method handles GradientPaint as a special case.</en>
   *
   * @param p1  paint 1 (null permitted).
   * @param p2  paint 2 (null permitted).
   *
   * @return A boolean.
   */
  public static boolean equals(final Paint p1, final Paint p2) {
    // handle cases where either or both arguments are null
    if (p1 == null) {
      return (p2 == null) ;
    }
    if (p2 == null) {
      return false ;
    }

    boolean result = false ;
    // handle GradientPaint as a special case...
    if (p1 instanceof GradientPaint && p2 instanceof GradientPaint) {
      final GradientPaint gp1 = (GradientPaint) p1 ;
      final GradientPaint gp2 = (GradientPaint) p2 ;
      result = gp1.getColor1().equals(gp2.getColor1())
               && gp1.getColor2().equals(gp2.getColor2())
               && gp1.getPoint1().equals(gp2.getPoint1())
               && gp1.getPoint2().equals(gp2.getPoint2())
               && gp1.isCyclic() == gp2.isCyclic()
               && gp1.getTransparency() == gp1.getTransparency() ;
    } else {
      result = p1.equals(p2) ;
    }
    return result ;

  }

  /**
   * <zh>{@link Shape}Ƿ.</zh>
   * <en>Tests two shapes for equality.  If both shapes are null,
   * this method will return true.
   * <p>
   * In the current implementation, the following shapes are supported:
   * Ellipse2D, Line2D and Rectangle2D
   * (implicit).</en>
   *
   * @param s1 <zh>һShape (Ϊnull).</zh>
   * <en>The first shape (null permitted).</en>
   * @param s2 <zh>ڶShape (Ϊnull).</zh>
   * <en>The second shape (null permitted).</en>
   *
   * @return A boolean.
   */
  public static boolean equals(final Shape s1, final Shape s2) {
    if (s1 == null || s2 == null) {
      return false ;
    }

    if (s1 instanceof Line2D && s2 instanceof Line2D) {
      return equals((Line2D) s1, (Line2D) s2) ;
    } else if (s1 instanceof Ellipse2D && s2 instanceof Ellipse2D) {
      return equals((Ellipse2D) s1, (Ellipse2D) s2) ;
    } else if (s1 instanceof Arc2D && s2 instanceof Arc2D) {
      return equals((Arc2D) s1, (Arc2D) s2) ;
    } else if (s1 instanceof Polygon && s2 instanceof Polygon) {
      return equals((Polygon) s1, (Polygon) s2) ;
    } else if (s1 instanceof GeneralPath && s2 instanceof GeneralPath) {
      return equals((GeneralPath) s1, (GeneralPath) s2) ;
    } else {
      return s1.equals(s2) ;
    }
  }

  /**
   * <zh>{@link Line2D}Ƿ.</zh>
   * <en>Compares two lines are returns true if they are equal or
   * both null.</en>
   *
   * @param l1 <zh>һLine2D (Ϊnull).</zh>
   * <en>The first line (null permitted).</en>
   * @param l2 <zh>ڶLine2D (Ϊnull).</zh>
   * <en>The second line (null permitted).</en>
   *
   * @return A boolean.
   */
  public static boolean equals(final Line2D l1, final Line2D l2) {
    if (l1 == null) {
      return (l2 == null) ;
    }
    if (l2 == null) {
      return false ;
    }

    if (!l1.getP1().equals(l2.getP1())) {
      return false ;
    }
    if (!l1.getP2().equals(l2.getP2())) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>{@link Ellipse2D}Ƿ.</zh>
   * <en>Compares two ellipses and returns true if they are equal or
   * both null.</en>
   *
   * @param e1 <zh>һEllipse2D (Ϊnull).</zh>
   * <en>The first ellipse (null permitted).</en>
   * @param e2 <zh>ڶEllipse2D (Ϊnull).</zh>
   * <en>The second ellipse (null permitted).</en>
   *
   * @return A boolean.
   */
  public static boolean equals(final Ellipse2D e1, final Ellipse2D e2) {
    if (e1 == null) {
      return (e2 == null) ;
    }
    if (e2 == null) {
      return false ;
    }
    if (!e1.getFrame().equals(e2.getFrame())) {
      return false ;
    }
    return true ;
  }

  /**
   * <zh>{@link Arc2D}Ƿ.</zh>
   * <en>Compares two arcs and returns true if they are equal or
   * both null.</en>
   *
   * @param a1 <zh>һArc2D (Ϊnull).</zh>
   * <en>The first arc (null permitted).</en>
   * @param a2 <zh>ڶArc2D (Ϊnull).</zh>
   * <en>The second arc (null permitted).</en>
   *
   * @return A boolean.
   */
  public static boolean equals(final Arc2D a1, final Arc2D a2) {
    if (a1 == null) {
      return (a2 == null) ;
    }
    if (a2 == null) {
      return false ;
    }

    if (!a1.getFrame().equals(a2.getFrame())) {
      return false ;
    }
    if (a1.getAngleStart() != a2.getAngleStart()) {
      return false ;
    }
    if (a1.getAngleExtent() != a2.getAngleExtent()) {
      return false ;
    }
    if (a1.getArcType() != a2.getArcType()) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>{@link Polygon}Ƿ.</zh>
   * <en>Tests two polygons for equality.  If both are null this
   * method returns true.</en>
   *
   * @param p1 polygon <zh>һPolygon (Ϊnull).</zh>
   * <en> 1 (null permitted).</en>
   * @param p2  polygon <zh>ڶPolygon (Ϊnull).</zh>
   * <en> 2 (null permitted).</en>
   *
   * @return A boolean.
   */
  public static boolean equals(final Polygon p1, final Polygon p2) {
    if (p1 == null) {
      return (p2 == null) ;
    }
    if (p2 == null) {
      return false ;
    }
    if (p1.npoints != p2.npoints) {
      return false ;
    }

    if (!Arrays.equals(p1.xpoints, p2.xpoints)) {
      return false ;
    }
    if (!Arrays.equals(p1.ypoints, p2.ypoints)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>{@link GeneralPath}Ƿ.</zh>
   * <en>Tests two GeneralPath for equality.  If both are null this
   * method returns true.</en>
   *
   * @param p1  path <zh>һGeneralPath.</zh>
   * <en>1 (null permitted).</en>
   * @param p2  path <zh>ڶGeneralPath.</zh>
   * <en>2 (null permitted).</en>
   *
   * @return A boolean.
   */
  public static boolean equals(final GeneralPath p1, final GeneralPath p2) {
    if (p1 == null) {
      return (p2 == null) ;
    }
    if (p2 == null) {
      return false ;
    }

    if (p1.getWindingRule() != p2.getWindingRule()) {
      return false ;
    }
    PathIterator iterator1 = p1.getPathIterator(null) ;
    PathIterator iterator2 = p1.getPathIterator(null) ;
    double[] d1 = new double[6] ;
    double[] d2 = new double[6] ;
    boolean done = iterator1.isDone() && iterator2.isDone() ;
    while (!done) {
      if (iterator1.isDone() != iterator2.isDone()) {
        return false ;
      }
      int seg1 = iterator1.currentSegment(d1) ;
      int seg2 = iterator2.currentSegment(d2) ;
      if (seg1 != seg2) {
        return false ;
      }
      if (!Arrays.equals(d1, d2)) {
        return false ;
      }
      iterator1.next() ;
      iterator2.next() ;
      done = iterator1.isDone() && iterator2.isDone() ;
    }

    return true ;
  }

  /**
   * <zh>{@link java.awt.Image}Ƿ,˴ʵֽͼƬĿȺ͸߶,ֽڱȶ.</zh>
   * <en>Compares two image are returns true if they are equal or
   * both null.</en>
   *
   * @param image1 <zh>һImage (Ϊnull).</zh>
   * <en>The first image (null permitted).</en>
   * @param image2 <zh>ڶImage (Ϊnull).</zh>
   * <en>The second image (null permitted).</en>
   *
   * @return A boolean. <zh>ͼεĿȺ͸߶ȶ,true,򷵻false.</zh>
   * <en>Returns true if the two image width and height are equal,otherwise return false.</en>
   */
  public static boolean equals(final Image image1, final Image image2) {
    if (image1 == null) {
      return (image2 == null) ;
    }
    if (image2 == null) {
      return false ;
    }

    return image1.getWidth(null) == image2.getWidth(null)
      && image1.getHeight(null) == image2.getHeight(null) ;
  }

  /**
   * <zh>ȽǷ</zh>
   * <en>Compare two object whether or not equal.</en>
   * @param obj1 Object
   * @param obj2 Object
   * @return boolean <zh>ȷtrue,򷵻false.</zh>
   * <en>If the two object equals return true,else return false.</en>
   */
  public static boolean equals(Object obj1, Object obj2) {
    if (obj1 == obj2) {
      return true ;
    }

    if (obj1 instanceof Arc2D && obj2 instanceof Arc2D) {
      return equals((Arc2D) obj1, (Arc2D) obj2) ;
    } else if (obj1 instanceof Ellipse2D && obj2 instanceof Ellipse2D) {
      return equals((Ellipse2D) obj1, (Ellipse2D) obj2) ;
    } else if (obj1 instanceof GeneralPath && obj2 instanceof GeneralPath) {
      return equals((GeneralPath) obj1, (GeneralPath) obj2) ;
    } else if (obj1 instanceof Image && obj2 instanceof Image) {
      return equals((Image) obj1, (Image) obj2) ;
    } else if (obj1 instanceof Line2D && obj2 instanceof Line2D) {
      return equals((Line2D) obj1, (Line2D) obj2) ;
    } else if (obj1 instanceof Paint && obj2 instanceof Paint) {
      return equals((Paint) obj1, (Paint) obj2) ;
    } else if (obj1 instanceof Polygon && obj2 instanceof Polygon) {
      return equals((Polygon) obj1, (Polygon) obj2) ;
    } else if (obj1 instanceof Shape && obj2 instanceof Shape) {
      return equals((Shape) obj1, (Shape) obj2) ;
    }

    if (obj1 != null) {
      return obj1.equals(obj2) ;
    } else {
      return false ;
    }
  }

  /**
   * <zh>ȽǷ</zh>
   * <en>Compare that two doubles are equal concerning a delta.</en>
   *
   * @param num1 double
   * @param num2 double
   * @param delta double <zh>С.</zh>
   * <en>The delta.</en>
   *
   * @return boolean
   */
  public static boolean equals(double num1, double num2, double delta) {
    return Math.abs(num1 - num2) < delta ;
  }

  /**
   * <zh>Ƚ{@link List}Ƿ.</zh>
   * <en>Test if two {@link List} whether or not equal.</en>
   * @param list1 List
   * @param list2 List
   * @return boolean <zh>ȷtrue,򷵻false.</zh>
   * <en>If equal return true,else return false.</en>
   */
  public static boolean equals(List list1, List list2) {
    if (list1 == list2) {
      return true ;
    }
    if (list1 != null && list2 == null) {
      return false ;
    }
    if (list2 != null && list1 == null) {
      return false ;
    }
    if (list1.size() != list2.size()) {
      return false ;
    }

    for (int i = 0 ; i < list1.size() ; i++) {
      if (!equals(list1.get(i), list2.get(i))) {
        return false ;
      }
    }

    return true ;
  }

  /**
   * <zh>ȽǷ.</zh>
   * <en>Test if two object array whether or not equal.</en>
   * @param array1 Object[]
   * @param array2 Object[]
   * @return boolean <zh>ȷtrue,򷵻false.</zh>
   * <en>If equal return true,else return false.</en>
   */
  public static boolean equals(Object[] array1, Object[] array2) {
    if (array1 == array2) {
      return true ;
    }
    if (array1 != null && array2 == null) {
      return false ;
    }
    if (array1 != null && array2 == null) {
      return false ;
    }
    if (array1.length != array2.length) {
      return false ;
    }

    for (int i = 0 ; i < array1.length ; i++) {
      if (!equals(array1[i], array2[i])) {
        return false ;
      }
    }

    return true ;
  }

  /**
   * <zh>Mapkey,PaintǷ.</zh>
   * <en>Tests two maps containing (object, Paint)
   * elements for equality.</en>
   *
   * @param map1 <zh>һMap (Ϊnull).</zh>
   * <en>The first map (null permitted).</en>
   * @param map2 <zh>ڶMap (Ϊnull).</zh>
   * <en>The second map (null permitted).</en>
   *
   * @return A boolean.
   */
  public static boolean equalsPaintMap(Map map1, Map map2) {
    if (map1 == map2) {
      return true ;
    }
    if (map1 == null && map2 != null) {
      return false ;
    }
    if (map1 != null && map2 == null) {
      return false ;
    }
    if (map1.size() != map2.size()) {
      return false ;
    }

    Set keys = map1.keySet() ;
    Iterator iterator = keys.iterator() ;
    while (iterator.hasNext()) {
      Object key = iterator.next() ;
      Paint p1 = (Paint) map1.get(key) ;
      Paint p2 = (Paint) map2.get(key) ;

      if (!equals(p1, p2)) {
        return false ;
      }
    }

    return true ;
  }

}
